@@ -1,3 +1,4 @@
+/* SPDX-License-Identifier: LGPL-3.0-or-later */
 /*
  * vim:noexpandtab:shiftwidth=8:tabstop=8:
  *
@@ -49,13 +50,13 @@
 ATTR_MODE     | ATTR_OWNER	  | ATTR_GROUP	      |  \
 ATTR_ATIME    | ATTR_CTIME	  | ATTR_MTIME	      |  \
 ATTR_SIZE     | ATTR_MTIME_SERVER | ATTR_ATIME_SERVER |  \
-ATTR_ACL      | ATTR4_SEC_LABEL)
+ATTR_ACL      | ATTR4_SEC_LABEL   | ATTR4_XATTR)
 
 /**
  * Override internal Gluster defines for the time being.
  */
-/* Values for valid falgs to be used when using XXXsetattr, to set multiple
- a ttribute values passed via t*he related stat structure.
+/* Values for valid flags to be used when using XXXsetattr, to set multiple
+ * attribute values passed via the related stat structure.
  */
 #define GLAPI_SET_ATTR_MODE  GFAPI_SET_ATTR_MODE
 #define GLAPI_SET_ATTR_UID   GFAPI_SET_ATTR_UID
@@ -155,6 +156,10 @@ struct glusterfs_export {
 	char *sec_label_xattr;
 };
 
+#ifdef USE_GLUSTER_DELEGATION
+#define GLAPI_LEASE_ID_SIZE GLFS_LEASE_ID_SIZE
+#endif
+
 struct glusterfs_fd {
 	/** The open and share mode etc. This MUST be first in every
 	 *  file descriptor structure.
@@ -167,6 +172,9 @@ struct glusterfs_fd {
 	/** Gluster file descriptor. */
 	struct glfs_fd *glfd;
 	struct user_cred creds; /* user creds opening fd*/
+#ifdef USE_GLUSTER_DELEGATION
+	char lease_id[GLAPI_LEASE_ID_SIZE];
+#endif
 };
 
 struct glusterfs_handle {
@@ -183,6 +191,10 @@ struct glusterfs_handle {
 	uint64_t rw_issued;
 	uint64_t rw_serial;
 	uint64_t rw_max_len;
+#ifdef USE_GLUSTER_DELEGATION
+	glfs_lease_types_t lease_type; /* Store lease_type granted to
+					 this file: NONE,RD or RW */
+#endif
 };
 
 /* Structures defined for PNFS */
@@ -236,14 +248,117 @@ struct glusterfs_state_fd {
 
 void setglustercreds(struct glusterfs_export *glfs_export, uid_t *uid,
 		     gid_t *gid, unsigned int ngrps, gid_t *groups,
+		     char *client_addr, unsigned int client_addr_len,
 		     char *file, int line, char *function);
 
-#define SET_GLUSTER_CREDS(glfs_export, uid, gid, glen, garray) do {	\
-	int old_errno = errno;						\
-	((void) setglustercreds(glfs_export, uid, gid, glen,		\
-				garray, (char *) __FILE__,		\
-				__LINE__, (char *) __func__));		\
-	errno = old_errno;						\
+#ifdef USE_GLUSTER_DELEGATION
+#define SET_GLUSTER_CREDS_OP_CTX(glfs_export)				    \
+do {									    \
+	int old_errno = errno;						    \
+	void *sa = NULL;						    \
+	size_t sa_len = 0;						    \
+									    \
+	if (op_ctx->caller_addr != NULL) {				    \
+		sa = socket_addr(op_ctx->caller_addr);			    \
+		sa_len = socket_addr_len(op_ctx->caller_addr);		    \
+	} else if (op_ctx->client != NULL) {				    \
+		sa = socket_addr(&op_ctx->client->cl_addrbuf);		    \
+		sa_len = socket_addr_len(&op_ctx->client->cl_addrbuf);      \
+	}								    \
+									    \
+	((void) setglustercreds(glfs_export,				    \
+				&op_ctx->creds.caller_uid,		    \
+				&op_ctx->creds.caller_gid,		    \
+				op_ctx->creds.caller_glen,		    \
+				op_ctx->creds.caller_garray,		    \
+				sa, sa_len,				    \
+			       (char *) __FILE__,			    \
+			       __LINE__, (char *) __func__));		    \
+	errno = old_errno;						    \
+} while (0)
+
+#define SET_GLUSTER_CREDS_MY_FD(glfs_export, my_fd)			    \
+do {									    \
+	int old_errno = errno;						    \
+									    \
+	((void) setglustercreds(glfs_export,				    \
+				&(my_fd)->creds.caller_uid,		    \
+				&(my_fd)->creds.caller_gid,		    \
+				(my_fd)->creds.caller_glen,		    \
+				(my_fd)->creds.caller_garray,		    \
+				(my_fd)->lease_id, GLAPI_LEASE_ID_SIZE,	    \
+			       (char *) __FILE__,			    \
+			       __LINE__, (char *) __func__));		    \
+	errno = old_errno;						    \
+} while (0)
+
+/** @todo: This doesn't quite work right if op_ctx has no client, hopefully
+ *         in that case there won't actually ever be any leases...
+ */
+#define SET_GLUSTER_LEASE_ID(my_fd)					    \
+do {									    \
+	void *sa = NULL;						    \
+	size_t sa_len = 0;						    \
+									    \
+	if (op_ctx->caller_addr != NULL) {				    \
+		sa = socket_addr(op_ctx->caller_addr);			    \
+		sa_len = socket_addr_len(op_ctx->caller_addr);		    \
+	} else if (op_ctx->client != NULL) {				    \
+		sa = socket_addr(&op_ctx->client->cl_addrbuf);		    \
+		sa_len = socket_addr_len(&op_ctx->client->cl_addrbuf);      \
+	}								    \
+									    \
+	memset((my_fd)->lease_id, 0, GLAPI_LEASE_ID_SIZE);		    \
+									    \
+	if (sa_len != 0 && sa_len <= GLAPI_LEASE_ID_SIZE) {		    \
+		memcpy((my_fd)->lease_id, sa, sa_len);			    \
+	}								    \
+} while (0)
+
+#else
+#define SET_GLUSTER_CREDS_OP_CTX(glfs_export)				    \
+do {									    \
+	int old_errno = errno;						    \
+									    \
+	((void) setglustercreds(glfs_export,				    \
+				&op_ctx->creds.caller_uid,		    \
+				&op_ctx->creds.caller_gid,		    \
+				op_ctx->creds.caller_glen,		    \
+				op_ctx->creds.caller_garray,		    \
+				NULL, 0,				    \
+			       (char *) __FILE__,			    \
+			       __LINE__, (char *) __func__));		    \
+	errno = old_errno;						    \
+} while (0)
+
+#define SET_GLUSTER_CREDS_MY_FD(glfs_export, my_fd)			    \
+do {									    \
+	int old_errno = errno;						    \
+									    \
+	((void) setglustercreds(glfs_export,				    \
+				&(my_fd)->creds.caller_uid,		    \
+				&(my_fd)->creds.caller_gid,		    \
+				(my_fd)->creds.caller_glen,		    \
+				(my_fd)->creds.caller_garray,		    \
+				NULL, 0,				    \
+			       (char *) __FILE__,			    \
+			       __LINE__, (char *) __func__));		    \
+	errno = old_errno;						    \
+} while (0)
+
+#define SET_GLUSTER_LEASE_ID(my_fd)					    \
+do {									    \
+	/* NOTHING TO DO */						    \
+} while (0)
+#endif
+
+#define RESET_GLUSTER_CREDS(glfs_export) \
+do {									    \
+	int old_errno = errno;						    \
+	((void) setglustercreds(glfs_export, NULL, NULL, 0, NULL, NULL, 0,  \
+			       (char *) __FILE__,			    \
+			       __LINE__, (char *) __func__));		    \
+	errno = old_errno;						    \
 } while (0)
 
 #ifdef GLTIMING
@@ -260,7 +375,7 @@ void handle_ops_init(struct fsal_obj_ops *ops);
 fsal_status_t gluster2fsal_error(const int gluster_errorcode);
 
 void stat2fsal_attributes(const struct stat *buffstat,
-			  struct attrlist *fsalattr);
+			  struct fsal_attrlist *fsalattr);
 
 void construct_handle(struct glusterfs_export *glexport, const struct stat *st,
 		      struct glfs_object *glhandle, unsigned char *globjhdl,
@@ -270,7 +385,7 @@ fsal_status_t glfs2fsal_handle(struct glusterfs_export *glfs_export,
 			       struct glfs_object *glhandle,
 			       struct fsal_obj_handle **pub_handle,
 			       struct stat *sb,
-			       struct attrlist *attrs_out);
+			       struct fsal_attrlist *attrs_out);
 
 fsal_status_t glusterfs_create_export(struct fsal_module *fsal_hdl,
 				      void *parse_node,
@@ -279,13 +394,10 @@ fsal_status_t glusterfs_create_export(struct fsal_module *fsal_hdl,
 
 void gluster_cleanup_vars(struct glfs_object *glhandle);
 
-bool fs_specific_has(const char *fs_specific, const char *key, char *val,
-		     int *max_val_bytes);
-
 fsal_status_t glusterfs_get_acl(struct glusterfs_export *glfs_export,
 				 struct glfs_object *objhandle,
 				 glusterfs_fsal_xstat_t *buffxstat,
-				 struct attrlist *fsalattr);
+				 struct fsal_attrlist *fsalattr);
 
 fsal_status_t glusterfs_set_acl(struct glusterfs_export *glfs_export,
 				 struct glusterfs_handle *objhandle,
@@ -293,7 +405,7 @@ fsal_status_t glusterfs_set_acl(struct glusterfs_export *glfs_export,
 
 fsal_status_t glusterfs_process_acl(struct glfs *fs,
 				    struct glfs_object *object,
-				    struct attrlist *attrs,
+				    struct fsal_attrlist *attrs,
 				    glusterfs_fsal_xstat_t *buffxstat);
 
 void glusterfs_free_fs(struct glusterfs_fs *gl_fs);
@@ -322,8 +434,9 @@ nfsstat4 getdeviceinfo(struct fsal_module *fsal_hdl,
 /* UP thread routines */
 void *GLUSTERFSAL_UP_Thread(void *Arg);
 int initiate_up_thread(struct glusterfs_fs *gl_fs);
-int upcall_inode_invalidate(struct glusterfs_fs *gl_fs,
-			    struct glfs_object *object);
+int up_process_event_object(struct glusterfs_fs *gl_fs,
+			    struct glfs_object *object,
+			    enum glfs_upcall_reason reason);
 void gluster_process_upcall(struct glfs_upcall *cbk, void *data);
 
 fsal_status_t glusterfs_close_my_fd(struct glusterfs_fd *my_fd);
