@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: LGPL-3.0-or-later
 /*
  * vim:noexpandtab:shiftwidth=8:tabstop=8:
  *
@@ -39,7 +40,7 @@
 /* GLUSTERFS FSAL module private storage
  */
 
-const char glfsal_name[] = "GLUSTER";
+static const char glfsal_name[] = "GLUSTER";
 
 /**
  * Gluster global module object
