@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: LGPL-3.0-or-later
 /*
  * vim:noexpandtab:shiftwidth=8:tabstop=8:
  *
@@ -131,7 +132,6 @@ int glfs_get_ds_addr(struct glfs *fs, struct glfs_object *object,
  * Grants whole layout of the file requested.
  *
  * @param[in]     obj_pub  Public object handle
- * @param[in]     req_ctx  Request context
  * @param[out]    loc_body An XDR stream to which the FSAL must encode
  *                         the layout specific portion of the granted
  *                         layout segment.
@@ -142,14 +142,13 @@ int glfs_get_ds_addr(struct glfs *fs, struct glfs_object *object,
  */
 
 static nfsstat4 pnfs_layout_get(struct fsal_obj_handle          *obj_pub,
-				struct req_op_context           *req_ctx,
 				XDR                             *loc_body,
 				const struct fsal_layoutget_arg *arg,
 				struct fsal_layoutget_res       *res)
 {
 
 	struct glusterfs_export *export =
-		container_of(req_ctx->fsal_export,
+		container_of(op_ctx->fsal_export,
 			     struct glusterfs_export, export);
 
 	struct glusterfs_handle *handle =
@@ -228,7 +227,7 @@ static nfsstat4 pnfs_layout_get(struct fsal_obj_handle          *obj_pub,
 	ds_desc.addr     = &ds_wire;
 	ds_desc.len      = sizeof(struct glfs_ds_wire);
 	nfs_status = FSAL_encode_file_layout(loc_body, &deviceid, util, 0, 0,
-					     &req_ctx->ctx_export->export_id, 1,
+					     &op_ctx->ctx_export->export_id, 1,
 					     &ds_desc);
 	if (nfs_status) {
 		LogMajor(COMPONENT_PNFS,
@@ -253,7 +252,6 @@ out:
  * pins to release, always succeed
  *
  * @param[in] obj_pub  Public object handle
- * @param[in] req_ctx  Request context
  * @param[in] lrf_body Nothing for us
  * @param[in] arg      Input arguments of the function
  *
@@ -261,7 +259,6 @@ out:
  */
 
 static nfsstat4 pnfs_layout_return(struct fsal_obj_handle *obj_pub,
-				   struct req_op_context *req_ctx,
 				   XDR *lrf_body,
 				   const struct fsal_layoutreturn_arg *arg)
 {
@@ -281,7 +278,6 @@ static nfsstat4 pnfs_layout_return(struct fsal_obj_handle *obj_pub,
  * Update the size and time for a file accessed through a layout.
  *
  * @param[in]     obj_pub  Public object handle
- * @param[in]     req_ctx  Request context
  * @param[in]     lou_body An XDR stream containing the layout
  *                         type-specific portion of the LAYOUTCOMMIT
  *                         arguments.
@@ -292,7 +288,6 @@ static nfsstat4 pnfs_layout_return(struct fsal_obj_handle *obj_pub,
  */
 
 static nfsstat4 pnfs_layout_commit(struct fsal_obj_handle *obj_pub,
-				   struct req_op_context *req_ctx,
 				   XDR *lou_body,
 				   const struct fsal_layoutcommit_arg *arg,
 				   struct fsal_layoutcommit_res *res)
@@ -323,7 +318,7 @@ static nfsstat4 pnfs_layout_commit(struct fsal_obj_handle *obj_pub,
 
 	if (rc != 0) {
 		LogMajor(COMPONENT_PNFS,
-			 "Commit layout, stat unsucessfully completed");
+			 "Commit layout, stat unsuccessfully completed");
 		return NFS4ERR_INVAL;
 	}
 	memset(&new_stat, 0, sizeof(struct stat));
@@ -338,7 +333,7 @@ static nfsstat4 pnfs_layout_commit(struct fsal_obj_handle *obj_pub,
 					     res->new_size);
 			if (rc != 0) {
 				LogMajor(COMPONENT_PNFS,
-					 "Commit layout, size changed unsucessfully completed");
+					 "Commit layout, size change unsuccessfully completed");
 				return NFS4ERR_INVAL;
 			}
 		}
@@ -353,21 +348,18 @@ static nfsstat4 pnfs_layout_commit(struct fsal_obj_handle *obj_pub,
 
 	mask |= GLAPI_SET_ATTR_MTIME;
 
-	SET_GLUSTER_CREDS(glfs_export, &op_ctx->creds->caller_uid,
-			  &op_ctx->creds->caller_gid,
-			  op_ctx->creds->caller_glen,
-			  op_ctx->creds->caller_garray);
+	SET_GLUSTER_CREDS_OP_CTX(glfs_export);
 
 	rc = glfs_h_setattrs(glfs_export->gl_fs->fs,
 			     objhandle->glhandle,
 			     &new_stat,
 			     mask);
 
-	SET_GLUSTER_CREDS(glfs_export, NULL, NULL, 0, NULL);
+	RESET_GLUSTER_CREDS(glfs_export);
 
 	if ((rc != 0) || (status.major != ERR_FSAL_NO_ERROR)) {
 		LogMajor(COMPONENT_PNFS,
-			 "commit layout, setattr unsucessflly completed");
+			 "commit layout, setattr unsuccessfully completed");
 		return NFS4ERR_INVAL;
 	}
 	res->commit_done = true;
@@ -560,7 +552,7 @@ select_ds(struct glfs_object *object, char *pathinfo, char *hostname,
 	  size_t size)
 {
 	/* Represents starting of each server in the list*/
-	const char posix[10]    = "POSIX";
+	static const char posix[10]    = "POSIX";
 	/* Array of pathinfo of available dses */
 	char	*ds_path_info[MAX_DS_COUNT];
 	/* Key for hashing */
@@ -585,7 +577,7 @@ select_ds(struct glfs_object *object, char *pathinfo, char *hostname,
 		tmp++;
 		no_of_ds++;
 		/* *
-		 * If no of dses reaches maxmium count, then
+		 * If no of dses reaches maximum count, then
 		 * perform load balance on current list
 		 */
 		if (no_of_ds == MAX_DS_COUNT)
@@ -645,7 +637,6 @@ glfs_get_ds_addr(struct glfs *fs, struct glfs_object *object, uint32_t *ds_addr)
 	int             ret                    = 0;
 	char            pathinfo[1024]         = {0, };
 	char            hostname[256]          = {0, };
-	char		scratch[SOCK_NAME_MAX] = {0, };
 	struct addrinfo hints                  = {0, };
 	struct addrinfo *res                   = NULL;
 	const char      *pathinfokey           = "trusted.glusterfs.pathinfo";
@@ -670,8 +661,16 @@ glfs_get_ds_addr(struct glfs *fs, struct glfs_object *object, uint32_t *ds_addr)
 		LogMajor(COMPONENT_PNFS, "error %s\n", gai_strerror(ret));
 		goto out;
 	}
-	sprint_sockip((sockaddr_t *)res->ai_addr, scratch, sizeof(scratch));
-	LogDebug(COMPONENT_PNFS, "ip address : %s", scratch);
+
+	if (isDebug(COMPONENT_PNFS)) {
+		char scratch[SOCK_NAME_MAX];
+		struct display_buffer dspbuf = {
+					sizeof(scratch), scratch, scratch};
+
+		display_sockip(&dspbuf, (sockaddr_t *)res->ai_addr);
+		LogDebug(COMPONENT_PNFS, "ip address : %s", scratch);
+	}
+
 	*ds_addr = ((struct sockaddr_in *)(res->ai_addr))->sin_addr.s_addr;
 out:
 	freeaddrinfo(res);
