@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: LGPL-3.0-or-later
 /*
  * Copyright © 2012 CohortFS, LLC.
  * Author: Adam C. Emerson <aemerson@linuxbox.com>
@@ -38,6 +39,7 @@
 #include <fcntl.h>
 #include "fsal_api.h"
 #include "FSAL/fsal_commonlib.h"
+#include "FSAL/fsal_localfs.h"
 #include "../fsal_private.h"
 #include "fsal_convert.h"
 #include "fsal_internal.h"
@@ -55,12 +57,11 @@
  *
  * @param[in] ds_pub The object to release
  */
-static void ds_release(struct fsal_ds_handle *const ds_pub)
+static void ds_handle_release(struct fsal_ds_handle *const ds_pub)
 {
 	/* The private 'full' DS handle */
 	struct gpfs_ds *ds = container_of(ds_pub, struct gpfs_ds, ds);
 
-	fsal_ds_handle_fini(&ds->ds);
 	gsh_free(ds);
 }
 
@@ -69,11 +70,10 @@ static void ds_release(struct fsal_ds_handle *const ds_pub)
  *
  * NFSv4.1 data server handles are disjount from normal
  * filehandles (in Ganesha, there is a ds_flag in the filehandle_v4_t
- * structure) and do not get loaded into cache_inode or processed the
+ * structure) and do not get loaded into mdcache or processed the
  * normal way.
  *
  * @param[in]  ds_pub           FSAL DS handle
- * @param[in]  req_ctx          Credentials
  * @param[in]  stateid          The stateid supplied with the READ operation,
  *                              for validation
  * @param[in]  offset           The offset at which to read
@@ -85,7 +85,6 @@ static void ds_release(struct fsal_ds_handle *const ds_pub)
  * @return An NFSv4.1 status code.
  */
 static nfsstat4 ds_read(struct fsal_ds_handle *const ds_pub,
-			struct req_op_context *const req_ctx,
 			const stateid4 *stateid, const offset4 offset,
 			const count4 requested_length, void *const buffer,
 			count4 * const supplied_length,
@@ -111,6 +110,7 @@ static nfsstat4 ds_read(struct fsal_ds_handle *const ds_pub,
 	rarg.offset = offset;
 	rarg.length = requested_length;
 	rarg.options = 0;
+	rarg.cli_ip = NULL;
 	if (op_ctx && op_ctx->client)
 		rarg.cli_ip = op_ctx->client->hostaddr_str;
 
@@ -141,11 +141,10 @@ static nfsstat4 ds_read(struct fsal_ds_handle *const ds_pub,
  *
  * NFSv4.2 data server handles are disjount from normal
  * filehandles (in Ganesha, there is a ds_flag in the filehandle_v4_t
- * structure) and do not get loaded into cache_inode or processed the
+ * structure) and do not get loaded into mdcache or processed the
  * normal way.
  *
  * @param[in]  ds_pub           FSAL DS handle
- * @param[in]  req_ctx          Credentials
  * @param[in]  stateid          The stateid supplied with the READ operation,
  *                              for validation
  * @param[in]  offset           The offset at which to read
@@ -158,7 +157,6 @@ static nfsstat4 ds_read(struct fsal_ds_handle *const ds_pub,
  * @return An NFSv4.2 status code.
  */
 static nfsstat4 ds_read_plus(struct fsal_ds_handle *const ds_pub,
-			struct req_op_context *const req_ctx,
 			const stateid4 *stateid, const offset4 offset,
 			const count4 requested_length, void *const buffer,
 			const count4 supplied_length,
@@ -187,6 +185,7 @@ static nfsstat4 ds_read_plus(struct fsal_ds_handle *const ds_pub,
 	rarg.length = requested_length;
 	rarg.filesize = &filesize;
 	rarg.options = IO_SKIP_HOLE;
+	rarg.cli_ip = NULL;
 	if (op_ctx && op_ctx->client)
 		rarg.cli_ip = op_ctx->client->hostaddr_str;
 
@@ -239,7 +238,6 @@ static nfsstat4 ds_read_plus(struct fsal_ds_handle *const ds_pub,
  * and performs an MDS write.
  *
  * @param[in]  ds_pub           FSAL DS handle
- * @param[in]  req_ctx          Credentials
  * @param[in]  stateid          The stateid supplied with the READ operation,
  *                              for validation
  * @param[in]  offset           The offset at which to read
@@ -254,7 +252,6 @@ static nfsstat4 ds_read_plus(struct fsal_ds_handle *const ds_pub,
  * @return An NFSv4.1 status code.
  */
 static nfsstat4 ds_write(struct fsal_ds_handle *const ds_pub,
-			 struct req_op_context *const req_ctx,
 			 const stateid4 *stateid, const offset4 offset,
 			 const count4 write_length, const void *buffer,
 			 const stable_how4 stability_wanted,
@@ -288,6 +285,7 @@ static nfsstat4 ds_write(struct fsal_ds_handle *const ds_pub,
 	warg.stability_got = stability_got;
 	warg.verifier4 = (int32_t *) writeverf;
 	warg.options = 0;
+	warg.cli_ip = NULL;
 	if (op_ctx && op_ctx->client)
 		warg.cli_ip = op_ctx->client->hostaddr_str;
 
@@ -310,8 +308,8 @@ static nfsstat4 ds_write(struct fsal_ds_handle *const ds_pub,
 
 	key.addr = gpfs_handle;
 	key.len = gpfs_handle->handle_key_size;
-	req_ctx->fsal_export->up_ops->invalidate(
-			req_ctx->fsal_export->up_ops, &key,
+	op_ctx->fsal_export->up_ops->invalidate(
+			op_ctx->fsal_export->up_ops, &key,
 			FSAL_UP_INVALIDATE_CACHE);
 
 	*written_length = amount_written;
@@ -324,11 +322,10 @@ static nfsstat4 ds_write(struct fsal_ds_handle *const ds_pub,
  *
  * NFSv4.1 data server filehandles are disjount from normal
  * filehandles (in Ganesha, there is a ds_flag in the filehandle_v4_t
- * structure) and do not get loaded into cache_inode or processed the
+ * structure) and do not get loaded into mdcache or processed the
  * normal way.
  *
  * @param[in]  ds_pub    FSAL DS handle
- * @param[in]  req_ctx   Credentials
  * @param[in]  offset    Start of commit window
  * @param[in]  count     Length of commit window
  * @param[out] writeverf Write verifier
@@ -336,7 +333,6 @@ static nfsstat4 ds_write(struct fsal_ds_handle *const ds_pub,
  * @return An NFSv4.1 status code.
  */
 static nfsstat4 ds_commit(struct fsal_ds_handle *const ds_pub,
-			  struct req_op_context *const req_ctx,
 			  const offset4 offset, const count4 count,
 			  verifier4 * const writeverf)
 {
@@ -347,18 +343,6 @@ static nfsstat4 ds_commit(struct fsal_ds_handle *const ds_pub,
 	return NFS4ERR_INVAL;
 }
 
-static void dsh_ops_init(struct fsal_dsh_ops *ops)
-{
-	/* redundant copy, but you never know about the future... */
-	memcpy(ops, &def_dsh_ops, sizeof(struct fsal_dsh_ops));
-
-	ops->release = ds_release;
-	ops->read = ds_read;
-	ops->read_plus = ds_read_plus;
-	ops->write = ds_write;
-	ops->commit = ds_commit;
-}
-
 /**
  * @brief Try to create a FSAL data server handle from a wire handle
  *
@@ -431,7 +415,6 @@ static nfsstat4 make_ds_handle(struct fsal_pnfs_ds *const pds,
 	ds = gsh_calloc(1, sizeof(struct gpfs_ds));
 
 	*handle = &ds->ds;
-	fsal_ds_handle_init(*handle, pds);
 
 	/* Connect lazily when a FILE_SYNC4 write forces us to, not
 	   here. */
@@ -457,7 +440,11 @@ static nfsstat4 pds_permissions(struct fsal_pnfs_ds *const pds,
 void pnfs_ds_ops_init(struct fsal_pnfs_ds_ops *ops)
 {
 	memcpy(ops, &def_pnfs_ds_ops, sizeof(struct fsal_pnfs_ds_ops));
-	ops->permissions = pds_permissions;
+	ops->ds_permissions = pds_permissions;
 	ops->make_ds_handle = make_ds_handle;
-	ops->fsal_dsh_ops = dsh_ops_init;
+	ops->dsh_release = ds_handle_release;
+	ops->dsh_read = ds_read;
+	ops->dsh_read_plus = ds_read_plus;
+	ops->dsh_write = ds_write;
+	ops->dsh_commit = ds_commit;
 }
