@@ -1,7 +1,8 @@
+// SPDX-License-Identifier: LGPL-3.0-or-later
 /*
  * @file  fsal_internal.c
  * @date  $Date: 2006/01/17 14:20:07 $
- * @brief Defines the datas that are to be accessed as extern by the fsal
+ * @brief Defines the data that are to be accessed as extern by the fsal
  *        modules
  *
  * vim:noexpandtab:shiftwidth=8:tabstop=8:
@@ -56,7 +57,7 @@
 
 /*********************************************************************
  *
- *  GPFS FSAL char device driver interaces
+ *  GPFS FSAL char device driver interfaces
  *
  ********************************************************************/
 
@@ -78,6 +79,10 @@ fsal_status_t fsal_internal_close(int fd, void *owner, int cflags)
 	carg.close_fd = fd;
 	carg.close_flags = cflags;
 	carg.close_owner = owner;
+	carg.cli_ip = NULL;
+
+	if (op_ctx && op_ctx->client)
+		carg.cli_ip = op_ctx->client->hostaddr_str;
 
 	if (unlikely(gpfs_ganesha(OPENHANDLE_CLOSE_FILE, &carg) < 0))
 		return FSAL_INTERNAL_ERROR(errno, "OPENHANDLE_CLOSE_FILE");
@@ -90,7 +95,7 @@ fsal_status_t fsal_internal_close(int fd, void *owner, int cflags)
  *
  *  @param dirfd Open file descriptor of parent directory
  *  @param gpfs_fh Opaque filehandle
- *  @param fd File descriptor openned by the function
+ *  @param fd File descriptor opened by the function
  *  @param oflags Flags to open the file with
  *
  *  @return status of operation
@@ -102,12 +107,12 @@ fsal_internal_handle2fd(int dirfd, struct gpfs_file_handle *gpfs_fh,
 	struct open_arg oarg = {0};
 	int rc;
 
-	if (op_ctx && op_ctx->client)
-		oarg.cli_ip = op_ctx->client->hostaddr_str;
-
 	oarg.mountdirfd = dirfd;
 	oarg.handle = gpfs_fh;
 	oarg.flags = oflags;
+	oarg.cli_ip = NULL;
+	if (op_ctx && op_ctx->client)
+		oarg.cli_ip = op_ctx->client->hostaddr_str;
 
 	rc = gpfs_ganesha(OPENHANDLE_OPEN_BY_HANDLE, &oarg);
 	if (unlikely(rc < 0))
@@ -248,6 +253,9 @@ fsal_internal_link_fh(int dirfd, struct gpfs_file_handle *gpfs_fh_tgt,
 	linkarg.name = link_name;
 	linkarg.dir_fh = gpfs_fh;
 	linkarg.dst_fh = gpfs_fh_tgt;
+	linkarg.cli_ip = NULL;
+	if (op_ctx && op_ctx->client)
+		linkarg.cli_ip = op_ctx->client->hostaddr_str;
 
 	if (unlikely(gpfs_ganesha(OPENHANDLE_LINK_BY_FH, &linkarg) < 0))
 		return FSAL_INTERNAL_ERROR(errno, "OPENHANDLE_LINK_BY_FH");
@@ -280,6 +288,9 @@ fsal_internal_stat_name(int dirfd, struct gpfs_file_handle *gpfs_fh,
 	statarg.name = stat_name;
 	statarg.handle = gpfs_fh;
 	statarg.buf = buf;
+	statarg.cli_ip = NULL;
+	if (op_ctx && op_ctx->client)
+		statarg.cli_ip = op_ctx->client->hostaddr_str;
 
 	if (unlikely(gpfs_ganesha(OPENHANDLE_STAT_BY_NAME, &statarg) < 0))
 		return FSAL_INTERNAL_ERROR(errno, "OPENHANDLE_STAT_BY_NAME");
@@ -312,8 +323,11 @@ fsal_internal_unlink(int dirfd, struct gpfs_file_handle *gpfs_fh,
 	statarg.name = stat_name;
 	statarg.handle = gpfs_fh;
 	statarg.buf = buf;
+	statarg.cli_ip = NULL;
+	if (op_ctx && op_ctx->client)
+		statarg.cli_ip = op_ctx->client->hostaddr_str;
 
-	fsal_set_credentials(op_ctx->creds);
+	fsal_set_credentials(&op_ctx->creds);
 
 	rc = gpfs_ganesha(OPENHANDLE_UNLINK_BY_NAME, &statarg);
 	errsv = errno;
@@ -363,6 +377,9 @@ fsal_internal_create(struct fsal_obj_handle *dir_hdl, const char *stat_name,
 	crarg.buf = buf;
 	crarg.dir_fh = container_of(dir_hdl, struct gpfs_fsal_obj_handle,
 				    obj_handle)->handle;
+	crarg.cli_ip = NULL;
+	if (op_ctx && op_ctx->client)
+		crarg.cli_ip = op_ctx->client->hostaddr_str;
 
 	if (unlikely(gpfs_ganesha(OPENHANDLE_CREATE_BY_NAME, &crarg) < 0))
 		return FSAL_INTERNAL_ERROR(errno, "OPENHANDLE_CREATE_BY_NAME");
@@ -395,6 +412,9 @@ fsal_internal_mknode(struct fsal_obj_handle *dir_hdl, const char *stat_name,
 	crarg.buf = buf;
 	crarg.dir_fh = container_of(dir_hdl, struct gpfs_fsal_obj_handle,
 				    obj_handle)->handle;
+	crarg.cli_ip = NULL;
+	if (op_ctx && op_ctx->client)
+		crarg.cli_ip = op_ctx->client->hostaddr_str;
 
 	if (unlikely(gpfs_ganesha(OPENHANDLE_MKNODE_BY_NAME, &crarg) < 0))
 		return FSAL_INTERNAL_ERROR(errno, "OPENHANDLE_MKNODE_BY_NAME");
@@ -431,8 +451,11 @@ fsal_internal_rename_fh(int dirfd, struct gpfs_file_handle *gpfs_fh_old,
 	renamearg.new_name = new_name;
 	renamearg.old_fh = gpfs_fh_old;
 	renamearg.new_fh = gpfs_fh_new;
+	renamearg.cli_ip = NULL;
+	if (op_ctx && op_ctx->client)
+		renamearg.cli_ip = op_ctx->client->hostaddr_str;
 
-	fsal_set_credentials(op_ctx->creds);
+	fsal_set_credentials(&op_ctx->creds);
 
 	rc = gpfs_ganesha(OPENHANDLE_RENAME_BY_FH, &renamearg);
 	errsv = errno;
@@ -486,12 +509,21 @@ int fsal_internal_version(void)
 {
 	int rc;
 
-	if (unlikely(gpfs_ganesha(OPENHANDLE_GET_VERSION2, &rc) < 0))
-		FSAL_INTERNAL_ERROR(errno, "OPENHANDLE_GET_VERSION2");
-	else
-		LogDebug(COMPONENT_FSAL, "GPFS get version %d", rc);
+	/* Try VERSION4 first, followed by VERSION3,2 */
+	rc = gpfs_ganesha(OPENHANDLE_GET_VERSION4, NULL);
+	if (rc != -1)
+		return 0;
 
-	return rc;
+	rc = gpfs_ganesha(OPENHANDLE_GET_VERSION3, NULL);
+	if (rc != -1)
+		return 0;
+
+	rc = gpfs_ganesha(OPENHANDLE_GET_VERSION2, NULL);
+	if (rc != -1)
+		return 0;
+
+	LogMajor(COMPONENT_FSAL, "OPENHANDLE_GET_VERSION failed: %d", errno);
+	return errno;
 }
 
 /**
@@ -541,6 +573,9 @@ fsal_get_xstat_by_handle(int dirfd, struct gpfs_file_handle *gpfs_fh,
 	xstatarg.fsid = (struct fsal_fsid *)&buffxstat->fsal_fsid;
 	xstatarg.attr_valid |= XATTR_FSID;
 	xstatarg.expire_attr = expire_time_attr;
+	xstatarg.cli_ip = NULL;
+	if (op_ctx && op_ctx->client)
+		xstatarg.cli_ip = op_ctx->client->hostaddr_str;
 
 	rc = gpfs_ganesha(OPENHANDLE_GET_XSTAT, &xstatarg);
 	errsv = errno;
@@ -572,7 +607,7 @@ fsal_get_xstat_by_handle(int dirfd, struct gpfs_file_handle *gpfs_fh,
 					"GET_XSTAT returned buffer too small, passed len: %u, required len: %u, ",
 					acl_buflen, acl_buf->acl_len);
 				errno = 0;
-				break;
+				return fsalstat(ERR_FSAL_NO_ERROR, 0);
 			}
 
 			LogWarn(COMPONENT_FSAL,
@@ -601,9 +636,9 @@ fsal_get_xstat_by_handle(int dirfd, struct gpfs_file_handle *gpfs_fh,
 			return fsalstat(ERR_FSAL_SERVERFAULT, 0);
 		}
 		buffxstat->attr_valid = XATTR_FSID | XATTR_STAT | XATTR_ACL;
-	}
-	else
+	} else {
 		buffxstat->attr_valid = XATTR_FSID | XATTR_STAT;
+	}
 
 	return fsalstat(ERR_FSAL_NO_ERROR, 0);
 }
@@ -612,7 +647,6 @@ fsal_get_xstat_by_handle(int dirfd, struct gpfs_file_handle *gpfs_fh,
  *  @brief Set NFS4 ACL as well as stat.
  *
  *  @param dirfd Open file descriptor of parent directory
- *  @param op_ctx Context
  *  @param gpfs_fh file handle of file
  *  @param attr_valid Attributes valid
  *  @param attr_changed Attributes changed
@@ -621,7 +655,7 @@ fsal_get_xstat_by_handle(int dirfd, struct gpfs_file_handle *gpfs_fh,
  *  @return status of operation
  */
 fsal_status_t
-fsal_set_xstat_by_handle(int dirfd, const struct req_op_context *op_ctx,
+fsal_set_xstat_by_handle(int dirfd,
 			 struct gpfs_file_handle *gpfs_fh, int attr_valid,
 			 int attr_changed, gpfsfsal_xstat_t *buffxstat,
 			 gpfs_acl_t *acl_buf)
@@ -639,13 +673,16 @@ fsal_set_xstat_by_handle(int dirfd, const struct req_op_context *op_ctx,
 	xstatarg.acl = acl_buf;
 	xstatarg.attr_changed = attr_changed;
 	xstatarg.buf = &buffxstat->buffstat;
+	xstatarg.cli_ip = NULL;
+	if (op_ctx && op_ctx->client)
+		xstatarg.cli_ip = op_ctx->client->hostaddr_str;
 
 	/* We explicitly do NOT do setfsuid/setfsgid here because truncate,
 	   even to enlarge a file, doesn't actually allocate blocks. GPFS
 	   implements sparse files, so blocks of all 0 will not actually
 	   be allocated.
 	 */
-	fsal_set_credentials(op_ctx->creds);
+	fsal_set_credentials(&op_ctx->creds);
 
 	rc = gpfs_ganesha(OPENHANDLE_SET_XSTAT, &xstatarg);
 	errsv = errno;
@@ -666,14 +703,13 @@ fsal_set_xstat_by_handle(int dirfd, const struct req_op_context *op_ctx,
 
 /**
  *  @param dirfd Open file descriptor of parent directory
- *  @param op_ctx Context
  *  @param gpfs_fh file handle of file
- *  @param size Size
+ *  @param size The size to truncate the file to
  *
  *  @return status of operation
  */
 fsal_status_t
-fsal_trucate_by_handle(int dirfd, const struct req_op_context *op_ctx,
+fsal_trucate_by_handle(int dirfd,
 		       struct gpfs_file_handle *gpfs_fh, u_int64_t size)
 {
 	gpfsfsal_xstat_t buffxstat = {0};
@@ -683,7 +719,7 @@ fsal_trucate_by_handle(int dirfd, const struct req_op_context *op_ctx,
 
 	buffxstat.buffstat.st_size = size;
 
-	return fsal_set_xstat_by_handle(dirfd, op_ctx, gpfs_fh, XATTR_STAT,
+	return fsal_set_xstat_by_handle(dirfd, gpfs_fh, XATTR_STAT,
 					XATTR_SIZE, &buffxstat, NULL);
 }
 
