@@ -1,3 +1,4 @@
+/* SPDX-License-Identifier: LGPL-3.0-or-later */
 /*
  * vim:noexpandtab:shiftwidth=8:tabstop=8:
  *
@@ -93,7 +94,7 @@ struct gpfs_state_fd {
 /* defined the set of attributes supported with POSIX */
 #define GPFS_SUPPORTED_ATTRIBUTES ((const attrmask_t) (         \
 		ATTRS_POSIX | ATTR_ACL | ATTR4_SPACE_RESERVED | \
-		ATTR4_FS_LOCATIONS | ATTR4_XATTR))
+		ATTR4_FS_LOCATIONS))
 
 #define GPFS_MAX_FH_SIZE OPENHANDLE_HANDLE_LEN
 
@@ -136,7 +137,8 @@ fsal_status_t fsal_internal_get_handle_at(int dfd,
 				int expfd);
 
 fsal_status_t gpfsfsal_xstat_2_fsal_attributes(gpfsfsal_xstat_t *gpfs_buf,
-		struct attrlist *fsal_attr, gpfs_acl_t *acl_buf, bool use_acl);
+		struct fsal_attrlist *fsal_attr, gpfs_acl_t *acl_buf,
+		bool use_acl);
 
 fsal_status_t
 fsal_internal_handle2fd(int dirfd, struct gpfs_file_handle *phandle,
@@ -204,14 +206,12 @@ fsal_status_t fsal_get_xstat_by_handle(int dirfd,
 				       bool expire, bool use_acl);
 
 fsal_status_t fsal_set_xstat_by_handle(int dirfd,
-				       const struct req_op_context *p_context,
 				       struct gpfs_file_handle *p_handle,
 				       int attr_valid, int attr_changed,
 				       gpfsfsal_xstat_t *p_buffxstat,
 				       gpfs_acl_t *acl_buf);
 
 fsal_status_t fsal_trucate_by_handle(int dirfd,
-				     const struct req_op_context *p_context,
 				     struct gpfs_file_handle *p_handle,
 				     u_int64_t size);
 
@@ -219,62 +219,53 @@ fsal_status_t fsal_trucate_by_handle(int dirfd,
 
 fsal_status_t GPFSFSAL_getattrs(struct fsal_export *export,
 				struct gpfs_filesystem *gpfs_fs,
-				const struct req_op_context *p_context,
 				struct gpfs_file_handle *p_filehandle,
-				struct attrlist *p_object_attributes);
+				struct fsal_attrlist *p_object_attributes);
 
 fsal_status_t GPFSFSAL_fs_loc(struct fsal_export *export,
 			      struct gpfs_filesystem *gpfs_fs,
-			      const struct req_op_context *p_context,
 			      struct gpfs_file_handle *p_filehandle,
-			      struct attrlist *attrs);
+			      struct fsal_attrlist *attrs);
 
 fsal_status_t GPFSFSAL_statfs(int fd,
 			      struct fsal_obj_handle *obj_hdl,
 			      struct statfs *buf);
 
 fsal_status_t GPFSFSAL_setattrs(struct fsal_obj_handle *dir_hdl,
-				const struct req_op_context *p_context,
-				struct attrlist *p_object_attributes);
+				struct fsal_attrlist *p_object_attributes);
 
 fsal_status_t GPFSFSAL_create(struct fsal_obj_handle *dir_hdl,
 			      const char *p_filename,
-			      const struct req_op_context *p_context,
 			      uint32_t accessmode,
 			      struct gpfs_file_handle *p_object_handle,
-			      struct attrlist *p_object_attributes);
+			      struct fsal_attrlist *p_object_attributes);
 
 fsal_status_t GPFSFSAL_create2(struct fsal_obj_handle *dir_hdl,
 			      const char *p_filename,
-			      const struct req_op_context *p_context,
 			      mode_t unix_mode,
 			      struct gpfs_file_handle *p_object_handle,
 			      int posix_flags,
-			      struct attrlist *p_object_attributes);
+			      struct fsal_attrlist *p_object_attributes);
 
 fsal_status_t GPFSFSAL_mkdir(struct fsal_obj_handle *dir_hdl,
 			     const char *p_dirname,
-			     const struct req_op_context *p_context,
 			     uint32_t accessmode,
 			     struct gpfs_file_handle *p_object_handle,
-			     struct attrlist *p_object_attributes);
+			     struct fsal_attrlist *p_object_attributes);
 
 fsal_status_t GPFSFSAL_link(struct fsal_obj_handle *dir_hdl,
 			    struct gpfs_file_handle *p_target_handle,
-			    const char *p_link_name,
-			    const struct req_op_context *p_context);
+			    const char *p_link_name);
 
 fsal_status_t GPFSFSAL_mknode(struct fsal_obj_handle *dir_hdl,
 			      const char *p_node_name,
-			      const struct req_op_context *p_context,
 			      uint32_t accessmode,
 			      mode_t nodetype,
 			      fsal_dev_t *dev,
 			      struct gpfs_file_handle *p_object_handle,
-			      struct attrlist *node_attributes);
+			      struct fsal_attrlist *node_attributes);
 
 fsal_status_t GPFSFSAL_open(struct fsal_obj_handle *obj_hdl,
-			    const struct req_op_context *p_context,
 			    int posix_flags, int *p_file_descriptor);
 
 fsal_status_t GPFSFSAL_read(int fd,
@@ -291,7 +282,6 @@ fsal_status_t GPFSFSAL_write(int fd,
 			     void *buffer,
 			     size_t *p_write_amount,
 			     bool *fsal_stable,
-			     const struct req_op_context *p_context,
 			     int expfs);
 
 fsal_status_t GPFSFSAL_alloc(int fd,
@@ -299,10 +289,9 @@ fsal_status_t GPFSFSAL_alloc(int fd,
 			     uint64_t length,
 			     bool options);
 
-fsal_status_t GPFSFSAL_lookup(const struct req_op_context *p_context,
-			      struct fsal_obj_handle *parent,
+fsal_status_t GPFSFSAL_lookup(struct fsal_obj_handle *parent,
 			      const char *p_filename,
-			      struct attrlist *p_object_attr,
+			      struct fsal_attrlist *p_object_attr,
 			      struct gpfs_file_handle *fh,
 			      struct fsal_filesystem **new_fs);
 
@@ -315,25 +304,21 @@ fsal_status_t GPFSFSAL_lock_op(struct fsal_export *export,
 fsal_status_t GPFSFSAL_rename(struct fsal_obj_handle *old_hdl,
 			      const char *p_old_name,
 			      struct fsal_obj_handle *new_hdl,
-			      const char *p_new_name,
-			      const struct req_op_context *p_context);
+			      const char *p_new_name);
 
 fsal_status_t GPFSFSAL_readlink(struct fsal_obj_handle *dir_hdl,
-				const struct req_op_context *p_context,
 				char *p_link_content,
 				size_t link_len);
 
 fsal_status_t GPFSFSAL_symlink(struct fsal_obj_handle *dir_hdl,
 			       const char *p_linkname,
 			       const char *p_linkcontent,
-			       const struct req_op_context *p_context,
 			       uint32_t accessmode,	/* IN (ignored) */
 			       struct gpfs_file_handle *p_link_handle,
-			       struct attrlist *p_link_attributes);
+			       struct fsal_attrlist *p_link_attributes);
 
 fsal_status_t GPFSFSAL_unlink(struct fsal_obj_handle *dir_hdl,
-			      const char *p_object_name,
-			      const struct req_op_context *p_context);
+			      const char *p_object_name);
 
 void *GPFSFSAL_UP_Thread(void *Arg);
 
