@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: LGPL-3.0-or-later
 /**
  * @file    fsal_up.c
  * @brief   FSAL Upcall Interface
@@ -32,6 +33,7 @@
 #include <utime.h>
 #include <sys/time.h>
 #include <urcu-bp.h>
+#include "FSAL/fsal_localfs.h"
 
 /* Setup up_vector. File system's upvector_mutex must be held */
 static bool setup_up_vector(struct gpfs_filesystem *gpfs_fs)
@@ -48,11 +50,6 @@ static bool setup_up_vector(struct gpfs_filesystem *gpfs_fs)
 	/* wait for upcall readiness */
 	up_ready_wait(gpfs_fs->up_vector);
 
-	/* Set up op_ctx for the thread */
-	op_ctx = &gpfs_fs->req_ctx;
-	op_ctx->fsal_export = gpfs_fs->up_vector->up_fsal_export;
-	op_ctx->ctx_export = gpfs_fs->up_vector->up_gsh_export;
-
 	return true;
 }
 
@@ -82,6 +79,7 @@ void *GPFSFSAL_UP_Thread(void *Arg)
 	uint32_t upflags;
 	int errsv = 0;
 	fsal_status_t fsal_status = {0,};
+	struct req_op_context op_context;
 
 	rcu_register_thread();
 
@@ -116,8 +114,8 @@ void *GPFSFSAL_UP_Thread(void *Arg)
 			continue;
 		else {
 			LogEvent(COMPONENT_FSAL_UP,
-				 "nfs_init_wait_timeout() completed with "
-				 "rc %d", rc);
+				 "nfs_init_wait_timeout() completed with rc %d",
+				 rc);
 			return NULL;
 		}
 	}
@@ -226,6 +224,15 @@ void *GPFSFSAL_UP_Thread(void *Arg)
 			PTHREAD_MUTEX_unlock(&gpfs_fs->upvector_mutex);
 			goto out;
 		}
+
+		/* Get a ref to the gpfs_fs->up_vector->up_gsh_export and
+		 * initialize op_context for the thread.
+		 */
+		get_gsh_export_ref(gpfs_fs->up_vector->up_gsh_export);
+		init_op_context_simple(&op_context,
+				       gpfs_fs->up_vector->up_gsh_export,
+				       gpfs_fs->up_vector->up_fsal_export);
+
 		event_func = gpfs_fs->up_vector;
 
 		switch (reason) {
@@ -335,7 +342,7 @@ void *GPFSFSAL_UP_Thread(void *Arg)
 
 		case INODE_UPDATE:	/* Update Event */
 			{
-				struct attrlist attr;
+				struct fsal_attrlist attr;
 
 				LogMidDebug(COMPONENT_FSAL_UP,
 					    "inode update: flags:%x update ino %"
@@ -447,6 +454,7 @@ void *GPFSFSAL_UP_Thread(void *Arg)
 			LogDebug(COMPONENT_FSAL_UP,
 				"Terminating the GPFS up call thread for %d",
 				gpfs_fs->root_fd);
+			release_op_context();
 			PTHREAD_MUTEX_unlock(&gpfs_fs->upvector_mutex);
 			goto out;
 
@@ -468,15 +476,18 @@ void *GPFSFSAL_UP_Thread(void *Arg)
 			 * eventually get other errors that stop this
 			 * thread.
 			 */
+			release_op_context();
 			PTHREAD_MUTEX_unlock(&gpfs_fs->upvector_mutex);
 			continue; /* get next event */
 
 		default:
+			release_op_context();
 			PTHREAD_MUTEX_unlock(&gpfs_fs->upvector_mutex);
 			LogWarn(COMPONENT_FSAL_UP, "Unknown event: %d", reason);
 			continue;
 		}
 
+		release_op_context();
 		PTHREAD_MUTEX_unlock(&gpfs_fs->upvector_mutex);
 
 		if (FSAL_IS_ERROR(fsal_status) &&
