@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: LGPL-3.0-or-later
 /**
  * @file handle.c
  * @brief GPFS object (file|dir) handle object
@@ -41,6 +42,7 @@
 #include "fsal_internal.h"
 #include "fsal_convert.h"
 #include "FSAL/fsal_commonlib.h"
+#include "FSAL/fsal_localfs.h"
 #include "gpfs_methods.h"
 #include "nfs_proto_tools.h"
 
@@ -54,7 +56,7 @@
 
 struct gpfs_fsal_obj_handle *alloc_handle(struct gpfs_file_handle *fh,
 					 struct fsal_filesystem *fs,
-					 struct attrlist *attributes,
+					 struct fsal_attrlist *attributes,
 					 const char *link_content,
 					 struct fsal_export *exp_hdl)
 {
@@ -97,13 +99,13 @@ struct gpfs_fsal_obj_handle *alloc_handle(struct gpfs_file_handle *fh,
  */
 static fsal_status_t lookup(struct fsal_obj_handle *parent,
 			    const char *path, struct fsal_obj_handle **handle,
-			    struct attrlist *attrs_out)
+			    struct fsal_attrlist *attrs_out)
 {
 	fsal_errors_t fsal_error = ERR_FSAL_NO_ERROR;
 	int retval = 0;
 	fsal_status_t status;
 	struct gpfs_fsal_obj_handle *hdl;
-	struct attrlist attrib;
+	struct fsal_attrlist attrib;
 	struct gpfs_file_handle *fh = alloca(sizeof(struct gpfs_file_handle));
 	struct fsal_filesystem *fs;
 
@@ -132,7 +134,7 @@ static fsal_status_t lookup(struct fsal_obj_handle *parent,
 	if (attrs_out != NULL)
 		attrib.request_mask |= attrs_out->request_mask;
 
-	status = GPFSFSAL_lookup(op_ctx, parent, path, &attrib, fh, &fs);
+	status = GPFSFSAL_lookup(parent, path, &attrib, fh, &fs);
 	if (FSAL_IS_ERROR(status))
 		return status;
 
@@ -156,15 +158,15 @@ static fsal_status_t lookup(struct fsal_obj_handle *parent,
 }
 
 static fsal_status_t makedir(struct fsal_obj_handle *dir_hdl,
-			     const char *name, struct attrlist *attr_in,
+			     const char *name, struct fsal_attrlist *attr_in,
 			     struct fsal_obj_handle **handle,
-			     struct attrlist *attrs_out)
+			     struct fsal_attrlist *attrs_out)
 {
 	struct gpfs_fsal_obj_handle *hdl;
 	fsal_status_t status;
 	struct gpfs_file_handle *fh = alloca(sizeof(struct gpfs_file_handle));
-	/* Use a separate attrlist to getch the actual attributes into */
-	struct attrlist attrib;
+	/* Use a separate fsal_attrlist to getch the actual attributes into */
+	struct fsal_attrlist attrib;
 
 	*handle = NULL;		/* poison it */
 	if (!fsal_obj_handle_is(dir_hdl, DIRECTORY)) {
@@ -182,7 +184,7 @@ static fsal_status_t makedir(struct fsal_obj_handle *dir_hdl,
 		attrib.request_mask |= attrs_out->request_mask;
 
 	status =
-	    GPFSFSAL_mkdir(dir_hdl, name, op_ctx, attr_in->mode, fh, &attrib);
+	    GPFSFSAL_mkdir(dir_hdl, name, attr_in->mode, fh, &attrib);
 	if (FSAL_IS_ERROR(status))
 		return status;
 
@@ -225,15 +227,15 @@ static fsal_status_t makedir(struct fsal_obj_handle *dir_hdl,
 
 static fsal_status_t makenode(struct fsal_obj_handle *dir_hdl,
 			      const char *name, object_file_type_t nodetype,
-			      struct attrlist *attr_in,
+			      struct fsal_attrlist *attr_in,
 			      struct fsal_obj_handle **handle,
-			      struct attrlist *attrs_out)
+			      struct fsal_attrlist *attrs_out)
 {
 	fsal_status_t status;
 	struct gpfs_fsal_obj_handle *hdl;
 	struct gpfs_file_handle *fh = alloca(sizeof(struct gpfs_file_handle));
-	/* Use a separate attrlist to getch the actual attributes into */
-	struct attrlist attrib;
+	/* Use a separate fsal_attrlist to getch the actual attributes into */
+	struct fsal_attrlist attrib;
 
 	*handle = NULL;		/* poison it */
 	if (!fsal_obj_handle_is(dir_hdl, DIRECTORY)) {
@@ -252,7 +254,7 @@ static fsal_status_t makenode(struct fsal_obj_handle *dir_hdl,
 		attrib.request_mask |= attrs_out->request_mask;
 
 	status =
-	    GPFSFSAL_mknode(dir_hdl, name, op_ctx, attr_in->mode, nodetype,
+	    GPFSFSAL_mknode(dir_hdl, name, attr_in->mode, nodetype,
 			    &attr_in->rawdev, fh, &attrib);
 	if (FSAL_IS_ERROR(status))
 		return status;
@@ -301,15 +303,15 @@ static fsal_status_t makenode(struct fsal_obj_handle *dir_hdl,
  */
 static fsal_status_t makesymlink(struct fsal_obj_handle *dir_hdl,
 				 const char *name, const char *link_path,
-				 struct attrlist *attr_in,
+				 struct fsal_attrlist *attr_in,
 				 struct fsal_obj_handle **handle,
-				 struct attrlist *attrs_out)
+				 struct fsal_attrlist *attrs_out)
 {
 	fsal_status_t status;
 	struct gpfs_fsal_obj_handle *hdl;
 	struct gpfs_file_handle *fh = alloca(sizeof(struct gpfs_file_handle));
-	/* Use a separate attrlist to getch the actual attributes into */
-	struct attrlist attrib;
+	/* Use a separate fsal_attrlist to getch the actual attributes into */
+	struct fsal_attrlist attrib;
 
 	*handle = NULL;		/* poison it first */
 	if (!fsal_obj_handle_is(dir_hdl, DIRECTORY)) {
@@ -326,7 +328,7 @@ static fsal_status_t makesymlink(struct fsal_obj_handle *dir_hdl,
 	if (attrs_out != NULL)
 		attrib.request_mask |= attrs_out->request_mask;
 
-	status = GPFSFSAL_symlink(dir_hdl, name, link_path, op_ctx,
+	status = GPFSFSAL_symlink(dir_hdl, name, link_path,
 				  attr_in->mode, fh, &attrib);
 	if (FSAL_IS_ERROR(status))
 		return status;
@@ -389,7 +391,7 @@ static fsal_status_t readsymlink(struct fsal_obj_handle *obj_hdl,
 			myself->u.symlink.link_size = 0;
 		}
 
-		status = GPFSFSAL_readlink(obj_hdl, op_ctx, link_buff,
+		status = GPFSFSAL_readlink(obj_hdl, link_buff,
 					   sizeof(link_buff));
 
 		if (FSAL_IS_ERROR(status))
@@ -417,7 +419,7 @@ static fsal_status_t linkfile(struct fsal_obj_handle *obj_hdl,
 
 	myself = container_of(obj_hdl, struct gpfs_fsal_obj_handle, obj_handle);
 
-	status = GPFSFSAL_link(destdir_hdl, myself->handle, name, op_ctx);
+	status = GPFSFSAL_link(destdir_hdl, myself->handle, name);
 
 	return status;
 }
@@ -478,7 +480,7 @@ static fsal_status_t read_dirents(struct fsal_obj_handle *dir_hdl,
 			break;
 		for (bpos = 0; bpos < nread;) {
 			struct fsal_obj_handle *hdl;
-			struct attrlist attrs;
+			struct fsal_attrlist attrs;
 			enum fsal_dir_result cb_rc;
 
 			dentry = (struct dirent64 *)(buf + bpos);
@@ -509,7 +511,7 @@ static fsal_status_t read_dirents(struct fsal_obj_handle *dir_hdl,
 				}
 			}
 
-			/* callback to cache inode */
+			/* callback to MDCACHE */
 			cb_rc = cb(dentry->d_name, hdl, &attrs, dir_state,
 				   (fsal_cookie_t) dentry->d_off);
 
@@ -538,21 +540,19 @@ static fsal_status_t renamefile(struct fsal_obj_handle *obj_hdl,
 {
 	fsal_status_t status;
 
-	status =
-	    GPFSFSAL_rename(olddir_hdl, old_name, newdir_hdl, new_name,
-			    op_ctx);
+	status = GPFSFSAL_rename(olddir_hdl, old_name, newdir_hdl, new_name);
 	return status;
 }
 
 /* FIXME: attributes are now merged into fsal_obj_handle.  This
  * spreads everywhere these methods are used.  eventually deprecate
- * everywhere except where we explicitly want to to refresh them.
+ * everywhere except where we explicitly want to refresh them.
  * NOTE: this is done under protection of the attributes rwlock in the
  * cache entry.
  */
 
 static fsal_status_t getattrs(struct fsal_obj_handle *obj_hdl,
-			      struct attrlist *attrs)
+			      struct fsal_attrlist *attrs)
 {
 	struct gpfs_fsal_obj_handle *myself;
 	fsal_status_t status = {ERR_FSAL_NO_ERROR, 0};
@@ -562,7 +562,7 @@ static fsal_status_t getattrs(struct fsal_obj_handle *obj_hdl,
 
 	status = GPFSFSAL_getattrs(op_ctx->fsal_export,
 				   obj_hdl->fs->private_data,
-				   op_ctx, myself->handle,
+				   myself->handle,
 				   attrs);
 	if (FSAL_IS_ERROR(status)) {
 		goto out;
@@ -577,7 +577,7 @@ static fsal_status_t getattrs(struct fsal_obj_handle *obj_hdl,
 
 	fsal_status_t fs_loc_status = GPFSFSAL_fs_loc(op_ctx->fsal_export,
 						      obj_hdl->fs->private_data,
-						      op_ctx, myself->handle,
+						      myself->handle,
 						      attrs);
 
 	if (FSAL_IS_SUCCESS(fs_loc_status)) {
@@ -593,7 +593,7 @@ out:
 }
 
 static fsal_status_t getxattrs(struct fsal_obj_handle *obj_hdl,
-				xattrname4 *xa_name,
+				xattrkey4 *xa_name,
 				xattrvalue4 *xa_value)
 {
 	int rc;
@@ -639,8 +639,8 @@ static fsal_status_t getxattrs(struct fsal_obj_handle *obj_hdl,
 }
 
 static fsal_status_t setxattrs(struct fsal_obj_handle *obj_hdl,
-				setxattr_type4 sa_type,
-				xattrname4 *xa_name,
+				setxattr_option4 option,
+				xattrkey4 *xa_name,
 				xattrvalue4 *xa_value)
 {
 	int rc;
@@ -660,6 +660,7 @@ static fsal_status_t setxattrs(struct fsal_obj_handle *obj_hdl,
 	sxarg.name = xa_name->utf8string_val;
 	sxarg.value_len = xa_value->utf8string_len;
 	sxarg.value = xa_value->utf8string_val;
+	sxarg.cli_ip = NULL;
 	if (op_ctx && op_ctx->client)
 		sxarg.cli_ip = op_ctx->client->hostaddr_str;
 
@@ -675,7 +676,7 @@ static fsal_status_t setxattrs(struct fsal_obj_handle *obj_hdl,
 }
 
 static fsal_status_t removexattrs(struct fsal_obj_handle *obj_hdl,
-				xattrname4 *xa_name)
+				xattrkey4 *xa_name)
 {
 	int rc;
 	int errsv;
@@ -692,6 +693,7 @@ static fsal_status_t removexattrs(struct fsal_obj_handle *obj_hdl,
 	rxarg.handle = myself->handle;
 	rxarg.name_len = xa_name->utf8string_len;
 	rxarg.name = xa_name->utf8string_val;
+	rxarg.cli_ip = NULL;
 	if (op_ctx && op_ctx->client)
 		rxarg.cli_ip = op_ctx->client->hostaddr_str;
 
@@ -709,7 +711,6 @@ static fsal_status_t removexattrs(struct fsal_obj_handle *obj_hdl,
 static fsal_status_t listxattrs(struct fsal_obj_handle *obj_hdl,
 				count4 la_maxcount,
 				nfs_cookie4 *la_cookie,
-				verifier4 *la_cookieverf,
 				bool_t *lr_eof,
 				xattrlist4 *lr_names)
 {
@@ -720,7 +721,7 @@ static fsal_status_t listxattrs(struct fsal_obj_handle *obj_hdl,
 	char *buf = NULL;
 	struct listxattr_arg lxarg;
 	struct gpfs_fsal_obj_handle *myself;
-	component4 *entry = lr_names->entries;
+	component4 *entry = lr_names->xl4_entries;
 	struct gpfs_fsal_export *exp = container_of(op_ctx->fsal_export,
 					struct gpfs_fsal_export, export);
 	int export_fd = exp->export_fd;
@@ -736,17 +737,17 @@ static fsal_status_t listxattrs(struct fsal_obj_handle *obj_hdl,
 	lxarg.mountdirfd = export_fd;
 	lxarg.handle = myself->handle;
 	lxarg.cookie = 0; /* For now gpfs doesn't support cookie */
-	lxarg.verifier = *((uint64_t *)la_cookieverf);
+	lxarg.verifier = 0; /* @todo: protocol has no verifier now */
 	lxarg.eof = false;
 	lxarg.name_len = MAXCOUNT;
 	lxarg.names = buf;
+	lxarg.cli_ip = NULL;
 	if (op_ctx && op_ctx->client)
 		lxarg.cli_ip = op_ctx->client->hostaddr_str;
 
 	LogFullDebug(COMPONENT_FSAL,
-		"in cookie %llu len %d cookieverf %llx",
-		(unsigned long long)lxarg.cookie, la_maxcount,
-		(unsigned long long)lxarg.verifier);
+		"in cookie %llu len %d",
+		(unsigned long long)lxarg.cookie, la_maxcount);
 
 	rc = gpfs_ganesha(OPENHANDLE_LISTXATTRS, &lxarg);
 	if (rc < 0) {
@@ -779,22 +780,20 @@ static fsal_status_t listxattrs(struct fsal_obj_handle *obj_hdl,
 			 "nameP %s at offset %td", name, (next - name));
 
 		if (entryCount >= *la_cookie) {
-			if ((((char *)entry - (char *)lr_names->entries) +
+			if ((((char *)entry - (char *)lr_names->xl4_entries) +
 			     sizeof(component4) > la_maxcount) ||
 			     ((val - valstart)+(next - name) > la_maxcount)) {
 				gsh_free(buf);
 				*lr_eof = false;
 
-				lr_names->entryCount = entryCount - *la_cookie;
+				lr_names->xl4_count = entryCount - *la_cookie;
 				*la_cookie += entryCount;
 				LogFullDebug(COMPONENT_FSAL,
-				   "out1 cookie %llu off %td eof %d cookieverf %llx",
+				   "out1 cookie %llu off %td eof %d",
 				   (unsigned long long)*la_cookie,
-				   (next - name), *lr_eof,
-				   (unsigned long long)*
-				   ((uint64_t *)la_cookieverf));
+				   (next - name), *lr_eof);
 
-				if (lr_names->entryCount == 0)
+				if (lr_names->xl4_count == 0)
 					return fsalstat(ERR_FSAL_TOOSMALL, 0);
 				return fsalstat(ERR_FSAL_NO_ERROR, 0);
 			}
@@ -816,16 +815,15 @@ static fsal_status_t listxattrs(struct fsal_obj_handle *obj_hdl,
 		name = next;
 		entryCount += 1;
 	}
-	lr_names->entryCount = entryCount - *la_cookie;
+	lr_names->xl4_count = entryCount - *la_cookie;
 	*la_cookie = 0;
 	*lr_eof = true;
 
 	gsh_free(buf);
 
 	LogFullDebug(COMPONENT_FSAL,
-		"out2 cookie %llu eof %d cookieverf %llx",
-		(unsigned long long)*la_cookie, *lr_eof,
-		(unsigned long long)*((uint64_t *)la_cookieverf));
+		"out2 cookie %llu eof %d",
+		(unsigned long long)*la_cookie, *lr_eof);
 
 	return fsalstat(ERR_FSAL_NO_ERROR, 0);
 }
@@ -836,11 +834,11 @@ static fsal_status_t listxattrs(struct fsal_obj_handle *obj_hdl,
 fsal_status_t gpfs_setattr2(struct fsal_obj_handle *obj_hdl,
 				   bool bypass,
 				   struct state_t *state,
-				   struct attrlist *attrs)
+				   struct fsal_attrlist *attrs)
 {
 	fsal_status_t status;
 
-	status = GPFSFSAL_setattrs(obj_hdl, op_ctx, attrs);
+	status = GPFSFSAL_setattrs(obj_hdl, attrs);
 
 	return status;
 }
@@ -854,7 +852,7 @@ static fsal_status_t file_unlink(struct fsal_obj_handle *dir_hdl,
 {
 	fsal_status_t status;
 
-	status = GPFSFSAL_unlink(dir_hdl, name, op_ctx);
+	status = GPFSFSAL_unlink(dir_hdl, name);
 
 	return status;
 }
@@ -922,8 +920,9 @@ static void handle_to_key(struct fsal_obj_handle *obj_hdl,
 	fh_desc->len = myself->handle->handle_key_size;
 }
 
-/*
- * release
+/**
+ * @brief release object handle
+ *
  * release our export first so they know we are gone
  */
 static void release(struct fsal_obj_handle *obj_hdl)
@@ -998,7 +997,7 @@ void gpfs_handle_ops_init(struct fsal_obj_ops *ops)
 
 /**
  *  @param exp_hdl Handle
- *  @param path Path
+ *  @param path The path to lookup
  *  @param handle Reference to handle
  *
  *  modelled on old api except we don't stuff attributes.
@@ -1007,14 +1006,14 @@ void gpfs_handle_ops_init(struct fsal_obj_ops *ops)
 fsal_status_t gpfs_lookup_path(struct fsal_export *exp_hdl,
 			       const char *path,
 			       struct fsal_obj_handle **handle,
-			       struct attrlist *attrs_out)
+			       struct fsal_attrlist *attrs_out)
 {
 	fsal_status_t fsal_status;
 	int retval = 0;
 	int dir_fd;
 	struct fsal_filesystem *fs;
 	struct gpfs_fsal_obj_handle *hdl;
-	struct attrlist attributes;
+	struct fsal_attrlist attributes;
 	gpfsfsal_xstat_t buffxstat;
 	struct gpfs_file_handle *fh = alloca(sizeof(struct gpfs_file_handle));
 	struct fsal_fsid__ fsid;
@@ -1164,7 +1163,7 @@ errout:
  * @return status
  *
  * Does what original FSAL_ExpandHandle did (sort of)
- * returns a ref counted handle to be later used in cache_inode etc.
+ * returns a ref counted handle to be later used in mdcache etc.
  * NOTE! you must release this thing when done with it!
  * BEWARE! Thanks to some holes in the *AT syscalls implementation,
  * we cannot get an fd on an AF_UNIX socket, nor reliably on block or
@@ -1176,12 +1175,12 @@ errout:
 fsal_status_t gpfs_create_handle(struct fsal_export *exp_hdl,
 				 struct gsh_buffdesc *hdl_desc,
 				 struct fsal_obj_handle **handle,
-				 struct attrlist *attrs_out)
+				 struct fsal_attrlist *attrs_out)
 {
 	fsal_status_t status;
 	struct gpfs_fsal_obj_handle *hdl;
 	struct gpfs_file_handle *fh;
-	struct attrlist attrib;
+	struct fsal_attrlist attrib;
 	char link_buff[PATH_MAX];
 	struct fsal_fsid__ fsid;
 	struct fsal_filesystem *fs;
@@ -1224,7 +1223,7 @@ fsal_status_t gpfs_create_handle(struct fsal_export *exp_hdl,
 	if (attrs_out != NULL)
 		attrib.request_mask |= attrs_out->request_mask;
 
-	status = GPFSFSAL_getattrs(exp_hdl, gpfs_fs, op_ctx, fh, &attrib);
+	status = GPFSFSAL_getattrs(exp_hdl, gpfs_fs, fh, &attrib);
 	if (FSAL_IS_ERROR(status))
 		return status;
 
