@@ -1,3 +1,4 @@
+/* SPDX-License-Identifier: BSD-3-Clause */
 /*                                                                              */
 /* Copyright (C) 2001 International Business Machines                           */
 /* All rights reserved.                                                         */
@@ -52,6 +53,7 @@ struct flock
 #define OPENHANDLE_GET_VERSION    100
 #define OPENHANDLE_GET_VERSION2   1002
 #define OPENHANDLE_GET_VERSION3   1003
+#define OPENHANDLE_GET_VERSION4   1004
 #define OPENHANDLE_NAME_TO_HANDLE 101
 #define OPENHANDLE_OPEN_BY_HANDLE 102
 #define OPENHANDLE_LAYOUT_TYPE    106
@@ -130,6 +132,7 @@ struct trace_arg
 #define ganesha_v1 1
 #define ganesha_v2 2
 #define ganesha_v3 3
+#define ganesha_v4 4
 
 int gpfs_ganesha(int op, void *oarg);
 
@@ -369,8 +372,8 @@ struct pnfstime4 {
 };
 
 struct nfsd4_pnfs_dev_iter_res {
-	uint64_t		gd_cookie;	/* request/repsonse */
-	uint64_t		gd_verf;	/* request/repsonse */
+	uint64_t		gd_cookie;	/* request/response */
+	uint64_t		gd_verf;	/* request/response */
 	uint64_t		gd_devid;	/* response */
 	uint32_t		gd_eof;		/* response */
 };
@@ -398,7 +401,7 @@ struct nfsd4_pnfs_layoutget_arg {
 };
 
 struct nfsd4_pnfs_layoutget_res {
-	struct nfsd4_layout_seg	lg_seg;	/* request/resopnse */
+	struct nfsd4_layout_seg	lg_seg;	/* request/response */
 	uint32_t		lg_return_on_close;
 };
 
@@ -569,7 +572,8 @@ struct layoutcommit_arg
                                new_offset if set, otherwise undefined. */
   uint32_t time_changed; /* True if the client provided a new value for mtime */
   struct gpfs_time_t new_time;  /* If time_changed is true, the client-supplied
-                               modification tiem for the file.  otherwise, undefined. */
+				   modification time for the file.  otherwise,
+				   undefined. */
   struct gpfs_exp_xdr_stream *xdr;
 };
 
@@ -625,6 +629,7 @@ struct stat_name_arg
     const char *name;
     struct gpfs_file_handle *handle;
     struct stat *buf;
+    const char *cli_ip;
 };
 
 struct callback_arg
@@ -703,6 +708,7 @@ struct xstat_arg
     struct stat *buf;
     struct fsal_fsid *fsid;
     uint32_t *expire_attr;
+    const char *cli_ip;
 };
 
 struct getxattr_arg {
