@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: LGPL-3.0-or-later
 /*
  * vim:noexpandtab:shiftwidth=8:tabstop=8:
  *
@@ -113,7 +114,7 @@ mem_update(struct mem_fsal_export *mfe, struct mem_fsal_obj_handle *hdl)
 	const struct fsal_up_vector *up_ops = mfe->export.up_ops;
 	fsal_status_t status;
 	struct gsh_buffdesc fh_desc;
-	struct attrlist attrs;
+	struct fsal_attrlist attrs;
 
 	LogFullDebug(COMPONENT_FSAL_UP, "updating %s", hdl->m_name);
 
@@ -182,7 +183,7 @@ mem_rand_obj(struct mem_fsal_export *mfe)
  *
  * - Pick a random obj in each export, and invalidate it
  *
- * @param[in] ctx	Thread context
+ * @param[in] ctx	Thread fridge context
  * @return Return description
  */
 static void
