@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: LGPL-3.0-or-later
 /*
  * vim:noexpandtab:shiftwidth=8:tabstop=8:
  *
@@ -50,13 +51,13 @@ typedef int (*xattr_getfunc_t) (struct fsal_obj_handle *, /* object handle */
 				void *,	/* output buff */
 				size_t,	/* output buff size */
 				size_t *,	/* output size */
-				void *arg);	/* optionnal argument */
+				void *arg);	/* optional argument */
 
 typedef int (*xattr_setfunc_t) (struct fsal_obj_handle *, /* object handle */
 				void *,	/* input buff */
 				size_t,	/* input size */
 				int,	/* creation flag */
-				void *arg);	/* optionnal argument */
+				void *arg);	/* optional argument */
 
 struct fsal_xattr_def {
 	char xattr_name[XATTR_NAME_SIZE];
@@ -455,7 +456,7 @@ fsal_status_t vfs_getextattr_value(struct vfs_fsal_obj_handle *vfs_hdl,
 	/* is it an xattr? */
 	rc = fgetxattr(local_fd, xattr_name, buffer_addr, buffer_size);
 	if (rc < 0) {
-		st = fsalstat(posix2fsal_error(rc), errno);
+		st = fsalstat(posix2fsal_error(errno), errno);
 		goto out;
 	}
 
