@@ -1,3 +1,4 @@
+/* SPDX-License-Identifier: LGPL-3.0-or-later */
 /*
  * vim:noexpandtab:shiftwidth=8:tabstop=8:
  *
@@ -39,11 +40,11 @@
  */
 
 /**
- * @brief Structure to hold MDCACHE paramaters
+ * @brief Structure to hold MDCACHE parameters
  */
 
 struct mdcache_parameter {
-	/** Partitions in the Cache_Inode tree.  Defaults to 7,
+	/** Partitions in the MDCACHE tree.  Defaults to 7,
 	 * settable with NParts. */
 	uint32_t nparts;
 	/** Per-partition hash table size.  Defaults to 32633,
@@ -69,6 +70,11 @@ struct mdcache_parameter {
 	/** High water mark for cache entries.  Defaults to 100000,
 	    settable by Entries_HWMark. */
 	uint32_t entries_hwmark;
+	/** When the handle cache is over the high water mark, attempt to
+	    release this number of entries in each pass until it's back below
+	    the high water mark. Set it to 0 does not attempt to release
+	    entries. Defaults to 100, settable by Entries_Release_Size. */
+	uint32_t entries_release_size;
 	/** High water mark for chunks.  Defaults to 100000,
 	    settable by Chunks_HWMark. */
 	uint32_t chunks_hwmark;
@@ -102,7 +108,7 @@ struct mdcache_parameter {
 	    Defaults to 40, settable with Biggest_Window */
 	uint32_t biggest_window;
 	/** Percentage of progress toward the high water mark required
-	    in in a pass through the thread when in extremis.
+	    in a pass through the thread when in extremis.
 	    Defaults to 5, settable with Required_Progress. */
 	uint32_t required_progress;
 	/** Number of failures to approach the high watermark before
