@@ -1,7 +1,8 @@
+// SPDX-License-Identifier: LGPL-3.0-or-later
 /*
  * vim:noexpandtab:shiftwidth=8:tabstop=8:
  *
- * Copyright 2015-2019 Red Hat, Inc. and/or its affiliates.
+ * Copyright 2015-2021 Red Hat, Inc. and/or its affiliates.
  * Author: Daniel Gryniewicz <dang@redhat.com>
  *
  * This program is free software; you can redistribute it and/or
@@ -86,8 +87,8 @@ fsal_status_t mdcache_alloc_and_check_handle(
 		struct fsal_obj_handle *sub_handle,
 		struct fsal_obj_handle **new_obj,
 		bool new_directory,
-		struct attrlist *attrs_in,
-		struct attrlist *attrs_out,
+		struct fsal_attrlist *attrs_in,
+		struct fsal_attrlist *attrs_out,
 		const char *tag,
 		mdcache_entry_t *parent,
 		const char *name,
@@ -97,8 +98,8 @@ fsal_status_t mdcache_alloc_and_check_handle(
 	fsal_status_t status;
 	mdcache_entry_t *new_entry;
 
-	status = mdcache_new_entry(export, sub_handle, attrs_in, attrs_out,
-				   new_directory, &new_entry, state,
+	status = mdcache_new_entry(export, sub_handle, attrs_in, false,
+				   attrs_out, new_directory, &new_entry, state,
 				   MDC_REASON_DEFAULT);
 
 	if (FSAL_IS_ERROR(status)) {
@@ -169,7 +170,7 @@ fsal_status_t mdcache_alloc_and_check_handle(
 static fsal_status_t mdcache_lookup(struct fsal_obj_handle *parent,
 				    const char *name,
 				    struct fsal_obj_handle **handle,
-				    struct attrlist *attrs_out)
+				    struct fsal_attrlist *attrs_out)
 {
 	mdcache_entry_t *mdc_parent =
 		container_of(parent, mdcache_entry_t, obj_handle);
@@ -197,9 +198,9 @@ static fsal_status_t mdcache_lookup(struct fsal_obj_handle *parent,
  * @return FSAL status
  */
 static fsal_status_t mdcache_mkdir(struct fsal_obj_handle *dir_hdl,
-			     const char *name, struct attrlist *attrib,
+			     const char *name, struct fsal_attrlist *attrib,
 			     struct fsal_obj_handle **handle,
-			     struct attrlist *attrs_out)
+			     struct fsal_attrlist *attrs_out)
 {
 	mdcache_entry_t *parent =
 		container_of(dir_hdl, mdcache_entry_t,
@@ -207,7 +208,7 @@ static fsal_status_t mdcache_mkdir(struct fsal_obj_handle *dir_hdl,
 	struct mdcache_fsal_export *export = mdc_cur_export();
 	struct fsal_obj_handle *sub_handle;
 	fsal_status_t status;
-	struct attrlist attrs;
+	struct fsal_attrlist attrs;
 	bool invalidate = true;
 
 	*handle = NULL;
@@ -276,9 +277,9 @@ static fsal_status_t mdcache_mkdir(struct fsal_obj_handle *dir_hdl,
  */
 static fsal_status_t mdcache_mknode(struct fsal_obj_handle *dir_hdl,
 			      const char *name, object_file_type_t nodetype,
-			      struct attrlist *attrib,
+			      struct fsal_attrlist *attrib,
 			      struct fsal_obj_handle **handle,
-			      struct attrlist *attrs_out)
+			      struct fsal_attrlist *attrs_out)
 {
 	mdcache_entry_t *parent =
 		container_of(dir_hdl, mdcache_entry_t,
@@ -286,7 +287,7 @@ static fsal_status_t mdcache_mknode(struct fsal_obj_handle *dir_hdl,
 	struct mdcache_fsal_export *export = mdc_cur_export();
 	struct fsal_obj_handle *sub_handle;
 	fsal_status_t status;
-	struct attrlist attrs;
+	struct fsal_attrlist attrs;
 	bool invalidate = true;
 
 	*handle = NULL;
@@ -356,9 +357,9 @@ static fsal_status_t mdcache_mknode(struct fsal_obj_handle *dir_hdl,
  */
 static fsal_status_t mdcache_symlink(struct fsal_obj_handle *dir_hdl,
 				 const char *name, const char *link_path,
-				 struct attrlist *attrib,
+				 struct fsal_attrlist *attrib,
 				 struct fsal_obj_handle **handle,
-				 struct attrlist *attrs_out)
+				 struct fsal_attrlist *attrs_out)
 {
 	mdcache_entry_t *parent =
 		container_of(dir_hdl, mdcache_entry_t,
@@ -366,7 +367,7 @@ static fsal_status_t mdcache_symlink(struct fsal_obj_handle *dir_hdl,
 	struct mdcache_fsal_export *export = mdc_cur_export();
 	struct fsal_obj_handle *sub_handle;
 	fsal_status_t status;
-	struct attrlist attrs;
+	struct fsal_attrlist attrs;
 	bool invalidate = true;
 
 	*handle = NULL;
@@ -468,7 +469,7 @@ static fsal_status_t mdcache_readlink(struct fsal_obj_handle *obj_hdl,
  * @brief Create a hard link
  *
  * @param[in] obj_hdl	Object to link to.
- * @param[in] destdir_hdl	Destination dirctory into which to link
+ * @param[in] destdir_hdl	Destination directory into which to link
  * @param[in] name	Name of new link
  * @return FSAL status
  */
@@ -519,7 +520,7 @@ static fsal_status_t mdcache_link(struct fsal_obj_handle *obj_hdl,
 }
 
 /**
- * Read the contents of a dirctory
+ * Read the contents of a directory
  *
  * If necessary, populate the dirent cache from the underlying FSAL.  Then, walk
  * the dirent cache calling the callback.
@@ -594,7 +595,7 @@ static fsal_status_t mdcache_test_access(struct fsal_obj_handle *obj_hdl,
 	mdcache_entry_t *entry =
 		container_of(obj_hdl, mdcache_entry_t, obj_handle);
 
-	if (owner_skip && entry->attrs.owner == op_ctx->creds->caller_uid)
+	if (owner_skip && entry->attrs.owner == op_ctx->creds.caller_uid)
 		return fsalstat(ERR_FSAL_NO_ERROR, 0);
 
 	return fsal_test_access(obj_hdl, access_type, allowed, denied,
@@ -801,14 +802,16 @@ out:
  * @param[in] entry		The mdcache entry to refresh attributes for.
  * @param[in] need_acl		Indicates if the ACL needs updating.
  * @param[in] need_fslocations	Indicates if the fslocations are needed.
+ * @param[in] need_seclabel	Indicates if security label is needed.
  * @param[in] invalidate	Invalidate the dirent cache if the entry is a
  *				directory.
  */
 
 fsal_status_t mdcache_refresh_attrs(mdcache_entry_t *entry, bool need_acl,
-				    bool need_fslocations, bool invalidate)
+				    bool need_fslocations, bool need_seclabel,
+				    bool invalidate)
 {
-	struct attrlist attrs;
+	struct fsal_attrlist attrs;
 	fsal_status_t status = {0, 0};
 	struct timespec oldmtime;
 	bool file_deleg = false;
@@ -838,6 +841,10 @@ fsal_status_t mdcache_refresh_attrs(mdcache_entry_t *entry, bool need_acl,
 		attrs.request_mask &= ~ATTR4_FS_LOCATIONS;
 	}
 
+	if (!need_seclabel) {
+		attrs.request_mask &= ~ATTR4_SEC_LABEL;
+	}
+
 	if (file_deleg && entry->attrs.expire_time_attr) {
 		/* If the file is delegated, then we can trust
 		 * the attributes already fetched (i.e, which
@@ -860,6 +867,16 @@ fsal_status_t mdcache_refresh_attrs(mdcache_entry_t *entry, bool need_acl,
 		/* request_mask & ATTR_ACL must match attrs.acl */
 		entry->attrs.request_mask |= ATTR_ACL;
 	}
+	if (entry->attrs.fs_locations != NULL) {
+		/* request_mask & ATTR_FS_LOCATIONS must match
+		 * attrs.fs_locations */
+		entry->attrs.request_mask |= ATTR4_FS_LOCATIONS;
+	}
+	if (entry->attrs.sec_label.slai_data.slai_data_val != NULL) {
+		/* request_mask & ATTR_ACL must match
+		 * attrs.sec_label.slai_data.slai_data_val */
+		entry->attrs.request_mask |= ATTR4_SEC_LABEL;
+	}
 
 	original_generation = atomic_fetch_int32_t(&entry->attr_generation);
 	subcall(
@@ -922,7 +939,7 @@ out:
  * @return FSAL status
  */
 static fsal_status_t mdcache_getattrs(struct fsal_obj_handle *obj_hdl,
-				      struct attrlist *attrs_out)
+				      struct fsal_attrlist *attrs_out)
 {
 	mdcache_entry_t *entry =
 		container_of(obj_hdl, mdcache_entry_t, obj_handle);
@@ -947,6 +964,7 @@ static fsal_status_t mdcache_getattrs(struct fsal_obj_handle *obj_hdl,
 	status = mdcache_refresh_attrs(
 			entry, (attrs_out->request_mask & ATTR_ACL) != 0,
 			(attrs_out->request_mask & ATTR4_FS_LOCATIONS) != 0,
+			(attrs_out->request_mask & ATTR4_SEC_LABEL) != 0,
 			true);
 
 	if (FSAL_IS_ERROR(status)) {
@@ -988,7 +1006,7 @@ unlock_no_attrs:
 static fsal_status_t mdcache_setattr2(struct fsal_obj_handle *obj_hdl,
 				      bool bypass,
 				      struct state_t *state,
-				      struct attrlist *attrs)
+				      struct fsal_attrlist *attrs)
 {
 	mdcache_entry_t *entry =
 		container_of(obj_hdl, mdcache_entry_t, obj_handle);
@@ -1020,8 +1038,7 @@ static fsal_status_t mdcache_setattr2(struct fsal_obj_handle *obj_hdl,
 	}
 
 	PTHREAD_RWLOCK_wrlock(&entry->attr_lock);
-	status2 = mdcache_refresh_attrs(entry, need_acl,
-					false /*need_fslocations*/, false);
+	status2 = mdcache_refresh_attrs(entry, need_acl, false, false, false);
 	if (FSAL_IS_ERROR(status2)) {
 		/* Assume that the cache is bogus now */
 		atomic_clear_uint32_t_bits(&entry->mde_flags,
@@ -1032,9 +1049,9 @@ static fsal_status_t mdcache_setattr2(struct fsal_obj_handle *obj_hdl,
 			kill_entry = true;
 	} else if (change == entry->attrs.change) {
 		LogDebug(COMPONENT_CACHE_INODE,
-			 "setattr2 did not change change attribute before %lld after = %lld",
-			 (long long int) change,
-			 (long long int) entry->attrs.change);
+			 "setattr2 did not change attribute before %lld after = %lld",
+			 (long long) change,
+			 (long long) entry->attrs.change);
 		entry->attrs.change = change + 1;
 	}
 	PTHREAD_RWLOCK_unlock(&entry->attr_lock);
@@ -1204,7 +1221,6 @@ static bool mdcache_handle_cmp(struct fsal_obj_handle *obj_hdl1,
  *
  * @param[in]     obj_hdl  The handle of the file on which the layout is
  *                         requested.
- * @param[in]     req_ctx  Request context
  * @param[out]    loc_body An XDR stream to which the FSAL must encode
  *                         the layout specific portion of the granted
  *                         layout segment.
@@ -1214,7 +1230,6 @@ static bool mdcache_handle_cmp(struct fsal_obj_handle *obj_hdl1,
  * @return Valid error codes in RFC 5661, pp. 366-7.
  */
 static nfsstat4 mdcache_layoutget(struct fsal_obj_handle *obj_hdl,
-				  struct req_op_context *req_ctx,
 				  XDR *loc_body,
 				  const struct fsal_layoutget_arg *arg,
 				  struct fsal_layoutget_res *res)
@@ -1225,7 +1240,7 @@ static nfsstat4 mdcache_layoutget(struct fsal_obj_handle *obj_hdl,
 
 	subcall(
 		status = entry->sub_handle->obj_ops->layoutget(
-			entry->sub_handle, req_ctx, loc_body, arg, res)
+			entry->sub_handle, loc_body, arg, res)
 	       );
 
 	return status;
@@ -1237,7 +1252,6 @@ static nfsstat4 mdcache_layoutget(struct fsal_obj_handle *obj_hdl,
  * Delegate to sub-FSAL
  *
  * @param[in] obj_hdl  The object on which a segment is to be returned
- * @param[in] req_ctx  Request context
  * @param[in] lrf_body In the case of a non-synthetic return, this is
  *                     an XDR stream corresponding to the layout
  *                     type-specific argument to LAYOUTRETURN.  In
@@ -1248,7 +1262,6 @@ static nfsstat4 mdcache_layoutget(struct fsal_obj_handle *obj_hdl,
  * @return Valid error codes in RFC 5661, p. 367.
  */
 static nfsstat4 mdcache_layoutreturn(struct fsal_obj_handle *obj_hdl,
-				     struct req_op_context *req_ctx,
 				     XDR *lrf_body,
 				     const struct fsal_layoutreturn_arg *arg)
 {
@@ -1258,7 +1271,7 @@ static nfsstat4 mdcache_layoutreturn(struct fsal_obj_handle *obj_hdl,
 
 	subcall(
 		status = entry->sub_handle->obj_ops->layoutreturn(
-			entry->sub_handle, req_ctx, lrf_body, arg)
+			entry->sub_handle, lrf_body, arg)
 	       );
 
 	return status;
@@ -1270,7 +1283,6 @@ static nfsstat4 mdcache_layoutreturn(struct fsal_obj_handle *obj_hdl,
  * Delegate to sub-FSAL
  *
  * @param[in]     obj_hdl  The object on which to commit
- * @param[in]     req_ctx  Request context
  * @param[in]     lou_body An XDR stream containing the layout
  *                         type-specific portion of the LAYOUTCOMMIT
  *                         arguments.
@@ -1280,7 +1292,6 @@ static nfsstat4 mdcache_layoutreturn(struct fsal_obj_handle *obj_hdl,
  * @return Valid error codes in RFC 5661, p. 366.
  */
 static nfsstat4 mdcache_layoutcommit(struct fsal_obj_handle *obj_hdl,
-				     struct req_op_context *req_ctx,
 				     XDR *lou_body,
 				     const struct fsal_layoutcommit_arg *arg,
 				     struct fsal_layoutcommit_res *res)
@@ -1291,7 +1302,7 @@ static nfsstat4 mdcache_layoutcommit(struct fsal_obj_handle *obj_hdl,
 
 	subcall(
 		status = entry->sub_handle->obj_ops->layoutcommit(
-			entry->sub_handle, req_ctx, lou_body, arg, res)
+			entry->sub_handle, lou_body, arg, res)
 	       );
 
 	if (status == NFS4_OK)
@@ -1376,14 +1387,14 @@ static fsal_status_t mdcache_merge(struct fsal_obj_handle *orig_hdl,
 
 
 static bool mdcache_is_referral(struct fsal_obj_handle *obj_hdl,
-				struct attrlist *unused,
+				struct fsal_attrlist *unused,
 				bool cache_attrs)
 {
 	mdcache_entry_t *entry =
 		container_of(obj_hdl, mdcache_entry_t, obj_handle);
 	bool result, locked, write_locked;
 	attrmask_t valid_request_mask = 0;
-	struct attrlist attrs[1];
+	struct fsal_attrlist attrs[1];
 
 	fsal_prepare_attrs(attrs, ATTR_MODE | ATTR_TYPE);
 
@@ -1542,14 +1553,14 @@ void mdcache_handle_ops_init(struct fsal_obj_ops *ops)
 fsal_status_t mdcache_lookup_path(struct fsal_export *exp_hdl,
 				 const char *path,
 				 struct fsal_obj_handle **handle,
-				 struct attrlist *attrs_out)
+				 struct fsal_attrlist *attrs_out)
 {
 	struct fsal_obj_handle *sub_handle = NULL;
 	struct mdcache_fsal_export *export =
 		container_of(exp_hdl, struct mdcache_fsal_export, mfe_exp);
 	struct fsal_export *sub_export = export->mfe_exp.sub_export;
 	fsal_status_t status;
-	struct attrlist attrs;
+	struct fsal_attrlist attrs;
 	mdcache_entry_t *new_entry;
 
 	*handle = NULL;
@@ -1574,7 +1585,7 @@ fsal_status_t mdcache_lookup_path(struct fsal_export *exp_hdl,
 		return status;
 	}
 
-	status = mdcache_new_entry(export, sub_handle, &attrs, attrs_out,
+	status = mdcache_new_entry(export, sub_handle, &attrs, false, attrs_out,
 				   false, &new_entry, NULL, MDC_REASON_DEFAULT);
 
 	fsal_release_attrs(&attrs);
@@ -1613,7 +1624,7 @@ fsal_status_t mdcache_lookup_path(struct fsal_export *exp_hdl,
 fsal_status_t mdcache_create_handle(struct fsal_export *exp_hdl,
 				   struct gsh_buffdesc *fh_desc,
 				   struct fsal_obj_handle **handle,
-				   struct attrlist *attrs_out)
+				   struct fsal_attrlist *attrs_out)
 {
 	struct mdcache_fsal_export *export =
 		container_of(exp_hdl, struct mdcache_fsal_export, mfe_exp);
