@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: LGPL-3.0-or-later
 /*
  * vim:noexpandtab:shiftwidth=8:tabstop=8:
  *
@@ -51,7 +52,7 @@
  *
  * @file mdcache_hash.c
  * @author Matt Benjamin
- * @brief Cache inode hashed dictionary package
+ * @brief MDCACHE hashed dictionary package
  *
  * @section Description
  *
@@ -80,8 +81,6 @@ cih_pkginit(void)
 		&rwlock_attr,
 		PTHREAD_RWLOCK_PREFER_WRITER_NONRECURSIVE_NP);
 #endif
-	pthread_rwlockattr_destroy(&rwlock_attr);
-
 	cih_fhcache.npart = mdcache_param.nparts;
 	cih_fhcache.partition =
 		gsh_calloc(cih_fhcache.npart, sizeof(cih_partition_t));
@@ -95,6 +94,8 @@ cih_pkginit(void)
 			gsh_calloc(cih_fhcache.cache_sz,
 				sizeof(struct avltree_node *));
 	}
+
+	pthread_rwlockattr_destroy(&rwlock_attr);
 	initialized = true;
 }
 
@@ -111,7 +112,7 @@ cih_pkgdestroy(void)
 	for (ix = 0; ix < cih_fhcache.npart; ++ix) {
 		if (avltree_first(&cih_fhcache.partition[ix].t) != NULL)
 			LogMajor(COMPONENT_CACHE_INODE,
-				 "Cache inode AVL tree not empty");
+				 "MDCACHE AVL tree not empty");
 		PTHREAD_RWLOCK_destroy(&cih_fhcache.partition[ix].lock);
 		gsh_free(cih_fhcache.partition[ix].cache);
 	}
