@@ -1,3 +1,4 @@
+/* SPDX-License-Identifier: LGPL-3.0-or-later */
 /*
  * vim:noexpandtab:shiftwidth=8:tabstop=8:
  *
@@ -34,7 +35,7 @@
 
 /**
  * @file mdcache_hash.h
- * @brief Cache inode hashed dictionary package
+ * @brief MDCACHE hashed dictionary package
  *
  * This module exports an interface for efficient lookup of cache entries
  * by file handle, (etc?).  Refactored from the prior abstract HashTable
@@ -117,7 +118,7 @@ void cih_pkgdestroy(void);
  * @brief Compute cache slot for an entry
  *
  * This function computes a hash slot, taking an address modulo the
- * number of cache slotes (which should be prime).
+ * number of cache slots (which should be prime).
  *
  * @param wt [in] The table
  * @param ptr [in] Entry address
@@ -131,7 +132,7 @@ cih_cache_offsetof(struct cih_lookup_table *lt, uint64_t k)
 }
 
 /**
- * @brief Cache inode FH hashed comparison function.
+ * @brief MDCACHE FH hashed comparison function.
  *
  * Entries are ordered by integer hash first, and second by bitwise
  * comparison of the corresponding file handle.
@@ -242,10 +243,8 @@ cih_hash_release(cih_latch_t *latch)
  * @param key	[in] The key
  * @param latch [inout] Latch
  * @param flags [in] Flags
- *
- * @return true on success, false on failure
  */
-static inline bool
+static inline void
 cih_latch_entry(mdcache_key_t *key, cih_latch_t *latch, uint32_t flags,
 		const char *func, int line)
 {
@@ -263,8 +262,6 @@ cih_latch_entry(mdcache_key_t *key, cih_latch_t *latch, uint32_t flags,
 	cp->locktrace.func = (char *)func;
 	cp->locktrace.line = line;
 #endif
-
-	return true;
 }
 
 /**
@@ -288,8 +285,7 @@ cih_get_by_key_latch(mdcache_key_t *key, cih_latch_t *latch,
 	struct avltree_node *node;
 	void **cache_slot;
 
-	if (!cih_latch_entry(key, latch, flags, func, line))
-		return NULL;
+	cih_latch_entry(key, latch, flags, func, line);
 
 	k_entry.fh_hk.key = *key;
 
@@ -401,6 +397,8 @@ cih_remove_checked(mdcache_entry_t *entry)
 	PTHREAD_RWLOCK_wrlock(&cp->lock);
 	node = cih_fhcache_inline_lookup(&cp->t, &entry->fh_hk.node_k);
 	if (entry->fh_hk.inavl && node) {
+		LogFullDebug(COMPONENT_CACHE_INODE,
+			     "Unhashing entry %p", entry);
 #ifdef USE_LTTNG
 		tracepoint(mdcache, mdc_lru_remove, __func__, __LINE__,
 			   &entry->obj_handle, entry->lru.refcnt);
@@ -445,6 +443,8 @@ cih_remove_latched(mdcache_entry_t *entry, cih_latch_t *latch, uint32_t flags)
 	    cih_partition_of_scalar(&cih_fhcache, entry->fh_hk.key.hk);
 
 	if (entry->fh_hk.inavl) {
+		LogFullDebug(COMPONENT_CACHE_INODE,
+			     "Unhashing entry %p", entry);
 #ifdef USE_LTTNG
 		tracepoint(mdcache, mdc_lru_remove, __func__, __LINE__,
 			   &entry->obj_handle, entry->lru.refcnt);
