@@ -1,3 +1,4 @@
+/* SPDX-License-Identifier: LGPL-3.0-or-later */
 /*
  * vim:noexpandtab:shiftwidth=8:tabstop=8:
  *
@@ -94,7 +95,7 @@ struct mdcache_fsal_export {
 	struct fsal_up_vector up_ops;
 	/** Higher level up_ops for ops we don't consume */
 	struct fsal_up_vector super_up_ops;
-	/** The list of cache inode entries belonging to this export */
+	/** The list of cache entries belonging to this export */
 	struct glist_head entry_list;
 	/** Lock protecting entry_list */
 	pthread_rwlock_t mdc_exp_lock;
@@ -165,7 +166,7 @@ enum lru_q_id {
 
 typedef struct mdcache_lru__ {
 	struct glist_head q;	/*< Link in the physical deque
-				   impelmenting a portion of the logical
+				   implementing a portion of the logical
 				   LRU. */
 	enum lru_q_id qid;	/*< Queue identifier */
 	int32_t refcnt;		/*< Reference count.  This is signed to make
@@ -179,7 +180,7 @@ typedef struct mdcache_lru__ {
 } mdcache_lru_t;
 
 /**
- * cache inode statistics.
+ * MDCACHE statistics.
  */
 struct mdcache_stats {
 	uint64_t inode_req;
@@ -198,7 +199,7 @@ extern struct mdcache_stats *cache_stp;
  */
 
 struct entry_export_map {
-	/** The relevant cache inode entry */
+	/** The relevant cache entry */
 	mdcache_entry_t *entry;
 	/** The export the entry belongs to */
 	struct mdcache_fsal_export *exp;
@@ -269,7 +270,7 @@ static const uint32_t MDCACHE_UNREACHABLE = 0x100;
  * cache has moved into the handle as, well.  The mdcache_entry and
  * fsal_obj_handle are two parts of the same thing, a cached inode.
  * mdcache_entry holds the cache stuff and fsal_obj_handle holds the
- * stuff the the fsal has to manage, i.e. filesystem bits.
+ * stuff the fsal has to manage, i.e. filesystem bits.
  */
 
 struct mdcache_fsal_obj_handle {
@@ -280,7 +281,7 @@ struct mdcache_fsal_obj_handle {
 	/** Sub-FSAL handle */
 	struct fsal_obj_handle *sub_handle;
 	/** Cached attributes */
-	struct attrlist attrs;
+	struct fsal_attrlist attrs;
 	/** Attribute generation, increased for every write */
 	uint32_t attr_generation;
 	/** FH hash linkage */
@@ -428,7 +429,7 @@ typedef struct mdcache_dir_entry__ {
 } mdcache_dir_entry_t;
 
 /**
- * @brief Move a detached dirent to MRU postion in LRU list.
+ * @brief Move a detached dirent to MRU position in LRU list.
  *
  * @param[in]     parent  Parent entry
  * @param[in]     dirent  Dirent to move to MRU
@@ -473,8 +474,8 @@ fsal_status_t mdcache_alloc_and_check_handle(
 		struct fsal_obj_handle *sub_handle,
 		struct fsal_obj_handle **new_obj,
 		bool new_directory,
-		struct attrlist *attrs_in,
-		struct attrlist *attrs_out,
+		struct fsal_attrlist *attrs_in,
+		struct fsal_attrlist *attrs_out,
 		const char *tag,
 		mdcache_entry_t *parent,
 		const char *name,
@@ -482,12 +483,14 @@ fsal_status_t mdcache_alloc_and_check_handle(
 		struct state_t *state);
 
 fsal_status_t mdcache_refresh_attrs(mdcache_entry_t *entry, bool need_acl,
-				    bool need_fslocations, bool invalidate);
+				    bool need_fslocations, bool need_seclabel,
+				    bool invalidate);
 
 fsal_status_t mdcache_new_entry(struct mdcache_fsal_export *exp,
 				struct fsal_obj_handle *sub_handle,
-				struct attrlist *attrs_in,
-				struct attrlist *attrs_out,
+				struct fsal_attrlist *attrs_in,
+				bool prefer_attrs_in,
+				struct fsal_attrlist *attrs_out,
 				bool new_directory,
 				mdcache_entry_t **entry,
 				struct state_t *state,
@@ -500,16 +503,16 @@ fsal_status_t mdcache_find_keyed_reason(mdcache_key_t *key,
 fsal_status_t mdcache_locate_host(struct gsh_buffdesc *fh_desc,
 				  struct mdcache_fsal_export *exp,
 				  mdcache_entry_t **entry,
-				  struct attrlist *attrs_out);
+				  struct fsal_attrlist *attrs_out);
 fsal_status_t mdc_try_get_cached(mdcache_entry_t *mdc_parent, const char *name,
 				 mdcache_entry_t **entry);
 fsal_status_t mdc_lookup(mdcache_entry_t *mdc_parent, const char *name,
 			 bool uncached, mdcache_entry_t **new_entry,
-			 struct attrlist *attrs_out);
+			 struct fsal_attrlist *attrs_out);
 fsal_status_t mdc_lookup_uncached(mdcache_entry_t *mdc_parent,
 				  const char *name,
 				  mdcache_entry_t **new_entry,
-				  struct attrlist *attrs_out);
+				  struct fsal_attrlist *attrs_out);
 void mdcache_src_dest_lock(mdcache_entry_t *src, mdcache_entry_t *dest);
 void mdcache_src_dest_unlock(mdcache_entry_t *src, mdcache_entry_t *dest);
 void mdcache_dirent_remove(mdcache_entry_t *parent, const char *name);
@@ -538,7 +541,7 @@ fsal_status_t mdc_get_parent(struct mdcache_fsal_export *exp,
 		    mdcache_entry_t *entry,
 		    struct gsh_buffdesc *parent_out);
 
-void mdc_update_attr_cache(mdcache_entry_t *entry, struct attrlist *attrs);
+void mdc_update_attr_cache(mdcache_entry_t *entry, struct fsal_attrlist *attrs);
 
 static inline void mdcache_free_fh(struct gsh_buffdesc *fh_desc);
 
@@ -754,7 +757,7 @@ mdcache_free_fh(struct gsh_buffdesc *fh_desc)
  */
 
 static inline void
-mdc_fixup_md(mdcache_entry_t *entry, struct attrlist *attrs)
+mdc_fixup_md(mdcache_entry_t *entry, struct fsal_attrlist *attrs)
 {
 	uint32_t flags = 0;
 
@@ -845,7 +848,7 @@ mdcache_test_attrs_trust(mdcache_entry_t *entry, attrmask_t mask)
 	if (!test_mde_flags(entry, flags))
 		return false;
 
-	if ((entry->attrs.valid_mask & mask) != mask)
+	if ((entry->attrs.valid_mask & mask) != (mask & ~ATTR_RDATTR_ERR))
 		return false;
 
 	return true;
@@ -1016,12 +1019,12 @@ struct mdcache_readdir_state {
 fsal_status_t mdcache_lookup_path(struct fsal_export *exp_hdl,
 				 const char *path,
 				 struct fsal_obj_handle **handle,
-				 struct attrlist *attrs_out);
+				 struct fsal_attrlist *attrs_out);
 
 fsal_status_t mdcache_create_handle(struct fsal_export *exp_hdl,
 				   struct gsh_buffdesc *hdl_desc,
 				   struct fsal_obj_handle **handle,
-				   struct attrlist *attrs_out);
+				   struct fsal_attrlist *attrs_out);
 
 int mdcache_fsal_open(struct mdcache_fsal_obj_handle *, int, fsal_errors_t *);
 int mdcache_fsal_readlink(struct mdcache_fsal_obj_handle *, fsal_errors_t *);
@@ -1047,10 +1050,10 @@ fsal_status_t mdcache_open2(struct fsal_obj_handle *obj_hdl,
 			   fsal_openflags_t openflags,
 			   enum fsal_create_mode createmode,
 			   const char *name,
-			   struct attrlist *attrib_set,
+			   struct fsal_attrlist *attrib_set,
 			   fsal_verifier_t verifier,
 			   struct fsal_obj_handle **new_obj,
-			   struct attrlist *attrs_out,
+			   struct fsal_attrlist *attrs_out,
 			   bool *caller_perm_check);
 bool mdcache_check_verifier(struct fsal_obj_handle *obj_hdl,
 				     fsal_verifier_t verifier);
@@ -1127,16 +1130,15 @@ fsal_status_t mdcache_remove_extattr_by_id(struct fsal_obj_handle *obj_hdl,
 fsal_status_t mdcache_remove_extattr_by_name(struct fsal_obj_handle *obj_hdl,
 					    const char *xattr_name);
 fsal_status_t mdcache_getxattrs(struct fsal_obj_handle *obj_hdl,
-				xattrname4 *name, xattrvalue4 *value);
+				xattrkey4 *name, xattrvalue4 *value);
 fsal_status_t mdcache_setxattrs(struct fsal_obj_handle *obj_hdl,
-				setxattr_type4 type, xattrname4 *name,
+				setxattr_option4 option, xattrkey4 *name,
 				xattrvalue4 *value);
 fsal_status_t mdcache_removexattrs(struct fsal_obj_handle *obj_hdl,
-				   xattrname4 *name);
+				   xattrkey4 *name);
 fsal_status_t mdcache_listxattrs(struct fsal_obj_handle *obj_hdl,
 				 count4 len, nfs_cookie4 *cookie,
-				 verifier4 *verf, bool_t *eof,
-				 xattrlist4 *names);
+				 bool_t *eof, xattrlist4 *names);
 
 /* Handle functions */
 void mdcache_handle_ops_init(struct fsal_obj_ops *ops);
@@ -1163,8 +1165,7 @@ fsal_status_t mdcache_refresh_attrs_no_invalidate(mdcache_entry_t *entry)
 
 	PTHREAD_RWLOCK_wrlock(&entry->attr_lock);
 
-	status = mdcache_refresh_attrs(entry, false /*need_acl*/,
-				       false /*need_fslocations*/, false);
+	status = mdcache_refresh_attrs(entry, false, false, false, false);
 
 	PTHREAD_RWLOCK_unlock(&entry->attr_lock);
 
