@@ -1,3 +1,4 @@
+/* SPDX-License-Identifier: LGPL-3.0-or-later */
 /*
  * vim:noexpandtab:shiftwidth=8:tabstop=8:
  *
@@ -49,7 +50,7 @@ enum fd_states {
 /**
  * @file mdcache_lru.h
  * @author Matt Benjamin
- * @brief Constant-time cache inode cache management implementation
+ * @brief Constant-time MDCACHE cache management implementation
  *
  * @section DESCRIPTION
  *
@@ -121,16 +122,6 @@ extern pool_t *mdcache_entry_pool;
  */
 #define LRU_UNREF_QLOCKED 0x0008
 
-/**
- * entry->state_lock is held
- *
- * This will prevent cleanup on unref. The calling thread MUST hold another
- * reference that will be release without holding the state_lock (which SHOULD
- * be true in order to even be able to reference entry->state_lock), which
- * release will allow cleanup if necessary.
- */
-#define LRU_UNREF_STATE_LOCK_HELD 0x0010
-
 /**
  * The minimum reference count for a cache entry not being recycled.
  */
@@ -187,7 +178,7 @@ static inline fsal_status_t mdcache_get(mdcache_entry_t *entry)
  *
  * The result is typically to decrement the reference count on entry,
  * but additional side effects include LRU adjustment, movement
- * to/from the protected LRU partition, or recyling if the caller has
+ * to/from the protected LRU partition, or recycling if the caller has
  * raced an operation which made entry unreachable (and this current
  * caller has the last reference).  Caller MUST NOT make further
  * accesses to the memory pointed to by entry.
