@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: LGPL-3.0-or-later
 /*
  * vim:noexpandtab:shiftwidth=8:tabstop=8:
  *
@@ -54,7 +55,7 @@ struct mdcache_stats cache_st;
 struct mdcache_stats *cache_stp = &cache_st;
 
 /* FSAL name determines name of shared library: libfsal<name>.so */
-const char mdcachename[] = "MDCACHE";
+static const char mdcachename[] = "MDCACHE";
 
 /* my module private storage
  */
@@ -319,14 +320,14 @@ mdcache_fsal_unload(struct fsal_module *fsal_hdl)
 	fsal_status_t status;
 	int retval;
 
-	/* Destroy the cache inode AVL tree */
+	/* Destroy the MDCACHE AVL tree */
 	cih_pkgdestroy();
 
 	status = mdcache_lru_pkgshutdown();
 	if (FSAL_IS_ERROR(status))
 		fprintf(stderr, "MDCACHE LRU failed to shut down");
 
-	/* Destroy the cache inode entry pool */
+	/* Destroy the MDCACHE entry pool */
 	pool_destroy(mdcache_entry_pool);
 	mdcache_entry_pool = NULL;
 
@@ -361,9 +362,8 @@ void mdcache_fsal_init(void)
 /**
  * @brief Initialize the MDCACHE package.
  *
- * This shuold be called once at startup, after parsing config
+ * This should be called once at startup, after parsing config
  *
- * @param[in] parm	Parameter description
  * @return FSAL status
  */
 fsal_status_t mdcache_pkginit(void)
