@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: LGPL-3.0-or-later
 /*
  * vim:noexpandtab:shiftwidth=8:tabstop=8:
  *
@@ -25,13 +26,13 @@
  */
 
 /**
- * @addtogroup cache_inode
+ * @addtogroup mdcache
  * @{
  */
 
 /**
- * @file cache_inode_read_conf.c
- * @brief Cache inode configuration parameter tables.
+ * @file mdcache_read_conf.c
+ * @brief MDCACHE configuration parameter tables.
  */
 #include "config.h"
 #include "log.h"
@@ -47,7 +48,7 @@
 #include <pthread.h>
 #include <string.h>
 
-/** File cache configuration, settable in the CacheInode
+/** File cache configuration, settable in the CacheInode/MDCACHE
     stanza. */
 
 struct mdcache_parameter mdcache_param;
@@ -100,18 +101,10 @@ static void *mdcache_param_init(void *link_mem, void *self_struct)
 		return NULL;
 }
 
-struct config_block cache_inode_param_blk = {
-	.dbus_interface_name = "org.ganesha.nfsd.config.cache_inode",
-	.blk_desc.name = "CacheInode",
-	.blk_desc.type = CONFIG_BLOCK,
-	.blk_desc.u.blk.init = mdcache_param_init,
-	.blk_desc.u.blk.params = mdcache_params,
-	.blk_desc.u.blk.commit = noop_conf_commit
-};
-
 struct config_block mdcache_param_blk = {
 	.dbus_interface_name = "org.ganesha.nfsd.config.cache_inode",
 	.blk_desc.name = "MDCACHE",
+	.blk_desc.altname = "CacheInode",
 	.blk_desc.type = CONFIG_BLOCK,
 	.blk_desc.u.blk.init = mdcache_param_init,
 	.blk_desc.u.blk.params = mdcache_params,
@@ -121,17 +114,6 @@ struct config_block mdcache_param_blk = {
 int mdcache_set_param_from_conf(config_file_t parse_tree,
 				struct config_error_type *err_type)
 {
-	(void) load_config_from_parse(parse_tree,
-				      &cache_inode_param_blk,
-				      NULL,
-				      true,
-				      err_type);
-	if (!config_error_is_harmless(err_type)) {
-		LogCrit(COMPONENT_INIT,
-			"Error while parsing CACHEINODE specific configuration");
-		return -1;
-	}
-
 	(void) load_config_from_parse(parse_tree,
 				      &mdcache_param_blk,
 				      NULL,
