@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: LGPL-3.0-or-later
 /*
  * vim:noexpandtab:shiftwidth=8:tabstop=8:
  *
@@ -46,13 +47,15 @@ mdc_up_invalidate(const struct fsal_up_vector *vec, struct gsh_buffdesc *handle,
 {
 	mdcache_entry_t *entry;
 	fsal_status_t status;
-	struct req_op_context *save_ctx, req_ctx = {0};
+	struct req_op_context op_context;
 	mdcache_key_t key;
 
-	req_ctx.ctx_export = vec->up_gsh_export;
-	req_ctx.fsal_export = vec->up_fsal_export;
-	save_ctx = op_ctx;
-	op_ctx = &req_ctx;
+	/* Get a ref to the vec->up_gsh_export and initialize op_context for the
+	 * upcall
+	 */
+	get_gsh_export_ref(vec->up_gsh_export);
+	init_op_context_simple(&op_context, vec->up_gsh_export,
+			       vec->up_fsal_export);
 
 	key.fsal = vec->up_fsal_export->sub_export->fsal;
 	cih_hash_key(&key, vec->up_fsal_export->sub_export->fsal, handle,
@@ -85,7 +88,8 @@ mdc_up_invalidate(const struct fsal_up_vector *vec, struct gsh_buffdesc *handle,
 	mdcache_put(entry);
 
 out:
-	op_ctx = save_ctx;
+
+	release_op_context();
 	return status;
 }
 
@@ -163,13 +167,13 @@ mdc_up_try_release(const struct fsal_up_vector *vec,
 
 static fsal_status_t
 mdc_up_update(const struct fsal_up_vector *vec, struct gsh_buffdesc *handle,
-	      struct attrlist *attr, uint32_t flags)
+	      struct fsal_attrlist *attr, uint32_t flags)
 {
 	mdcache_entry_t *entry;
 	fsal_status_t status;
 	/* Have necessary changes been made? */
 	bool mutatis_mutandis = false;
-	struct req_op_context *save_ctx, req_ctx = {0};
+	struct req_op_context op_context;
 	mdcache_key_t key;
 	attrmask_t mask_set = 0;
 
@@ -192,10 +196,12 @@ mdc_up_update(const struct fsal_up_vector *vec, struct gsh_buffdesc *handle,
 		return fsalstat(ERR_FSAL_INVAL, 0);
 	}
 
-	req_ctx.ctx_export = vec->up_gsh_export;
-	req_ctx.fsal_export = vec->up_fsal_export;
-	save_ctx = op_ctx;
-	op_ctx = &req_ctx;
+	/* Get a ref to the vec->up_gsh_export and initialize op_context for the
+	 * upcall
+	 */
+	get_gsh_export_ref(vec->up_gsh_export);
+	init_op_context_simple(&op_context, vec->up_gsh_export,
+			       vec->up_fsal_export);
 
 	key.fsal = vec->up_fsal_export->sub_export->fsal;
 	cih_hash_key(&key, vec->up_fsal_export->sub_export->fsal, handle,
@@ -395,7 +401,8 @@ mdc_up_update(const struct fsal_up_vector *vec, struct gsh_buffdesc *handle,
 put:
 	mdcache_put(entry);
 out:
-	op_ctx = save_ctx;
+
+	release_op_context();
 	return status;
 }
 
@@ -440,16 +447,18 @@ state_status_t mdc_up_lock_grant(const struct fsal_up_vector *vec,
 {
 	struct mdcache_fsal_export *myself = mdc_export(vec->up_fsal_export);
 	state_status_t rc;
-	struct root_op_context root_ctx;
+	struct req_op_context op_context;
 
-	/* Initialize op context */
-	init_root_op_context(&root_ctx, vec->up_gsh_export,
-		vec->up_fsal_export, 0, 0, UNKNOWN_REQUEST);
+	/* Get a ref to the vec->up_gsh_export and initialize op_context for the
+	 * upcall
+	 */
+	get_gsh_export_ref(vec->up_gsh_export);
+	init_op_context_simple(&op_context, vec->up_gsh_export,
+			       vec->up_fsal_export);
 
-	rc = myself->super_up_ops.lock_grant(vec, file, owner,
-					     lock_param);
+	rc = myself->super_up_ops.lock_grant(vec, file, owner, lock_param);
 
-	release_root_op_context();
+	release_op_context();
 
 	return rc;
 }
@@ -471,16 +480,19 @@ state_status_t mdc_up_lock_avail(const struct fsal_up_vector *vec,
 {
 	struct mdcache_fsal_export *myself = mdc_export(vec->up_fsal_export);
 	state_status_t rc;
-	struct root_op_context root_ctx;
+	struct req_op_context op_context;
 
 	/* Initialize op context */
-	init_root_op_context(&root_ctx, vec->up_gsh_export,
-		vec->up_fsal_export, 0, 0, UNKNOWN_REQUEST);
+	/* Get a ref to the vec->up_gsh_export and initialize op_context for the
+	 * upcall
+	 */
+	get_gsh_export_ref(vec->up_gsh_export);
+	init_op_context_simple(&op_context, vec->up_gsh_export,
+			       vec->up_fsal_export);
 
-	rc = myself->super_up_ops.lock_avail(vec, file, owner,
-					     lock_param);
+	rc = myself->super_up_ops.lock_avail(vec, file, owner, lock_param);
 
-	release_root_op_context();
+	release_op_context();
 
 	return rc;
 }
@@ -511,17 +523,19 @@ state_status_t mdc_up_layoutrecall(const struct fsal_up_vector *vec,
 {
 	struct mdcache_fsal_export *myself = mdc_export(vec->up_fsal_export);
 	state_status_t rc;
-	struct req_op_context *save_ctx, req_ctx = {0};
+	struct req_op_context op_context;
 
-	req_ctx.ctx_export = vec->up_gsh_export;
-	req_ctx.fsal_export = vec->up_fsal_export;
-	save_ctx = op_ctx;
-	op_ctx = &req_ctx;
+	/* Get a ref to the vec->up_gsh_export and initialize op_context for the
+	 * upcall
+	 */
+	get_gsh_export_ref(vec->up_gsh_export);
+	init_op_context_simple(&op_context, vec->up_gsh_export,
+			       vec->up_fsal_export);
 
 	rc = myself->super_up_ops.layoutrecall(vec, handle, layout_type,
 					       changed, segment, cookie, spec);
 
-	op_ctx = save_ctx;
+	release_op_context();
 
 	return rc;
 }
@@ -531,23 +545,25 @@ state_status_t mdc_up_layoutrecall(const struct fsal_up_vector *vec,
  * Pass to upper layer
  *
  * @param[in] vec	Up ops vector
- * @param[in] handle Handle on which the delegation is held
+ * @param[in] handle    The handle on which the delegation is held
  */
 state_status_t mdc_up_delegrecall(const struct fsal_up_vector *vec,
 				  struct gsh_buffdesc *handle)
 {
 	struct mdcache_fsal_export *myself = mdc_export(vec->up_fsal_export);
 	state_status_t rc;
-	struct req_op_context *save_ctx, req_ctx = {0};
+	struct req_op_context op_context;
 
-	req_ctx.ctx_export = vec->up_gsh_export;
-	req_ctx.fsal_export = vec->up_fsal_export;
-	save_ctx = op_ctx;
-	op_ctx = &req_ctx;
+	/* Get a ref to the vec->up_gsh_export and initialize op_context for the
+	 * upcall
+	 */
+	get_gsh_export_ref(vec->up_gsh_export);
+	init_op_context_simple(&op_context, vec->up_gsh_export,
+			       vec->up_fsal_export);
 
 	rc = myself->super_up_ops.delegrecall(vec, handle);
 
-	op_ctx = save_ctx;
+	release_op_context();
 
 	return rc;
 }
