@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: LGPL-3.0-or-later
 /*
  * vim:noexpandtab:shiftwidth=8:tabstop=8:
  *
@@ -288,7 +289,7 @@ fsal_status_t mdcache_remove_extattr_by_name(struct fsal_obj_handle *obj_hdl,
  * @return FSAL status
  */
 fsal_status_t mdcache_getxattrs(struct fsal_obj_handle *obj_hdl,
-				xattrname4 *name, xattrvalue4 *value)
+				xattrkey4 *name, xattrvalue4 *value)
 {
 	struct mdcache_fsal_obj_handle *handle =
 		container_of(obj_hdl, struct mdcache_fsal_obj_handle,
@@ -315,7 +316,7 @@ fsal_status_t mdcache_getxattrs(struct fsal_obj_handle *obj_hdl,
  * @return FSAL status
  */
 fsal_status_t mdcache_setxattrs(struct fsal_obj_handle *obj_hdl,
-				setxattr_type4 type, xattrname4 *name,
+				setxattr_option4 option, xattrkey4 *name,
 				xattrvalue4 *value)
 {
 	struct mdcache_fsal_obj_handle *handle =
@@ -325,7 +326,7 @@ fsal_status_t mdcache_setxattrs(struct fsal_obj_handle *obj_hdl,
 
 	subcall(
 		status = handle->sub_handle->obj_ops->setxattrs(
-			handle->sub_handle, type, name, value)
+			handle->sub_handle, option, name, value)
 	       );
 
 	return status;
@@ -341,7 +342,7 @@ fsal_status_t mdcache_setxattrs(struct fsal_obj_handle *obj_hdl,
  * @return FSAL status
  */
 fsal_status_t mdcache_removexattrs(struct fsal_obj_handle *obj_hdl,
-				   xattrname4 *name)
+				   xattrkey4 *name)
 {
 	struct mdcache_fsal_obj_handle *handle =
 		container_of(obj_hdl, struct mdcache_fsal_obj_handle,
@@ -363,7 +364,7 @@ fsal_status_t mdcache_removexattrs(struct fsal_obj_handle *obj_hdl,
  *
  * @param[in] obj_hdl	File to search
  * @param[in] len	Length of names buffer
- * @param[in,out] cookie cookie for list
+ * @param[in,out] cookie The cookie for list
  * @param[in,out] verf	cookie verifier
  * @param[out] eof	set if no more extended attributes
  * @param[out] names	list of extended attribute names
@@ -371,8 +372,7 @@ fsal_status_t mdcache_removexattrs(struct fsal_obj_handle *obj_hdl,
  */
 fsal_status_t mdcache_listxattrs(struct fsal_obj_handle *obj_hdl,
 				 count4 len, nfs_cookie4 *cookie,
-				 verifier4 *verf, bool_t *eof,
-				 xattrlist4 *names)
+				 bool_t *eof, xattrlist4 *names)
 {
 	struct mdcache_fsal_obj_handle *handle =
 		container_of(obj_hdl, struct mdcache_fsal_obj_handle,
@@ -381,7 +381,7 @@ fsal_status_t mdcache_listxattrs(struct fsal_obj_handle *obj_hdl,
 
 	subcall(
 		status = handle->sub_handle->obj_ops->listxattrs(
-			handle->sub_handle, len, cookie, verf, eof, names)
+			handle->sub_handle, len, cookie, eof, names)
 	       );
 
 	return status;
