@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: LGPL-3.0-or-later
 /*
  * vim:noexpandtab:shiftwidth=8:tabstop=8:
  *
@@ -76,8 +77,6 @@ void null_async_cb(struct fsal_obj_handle *obj, fsal_status_t ret,
 
 /* nullfs_close
  * Close the file if it is still open.
- * Yes, we ignor lock status.  Closing a file in POSIX
- * releases all locks but that is state and cache inode's problem.
  */
 
 fsal_status_t nullfs_close(struct fsal_obj_handle *obj_hdl)
@@ -104,10 +103,10 @@ fsal_status_t nullfs_open2(struct fsal_obj_handle *obj_hdl,
 			   fsal_openflags_t openflags,
 			   enum fsal_create_mode createmode,
 			   const char *name,
-			   struct attrlist *attrs_in,
+			   struct fsal_attrlist *attrs_in,
 			   fsal_verifier_t verifier,
 			   struct fsal_obj_handle **new_obj,
-			   struct attrlist *attrs_out,
+			   struct fsal_attrlist *attrs_out,
 			   bool *caller_perm_check)
 {
 	struct nullfs_fsal_obj_handle *handle =
