@@ -202,15 +202,16 @@ void fsal_pnfs_ds_init(struct fsal_pnfs_ds *pds, struct fsal_module *fsal)
 
 void fsal_pnfs_ds_fini(struct fsal_pnfs_ds *pds)
 {
+	assert(pds->fsal);
+
 	PTHREAD_RWLOCK_wrlock(&pds->fsal->lock);
 	glist_del(&pds->server);
 	PTHREAD_RWLOCK_unlock(&pds->fsal->lock);
 
 	memset(&pds->s_ops, 0, sizeof(pds->s_ops));	/* poison myself */
-	if (pds->fsal != NULL) {
-		fsal_put(pds->fsal);
-		pds->fsal = NULL;
-	}
+
+	fsal_put(pds->fsal);
+	pds->fsal = NULL;
 }
 
 /**
@@ -655,14 +656,7 @@ fsal_errors_t fsal_inherit_acls(struct fsal_attrlist *attrs, fsal_acl_t *sacl,
 		 * path changes that assumption, let's not release the
 		 * old ACL properly.
 		 */
-		int acl_status;
-
-		acl_status = nfs4_acl_release_entry(attrs->acl);
-
-		if (acl_status != NFS_V4_ACL_SUCCESS)
-			LogCrit(COMPONENT_FSAL,
-				"Failed to release old acl, status=%d",
-				acl_status);
+		nfs4_acl_release_entry(attrs->acl);
 	}
 
 	attrs->acl = nfs4_acl_alloc();
@@ -875,14 +869,7 @@ fsal_mode_gen_acl(struct fsal_attrlist *attrs)
 		 * path changes that assumption, let's not release the
 		 * old ACL properly.
 		 */
-		int acl_status;
-
-		acl_status = nfs4_acl_release_entry(attrs->acl);
-
-		if (acl_status != NFS_V4_ACL_SUCCESS)
-			LogCrit(COMPONENT_FSAL,
-				"Failed to release old acl, status=%d",
-				acl_status);
+		nfs4_acl_release_entry(attrs->acl);
 	}
 
 	attrs->acl = nfs4_acl_alloc();
@@ -940,14 +927,7 @@ fsal_status_t fsal_mode_to_acl(struct fsal_attrlist *attrs, fsal_acl_t *sacl)
 		 * path changes that assumption, let's not release the
 		 * old ACL properly.
 		 */
-		int acl_status;
-
-		acl_status = nfs4_acl_release_entry(attrs->acl);
-
-		if (acl_status != NFS_V4_ACL_SUCCESS)
-			LogCrit(COMPONENT_FSAL,
-				"Failed to release old acl, status=%d",
-				acl_status);
+		nfs4_acl_release_entry(attrs->acl);
 	}
 
 	attrs->acl = nfs4_acl_alloc();
@@ -1699,6 +1679,8 @@ fsal_status_t fsal_find_fd(struct fsal_fd **out_fd,
 {
 	fsal_status_t status = {ERR_FSAL_NO_ERROR, 0};
 	struct fsal_fd *state_fd;
+	struct fsal_fd *related_fd;
+	struct state_t *openstate;
 
 	if (state == NULL)
 		goto global;
@@ -1741,17 +1723,24 @@ fsal_status_t fsal_find_fd(struct fsal_fd **out_fd,
 
 		if (status.major == ERR_FSAL_ACCESS &&
 		   (state->state_type == STATE_TYPE_LOCK ||
-		    state->state_type == STATE_TYPE_NLM_LOCK) &&
-		    state->state_data.lock.openstate != NULL) {
-			/* Got an EACCESS and openstate is available, try
-			 * again with it's openflags.
-			 */
-			struct fsal_fd *related_fd = (struct fsal_fd *)
-					(state->state_data.lock.openstate + 1);
+		    state->state_type == STATE_TYPE_NLM_LOCK)) {
+			openstate = nfs4_State_Get_Pointer(
+				state->state_data.lock.openstate_key);
 
-			openflags = related_fd->openflags & FSAL_O_RDWR;
+			if (openstate != NULL) {
+				/* Got an EACCESS and openstate is available,
+				* try again with it's openflags.
+				*/
+				related_fd =
+					(struct fsal_fd *)(openstate + 1);
 
-			status = open_func(obj_hdl, openflags, state_fd);
+				openflags = related_fd->openflags & FSAL_O_RDWR;
+
+				dec_state_t_ref(openstate);
+
+				status = open_func(
+				    obj_hdl, openflags, state_fd);
+			}
 		}
 
 		if (FSAL_IS_ERROR(status)) {
@@ -1774,11 +1763,17 @@ fsal_status_t fsal_find_fd(struct fsal_fd **out_fd,
 	 * fd (this will support FSALs that have an open file per open state
 	 * but don't bother with opening a separate file for the lock state).
 	 */
-	if ((state->state_type == STATE_TYPE_LOCK ||
-	     state->state_type == STATE_TYPE_NLM_LOCK) &&
-	    state->state_data.lock.openstate != NULL) {
-		struct fsal_fd *related_fd = (struct fsal_fd *)
-				(state->state_data.lock.openstate + 1);
+	if (state->state_type == STATE_TYPE_LOCK ||
+	     state->state_type == STATE_TYPE_NLM_LOCK) {
+		openstate = nfs4_State_Get_Pointer(
+			state->state_data.lock.openstate_key);
+
+		if (!openstate)
+			goto global;
+
+		related_fd = (struct fsal_fd *)(openstate + 1);
+
+		dec_state_t_ref(openstate);
 
 		LogFullDebug(COMPONENT_FSAL,
 			     "related_fd->openflags = %d openflags = %d",
