@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: LGPL-3.0-or-later
 /*
  * posix_acls.c
  * vim:noexpandtab:shiftwidth=8:tabstop=8:
@@ -27,7 +28,6 @@
  *       URL: http://tools.ietf.org/html/draft-ietf-nfsv4-acl-mapping-05
  */
 
-
 #include "posix_acls.h"
 
 /* Checks whether ACE belongs to effective acl (ACCESS TYPE) */
@@ -138,8 +138,7 @@ acl_entry_t find_entry(acl_t acl, acl_tag_t tag,  unsigned int id)
 		}
 		if (tag == entryTag) {
 			if (tag == ACL_USER || tag == ACL_GROUP)
-				if (id != *(unsigned int *)
-						acl_get_qualifier(entry))
+				if (id != posix_acl_get_uid(entry))
 					continue;
 			break;
 		}
@@ -173,7 +172,13 @@ acl_entry_t get_entry(acl_t acl, acl_tag_t tag, unsigned int id)
 		ret = acl_set_tag_type(entry, tag);
 		if (ret)
 			LogWarn(COMPONENT_FSAL, "Cannot set tag for Entry");
-		ret = acl_set_qualifier(entry, &id);
+		if (tag == ACL_USER || tag == ACL_GROUP) {
+			ret = acl_set_qualifier(entry, &id);
+			if (ret) {
+				LogWarn(COMPONENT_FSAL, "Failed to set id");
+				return NULL;
+			}
+		}
 	}
 
 	return entry;
@@ -240,11 +245,11 @@ int posix_acl_2_fsal_acl(acl_t p_posixacl, bool is_dir, bool is_inherit,
 
 	/* *
 	 * Converts each entry in posix acl into fsal_ace by filling type, flag,
-	 * perm, iflag, flag and who(uid, gid) appropiately
+	 * perm, iflag, flag and who(uid, gid) appropriately
 	 *
-	 * Corresponding to each posix acl entry, there is a possiblity of two
+	 * Corresponding to each posix acl entry, there is a possibility of two
 	 * fsal_aces, it can either be ALLOW or DENY. The DENY added to list
-	 * depending on the permission set set of other entries.
+	 * depending on the permission set of other entries.
 	 *
 	 * Here both entries are created for a posix acl entry and filled up
 	 * correspondingly. Then at the end unnecessary DENY entries are removed
@@ -263,10 +268,6 @@ int posix_acl_2_fsal_acl(acl_t p_posixacl, bool is_dir, bool is_inherit,
 			continue;
 		}
 
-		/* Mask is not converted to a fsal_acl entry , skipping */
-		if (tag == ACL_MASK)
-			continue;
-
 		pace_deny->type = FSAL_ACE_TYPE_DENY;
 		pace_allow->type = FSAL_ACE_TYPE_ALLOW;
 
@@ -298,14 +299,20 @@ int posix_acl_2_fsal_acl(acl_t p_posixacl, bool is_dir, bool is_inherit,
 			break;
 		case  ACL_USER:
 			pace_allow->who.uid = pace_deny->who.uid =
-					*(uid_t *)acl_get_qualifier(entry);
+					posix_acl_get_uid(entry);
 			break;
 		case  ACL_GROUP:
 			pace_allow->who.gid = pace_deny->who.gid =
-					*(gid_t *)acl_get_qualifier(entry);
+					posix_acl_get_gid(entry);
 			pace_allow->flag = pace_deny->flag |=
 						FSAL_ACE_FLAG_GROUP_ID;
 			break;
+		case  ACL_MASK:
+			pace_allow->who.uid = pace_deny->who.uid =
+						FSAL_ACE_SPECIAL_MASK;
+			pace_allow->iflag = pace_deny->iflag =
+						FSAL_ACE_IFLAG_SPECIAL_ID;
+			break;
 		default:
 			LogWarn(COMPONENT_FSAL, "Invalid tag for the acl");
 		}
@@ -337,6 +344,10 @@ int posix_acl_2_fsal_acl(acl_t p_posixacl, bool is_dir, bool is_inherit,
 		if (acl_get_perm(p_permset, ACL_READ)) {
 			if (tag == ACL_USER_OBJ || tag == ACL_OTHER || readmask)
 				pace_allow->perm |= FSAL_ACE_PERM_READ_DATA;
+			if ((tag == ACL_USER || tag == ACL_GROUP ||
+					tag == ACL_GROUP_OBJ) && (!readmask))
+				pace_allow->iflag |=
+						FSAL_ACE_FLAG_MASK_READ_DENY;
 		} else
 			readcurrent = false;
 
@@ -351,6 +362,10 @@ int posix_acl_2_fsal_acl(acl_t p_posixacl, bool is_dir, bool is_inherit,
 			if (is_dir)
 				pace_allow->perm |=
 					FSAL_ACE_PERM_DELETE_CHILD;
+			if ((tag == ACL_USER || tag == ACL_GROUP ||
+					tag == ACL_GROUP_OBJ) && (!writemask))
+				pace_allow->iflag |=
+					FSAL_ACE_FLAG_MASK_WRITE_DENY;
 		} else
 			writecurrent = false;
 
@@ -358,6 +373,10 @@ int posix_acl_2_fsal_acl(acl_t p_posixacl, bool is_dir, bool is_inherit,
 			if (tag == ACL_USER_OBJ || tag == ACL_OTHER ||
 						executemask)
 				pace_allow->perm |= FSAL_ACE_PERM_EXECUTE;
+			if ((tag == ACL_USER || tag == ACL_GROUP ||
+					tag == ACL_GROUP_OBJ) && (!executemask))
+				pace_allow->iflag |=
+					FSAL_ACE_FLAG_MASK_EXECUTE_DENY;
 		} else
 			executecurrent = false;
 
@@ -480,9 +499,8 @@ acl_t fsal_acl_2_posix_acl(fsal_acl_t *p_fsalacl, acl_type_t type)
 	acl_entry_t a_entry, d_entry;
 	acl_permset_t a_permset, e_a_permset, d_permset, e_d_permset;
 	acl_tag_t tag = -1;
-	char *acl_str;
 	unsigned int id;
-	bool mask = false;
+	bool mask = false, mask_set = false;
 	bool deny_e_r = false, deny_e_w = false, deny_e_x = false;
 
 
@@ -521,6 +539,10 @@ acl_t fsal_acl_2_posix_acl(fsal_acl_t *p_fsalacl, acl_type_t type)
 		LogWarn(COMPONENT_FSAL, "Cannot set tag for ACL Entry");
 
 	ret = acl_get_permset(a_entry, &e_a_permset);
+	if (ret) {
+		LogWarn(COMPONENT_FSAL,
+			"Cannot retrieve permission set");
+	}
 
 	/*
 	 * Deny entry for @EVERYONE created only because it will ease the
@@ -536,6 +558,10 @@ acl_t fsal_acl_2_posix_acl(fsal_acl_t *p_fsalacl, acl_type_t type)
 		LogWarn(COMPONENT_FSAL, "Cannot set tag for ACL Entry");
 
 	ret = acl_get_permset(d_entry, &e_d_permset);
+	if (ret) {
+		LogWarn(COMPONENT_FSAL,
+			"Cannot retrieve permission set");
+	}
 
 	for (f_ace = p_fsalacl->aces;
 		f_ace < p_fsalacl->aces + p_fsalacl->naces; f_ace++) {
@@ -587,7 +613,7 @@ acl_t fsal_acl_2_posix_acl(fsal_acl_t *p_fsalacl, acl_type_t type)
 	if (ret)
 		LogWarn(COMPONENT_FSAL, "Cannot set tag for ACL Entry");
 
-	/** @todo: Annoymous users/groups (id = -1) should handle properly
+	/** @todo: Anonymous users/groups (id = -1) should handle properly
 	 */
 
 	/*
@@ -615,6 +641,8 @@ acl_t fsal_acl_2_posix_acl(fsal_acl_t *p_fsalacl, acl_type_t type)
 				tag = ACL_USER_OBJ;
 			if (IS_FSAL_ACE_SPECIAL_GROUP(*f_ace))
 				tag = ACL_GROUP_OBJ;
+			if (IS_FSAL_ACE_SPECIAL_MASK(*f_ace))
+				tag = ACL_MASK;
 		} else {
 			id = GET_FSAL_ACE_WHO(*f_ace);
 			if (IS_FSAL_ACE_GROUP_ID(*f_ace))
@@ -643,6 +671,10 @@ acl_t fsal_acl_2_posix_acl(fsal_acl_t *p_fsalacl, acl_type_t type)
 		a_entry = get_entry(allow_acl, tag, id);
 		d_entry = get_entry(deny_acl, tag, id);
 		ret = acl_get_permset(d_entry, &d_permset);
+		if (ret) {
+			LogWarn(COMPONENT_FSAL,
+				"Cannot retrieve permission set");
+		}
 
 		if (IS_FSAL_ACE_DENY(*f_ace)) {
 			if (IS_FSAL_ACE_READ_DATA(*f_ace))
@@ -653,20 +685,41 @@ acl_t fsal_acl_2_posix_acl(fsal_acl_t *p_fsalacl, acl_type_t type)
 				acl_add_perm(d_permset, ACL_EXECUTE);
 		}
 		ret = acl_get_permset(a_entry, &a_permset);
+		if (ret) {
+			LogWarn(COMPONENT_FSAL,
+				"Cannot retrieve permission set");
+		}
 
-		if (isallow(f_ace, e_a_permset, ACL_READ)
+		if (IS_FSAL_ACE_SPECIAL_MASK(*f_ace)) {
+			if (IS_FSAL_ACE_ALLOW(*f_ace)) {
+				if IS_FSAL_ACE_READ_DATA(*f_ace)
+					acl_add_perm(a_permset, ACL_READ);
+				if IS_FSAL_ACE_WRITE_DATA(*f_ace)
+					acl_add_perm(a_permset, ACL_WRITE);
+				if IS_FSAL_ACE_EXECUTE(*f_ace)
+					acl_add_perm(a_permset, ACL_EXECUTE);
+			}
+			mask_set = true;
+			continue;
+		}
+
+		if ((isallow(f_ace, e_a_permset, ACL_READ)
 		    && !isdeny(d_permset, e_d_permset, ACL_READ))
+		    || IS_FSAL_ACE_IFLAG(*f_ace, FSAL_ACE_FLAG_MASK_READ_DENY))
 			acl_add_perm(a_permset, ACL_READ);
 
-		if (isallow(f_ace, e_a_permset, ACL_WRITE)
+		if ((isallow(f_ace, e_a_permset, ACL_WRITE)
 		    && !isdeny(d_permset, e_d_permset, ACL_WRITE))
+		    || IS_FSAL_ACE_IFLAG(*f_ace, FSAL_ACE_FLAG_MASK_WRITE_DENY))
 			acl_add_perm(a_permset, ACL_WRITE);
 
-		if (isallow(f_ace, e_a_permset, ACL_EXECUTE)
+		if ((isallow(f_ace, e_a_permset, ACL_EXECUTE)
 		    && !isdeny(d_permset, e_d_permset, ACL_EXECUTE))
+		    || IS_FSAL_ACE_IFLAG(*f_ace,
+				FSAL_ACE_FLAG_MASK_EXECUTE_DENY))
 			acl_add_perm(a_permset, ACL_EXECUTE);
 	}
-	if (mask) {
+	if (!mask_set && mask) {
 		ret = acl_calc_mask(&allow_acl);
 		if (ret)
 			LogWarn(COMPONENT_FSAL,
@@ -687,13 +740,257 @@ acl_t fsal_acl_2_posix_acl(fsal_acl_t *p_fsalacl, acl_type_t type)
 
 	}
 
-	acl_str = acl_to_any_text(allow_acl, NULL, ',',
+	if (isDebug(COMPONENT_FSAL)) {
+		char *acl_str;
+
+		acl_str = acl_to_any_text(allow_acl, NULL, ',',
 				TEXT_ABBREVIATE | TEXT_NUMERIC_IDS);
-	LogDebug(COMPONENT_FSAL, "posix acl = %s ", acl_str);
+		LogDebug(COMPONENT_FSAL, "posix acl = %s ", acl_str);
+		acl_free(acl_str);
+	}
 
-	acl_free(acl_str);
 	if (deny_acl)
 		acl_free(deny_acl);
 
 	return allow_acl;
 }
+
+/*
+ * @brief Calculate the ACL xattr size by quantity of ACL entries
+ *
+ * @param[in]  count Quantity of ACL entries
+ *
+ * @return ACL xattr size
+ */
+
+size_t posix_acl_xattr_size(int count)
+{
+	return sizeof(struct acl_ea_header) +
+	    count * sizeof(struct acl_ea_entry);
+}
+
+/*
+ * @brief Calculate the quantity of ACL entries by xattr size
+ *
+ * @param[in]  size Length of extended attribute
+ *
+ * @return Quantity of ACL entries on success, -1 on failure.
+ */
+
+int posix_acl_entries_count(size_t size)
+{
+	if (size < sizeof(struct acl_ea_header))
+		return -1;
+	size -= sizeof(struct acl_ea_header);
+	if (size % sizeof(struct acl_ea_entry))
+		return -1;
+	return size / sizeof(struct acl_ea_entry);
+}
+
+/*
+ * @brief Convert extended attribute to POSIX ACL
+ *
+ * @param[in]  value	Extended attribute
+ * @param[in]  size	Size of the extended attribute
+ *
+ * @return Posix ACL on success, NULL on failure.
+ */
+
+acl_t xattr_2_posix_acl(const struct acl_ea_header *ea_header, size_t size)
+{
+	const struct acl_ea_entry *ea_entry = &ea_header->a_entries[0], *end;
+
+	int count;
+	int ret = 0;
+	acl_t acl = NULL;
+	acl_entry_t acl_entry;
+	acl_tag_t tag;
+	acl_permset_t permset;
+	uid_t uid;
+	gid_t gid;
+
+	count = posix_acl_entries_count(size);
+	if (count < 0) {
+		LogMajor(COMPONENT_FSAL,
+			"Invalid parameter: size = %d", (int)size);
+		return NULL;
+	}
+
+	if (count == 0)
+		return NULL;
+
+	if (ea_header->a_version != htole32(ACL_EA_VERSION)) {
+		LogMajor(COMPONENT_FSAL, "ACL ea version is inconsistent");
+		return NULL;
+	}
+
+	acl = acl_init(count);
+	if (!acl) {
+		LogMajor(COMPONENT_FSAL,
+			"Failed to ACL INIT: count = %d", count);
+		return NULL;
+	}
+
+	for (end = ea_entry + count; ea_entry != end; ea_entry++) {
+		ret = acl_create_entry(&acl, &acl_entry);
+		if (ret) {
+			LogMajor(COMPONENT_FSAL, "Failed to create acl entry");
+			goto out;
+		}
+
+		tag = le16toh(ea_entry->e_tag);
+		ret = acl_set_tag_type(acl_entry, tag);
+		if (ret) {
+			LogMajor(COMPONENT_FSAL, "Failed to set acl tag type");
+			goto out;
+		}
+
+		ret = acl_get_permset(acl_entry, &permset);
+		if (ret) {
+			LogWarn(COMPONENT_FSAL, "Failed to get acl permset");
+			goto out;
+		}
+
+		ret = acl_add_perm(permset, le16toh(ea_entry->e_perm));
+		if (ret) {
+			LogWarn(COMPONENT_FSAL, "Failed to add acl permission");
+			goto out;
+		}
+
+		switch (tag) {
+		case ACL_USER_OBJ:
+		case ACL_GROUP_OBJ:
+		case ACL_MASK:
+		case ACL_OTHER:
+			break;
+
+		case ACL_USER:
+			uid = le32toh(ea_entry->e_id);
+			ret = acl_set_qualifier(acl_entry, &uid);
+			if (ret) {
+				LogMajor(COMPONENT_FSAL, "Failed to set uid");
+				goto out;
+			}
+
+			break;
+
+		case ACL_GROUP:
+			gid = le32toh(ea_entry->e_id);
+			ret = acl_set_qualifier(acl_entry, &gid);
+			if (ret) {
+				LogMajor(COMPONENT_FSAL, "Failed to set gid");
+				goto out;
+			}
+
+			break;
+
+		default:
+			goto out;
+		}
+	}
+
+	if (isDebug(COMPONENT_FSAL)) {
+		char *acl_str;
+
+		acl_str = acl_to_any_text(acl, NULL, ',',
+				TEXT_ABBREVIATE | TEXT_NUMERIC_IDS);
+		LogDebug(COMPONENT_FSAL, "posix acl = %s ", acl_str);
+		acl_free(acl_str);
+	}
+
+	return acl;
+
+out:
+	if (acl) {
+		acl_free((void *)acl);
+	}
+
+	return NULL;
+}
+
+/*
+ * @brief Convert POSIX ACL to extended attribute
+ *
+ * @param[in]  acl	Posix ACL
+ * @param[in]  size	Size of the extended attribute buffer
+ * @param[out] buf	Buffer of the extended attribute
+ *
+ * @return Real size of extended attribute on success, -1 on failure.
+ */
+
+int posix_acl_2_xattr(acl_t acl, void *buf, size_t size)
+{
+	struct acl_ea_header *ea_header = buf;
+	struct acl_ea_entry *ea_entry;
+	acl_entry_t acl_entry;
+	acl_tag_t tag;
+	acl_permset_t permset;
+	int real_size, count, entry_id;
+	int ret = 0;
+
+	if (isDebug(COMPONENT_FSAL)) {
+		char *acl_str;
+
+		acl_str = acl_to_any_text(acl, NULL, ',',
+				TEXT_ABBREVIATE | TEXT_NUMERIC_IDS);
+		LogDebug(COMPONENT_FSAL, "posix acl = %s ", acl_str);
+		acl_free(acl_str);
+	}
+
+	count = acl_entries(acl);
+	real_size = sizeof(*ea_header) + count * sizeof(*ea_entry);
+
+	if (!buf)
+		return real_size;
+	if (real_size > size)
+		return -1;
+
+	ea_entry = (void *)(ea_header + 1);
+	ea_header->a_version = htole32(ACL_EA_VERSION);
+
+	for (entry_id = ACL_FIRST_ENTRY; ; entry_id = ACL_NEXT_ENTRY,
+	     ea_entry++) {
+		ret = acl_get_entry(acl, entry_id, &acl_entry);
+		if (ret == 0 || ret == -1) {
+			LogDebug(COMPONENT_FSAL,
+				"No more ACL entries remaining");
+			break;
+		}
+		if (acl_get_tag_type(acl_entry, &tag) == -1) {
+			LogWarn(COMPONENT_FSAL, "No entry tag for ACL Entry");
+			continue;
+		}
+
+		ret = acl_get_permset(acl_entry, &permset);
+		if (ret) {
+			LogWarn(COMPONENT_FSAL,
+			"Cannot retrieve permission set for the ACL Entry");
+			continue;
+		}
+
+		ea_entry->e_tag = htole16(tag);
+		ea_entry->e_perm = 0;
+		if (acl_get_perm(permset, ACL_READ))
+			ea_entry->e_perm |= htole16(ACL_READ);
+		if (acl_get_perm(permset, ACL_WRITE))
+			ea_entry->e_perm |= htole16(ACL_WRITE);
+		if (acl_get_perm(permset, ACL_EXECUTE))
+			ea_entry->e_perm |= htole16(ACL_EXECUTE);
+
+		switch (tag) {
+		case ACL_USER:
+			ea_entry->e_id =
+				htole32(posix_acl_get_uid(acl_entry));
+			break;
+		case ACL_GROUP:
+			ea_entry->e_id =
+				htole32(posix_acl_get_gid(acl_entry));
+			break;
+		default:
+			ea_entry->e_id = htole32(ACL_UNDEFINED_ID);
+			break;
+		}
+	}
+
+	return real_size;
+}
