@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: LGPL-3.0-or-later
 /*
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -74,7 +75,7 @@ static int schedule_delegrecall_task(struct delegrecall_context *ctx,
  * necessary.
  *
  * @param[in] vec    Up ops vector
- * @param[in] handle Handle being invalidated
+ * @param[in] handle The handle being invalidated
  * @param[in] flags  Flags governing invalidation
  *
  * @return FSAL status
@@ -91,7 +92,7 @@ static fsal_status_t invalidate_close(const struct fsal_up_vector *vec,
 /** Invalidate some or all of a cache entry
  *
  * @param[in] vec    Up ops vector
- * @param[in] handle Handle being invalidated
+ * @param[in] handle The handle being invalidated
  * @param[in] flags  Flags governing invalidation
  *
  * @return FSAL status
@@ -119,7 +120,7 @@ fsal_status_t invalidate(const struct fsal_up_vector *vec,
 
 static fsal_status_t update(const struct fsal_up_vector *vec,
 			    struct gsh_buffdesc *obj,
-			    struct attrlist *attr, uint32_t flags)
+			    struct fsal_attrlist *attr, uint32_t flags)
 {
 	/* No need to update with no cache */
 	return fsalstat(ERR_FSAL_NO_ERROR, 0);
@@ -181,7 +182,7 @@ static state_status_t lock_avail(const struct fsal_up_vector *vec,
 	return STATE_SUCCESS;
 }
 
-/* @note The state_lock MUST be held for write */
+/* @note The st_lock MUST be held */
 static void destroy_recall(struct state_layout_recall_file *recall)
 {
 	if (recall == NULL)
@@ -209,7 +210,7 @@ static void destroy_recall(struct state_layout_recall_file *recall)
  * This function creates the layout recall state and work list for a
  * LAYOUTRECALL operation on a file.
  *
- * @note the state_lock MUST be held for write
+ * @note the st_lock MUST be held
  *
  * @param[in,out] obj     The file on which to send the recall
  * @param[in]     type    The layout type
@@ -530,15 +531,14 @@ static void layoutrec_completion(rpc_call_t *call)
 	struct layoutrecall_cb_data *cb_data = call->call_arg;
 	bool deleted = false;
 	state_t *state = NULL;
-	struct root_op_context root_op_context;
+	struct req_op_context op_context;
 	struct fsal_obj_handle *obj = NULL;
 	struct gsh_export *export = NULL;
 	state_owner_t *owner = NULL;
 	bool ok = false;
 
-	/* Initialize req_ctx */
-	init_root_op_context(&root_op_context, NULL, NULL,
-			     0, 0, UNKNOWN_REQUEST);
+	/* Initialize op_context */
+	init_op_context_simple(&op_context, NULL, NULL);
 
 	LogFullDebug(COMPONENT_NFS_CB, "status %d cb_data %p",
 		     call->cbt.v_u.v4.res.status, cb_data);
@@ -628,10 +628,8 @@ static void layoutrec_completion(rpc_call_t *call)
 
 		STATELOCK_lock(obj);
 
-		root_op_context.req_ctx.clientid =
-			&owner->so_owner.so_nfs4_owner.so_clientid;
-		root_op_context.req_ctx.ctx_export = export;
-		root_op_context.req_ctx.fsal_export = export->fsal_export;
+		op_ctx->clientid = &owner->so_owner.so_nfs4_owner.so_clientid;
+		set_op_context_export(export);
 
 		nfs4_return_one_state(obj,
 				      LAYOUTRETURN4_FILE, circumstance,
@@ -651,18 +649,15 @@ static void layoutrec_completion(rpc_call_t *call)
 	gsh_free(cb_data);
 
 out:
-	release_root_op_context();
 
 	if (ok) {
-		/* Release the export */
-		put_gsh_export(export);
-
 		/* Release object ref */
 		obj->obj_ops->put_ref(obj);
 
 		/* Release the owner */
 		dec_state_owner_ref(owner);
 	}
+	release_op_context();
 }
 
 /**
@@ -682,27 +677,24 @@ static void return_one_async(void *arg)
 	struct layoutrecall_cb_data *cb_data = arg;
 	state_t *state;
 	bool deleted = false;
-	struct root_op_context root_op_context;
+	struct req_op_context op_context;
 	struct fsal_obj_handle *obj = NULL;
 	struct gsh_export *export = NULL;
 	state_owner_t *owner = NULL;
 	bool ok = false;
 
-	/* Initialize req_ctx */
-	init_root_op_context(&root_op_context, NULL, NULL,
-			     0, 0, UNKNOWN_REQUEST);
-
 	state = nfs4_State_Get_Pointer(cb_data->stateid_other);
 
 	ok = get_state_obj_export_owner_refs(state, &obj, &export, &owner);
 
 	if (ok) {
+		/* Initialize op_context */
+		init_op_context_simple(&op_context, export,
+				       export->fsal_export);
+
 		STATELOCK_lock(obj);
 
-		root_op_context.req_ctx.clientid =
-			&owner->so_owner.so_nfs4_owner.so_clientid;
-		root_op_context.req_ctx.ctx_export = export;
-		root_op_context.req_ctx.fsal_export = export->fsal_export;
+		op_ctx->clientid = &owner->so_owner.so_nfs4_owner.so_clientid;
 
 		nfs4_return_one_state(obj, LAYOUTRETURN4_FILE,
 				      circumstance_revoke, state,
@@ -711,7 +703,6 @@ static void return_one_async(void *arg)
 		STATELOCK_unlock(obj);
 	}
 
-	release_root_op_context();
 	gsh_free(cb_data);
 
 	if (state != NULL) {
@@ -720,14 +711,13 @@ static void return_one_async(void *arg)
 	}
 
 	if (ok) {
-		/* Release the export */
-		put_gsh_export(export);
-
 		/* Release object ref */
 		obj->obj_ops->put_ref(obj);
 
 		/* Release the owner */
 		dec_state_owner_ref(owner);
+
+		release_op_context();
 	}
 }
 
@@ -743,16 +733,12 @@ static void layoutrecall_one_call(void *arg)
 	struct layoutrecall_cb_data *cb_data = arg;
 	state_t *state;
 	int code;
-	struct root_op_context root_op_context;
+	struct req_op_context op_context;
 	struct fsal_obj_handle *obj = NULL;
 	struct gsh_export *export = NULL;
 	state_owner_t *owner = NULL;
 	bool ok = false;
 
-	/* Initialize req_ctx */
-	init_root_op_context(&root_op_context, NULL, NULL,
-			     0, 0, UNKNOWN_REQUEST);
-
 	if (cb_data->attempts == 0)
 		now(&cb_data->first_recall);
 
@@ -761,12 +747,13 @@ static void layoutrecall_one_call(void *arg)
 	ok = get_state_obj_export_owner_refs(state, &obj, &export, &owner);
 
 	if (ok) {
+		/* Initialize op_context */
+		init_op_context_simple(&op_context, export,
+				       export->fsal_export);
+
 		STATELOCK_lock(obj);
 
-		root_op_context.req_ctx.clientid =
-		    &owner->so_owner.so_nfs4_owner.so_clientid;
-		root_op_context.req_ctx.ctx_export = export;
-		root_op_context.req_ctx.fsal_export = export->fsal_export;
+		op_ctx->clientid = &owner->so_owner.so_nfs4_owner.so_clientid;
 
 		code = nfs_rpc_cb_single(cb_data->client, &cb_data->arg,
 					  &state->state_refer,
@@ -779,7 +766,7 @@ static void layoutrecall_one_call(void *arg)
 			 * ought to give the client at least one lease
 			 * period to establish a back channel before
 			 * we start revoking state.  We don't have the
-			 * infrasturcture to properly handle layout
+			 * infrastructure to properly handle layout
 			 * revocation, however.  Once we get the
 			 * capability to revoke layouts we should
 			 * queue requests on the clientid, obey the
@@ -825,22 +812,19 @@ static void layoutrecall_one_call(void *arg)
 		gsh_free(cb_data);
 	}
 
-	release_root_op_context();
-
 	if (state != NULL) {
 		/* Release the reference taken above */
 		dec_state_t_ref(state);
 	}
 
 	if (ok) {
-		/* Release the export */
-		put_gsh_export(export);
-
 		/* Release object ref */
 		obj->obj_ops->put_ref(obj);
 
 		/* Release the owner */
 		dec_state_owner_ref(owner);
+
+		release_op_context();
 	}
 }
 
@@ -966,7 +950,7 @@ state_status_t notify_device(notify_deviceid_type4 notify_type,
 /**
  * @brief Check if the delegation needs to be revoked.
  *
- * @param[in] deleg_entry SLE entry for the delegaion
+ * @param[in] deleg_entry SLE entry for the delegation
  *
  * @return true, if the delegation need to be revoked.
  * @return false, if the delegation should not be revoked.
@@ -1105,9 +1089,10 @@ static void delegrecall_completion_func(rpc_call_t *call)
 	char str[LOG_BUFF_LEN] = "\0";
 	struct display_buffer dspbuf = {sizeof(str), str, str};
 	CB_RECALL4args *opcbrecall;
-	struct req_op_context *save_ctx = op_ctx, req_ctx = {0};
+	struct req_op_context op_context;
 	struct gsh_export *export = NULL;
 	bool ret = false;
+	bool used_ctx = false;
 
 	LogDebug(COMPONENT_NFS_CB, "%p %s", call,
 		 !(call->states & NFS_CB_CALL_ABORTED) ? "Success" : "Failed");
@@ -1119,19 +1104,15 @@ static void delegrecall_completion_func(rpc_call_t *call)
 		goto out_free_drc;
 	}
 
-	op_ctx = &req_ctx;
-
-	ret = get_state_obj_export_owner_refs(state, &obj,
-					     &export,
-					     NULL);
+	ret = get_state_obj_export_owner_refs(state, &obj, &export, NULL);
 
 	if (!ret || obj == NULL) {
 		LogDebug(COMPONENT_NFS_CB, "Stale file");
 		goto out_free_drc;
 	}
 
-	op_ctx->ctx_export = export;
-	op_ctx->fsal_export = export->fsal_export;
+	init_op_context_simple(&op_context, export, export->fsal_export);
+	used_ctx = true;
 
 	if (isDebug(COMPONENT_NFS_CB)) {
 		char str[LOG_BUFF_LEN] = "\0";
@@ -1226,17 +1207,15 @@ out_free:
 	if (obj != NULL)
 		obj->obj_ops->put_ref(obj);
 
-	if (export != NULL)
-		put_gsh_export(export);
-
-	op_ctx = save_ctx;
+	if (used_ctx)
+		release_op_context();
 }
 
 /**
  * @brief Send one delegation recall to one client.
  *
  * This function sends a cb_recall for one delegation, the caller has to lock
- * cache_entry->state_lock before calling this function.
+ * cache_entry->st_lock before calling this function.
  *
  * @param[in] obj The file being delegated
  * @param[in] deleg_entry Lock entry covering the delegation
@@ -1336,9 +1315,10 @@ static void delegrevoke_check(void *ctx)
 	char str[LOG_BUFF_LEN] = "\0";
 	struct display_buffer dspbuf = {sizeof(str), str, str};
 	bool str_valid = false;
-	struct req_op_context *save_ctx = op_ctx, req_ctx = {0};
+	struct req_op_context op_context;
 	struct gsh_export *export = NULL;
-	bool   ret = false;
+	bool ret = false;
+	bool used_ctx = false;
 
 	state = nfs4_State_Get_Pointer(deleg_ctx->drc_stateid.other);
 
@@ -1352,20 +1332,15 @@ static void delegrevoke_check(void *ctx)
 		str_valid = true;
 	}
 
-	/* op_ctx may be used by state_del_locked and others */
-	op_ctx = &req_ctx;
-
-	ret = get_state_obj_export_owner_refs(state, &obj,
-					     &export,
-					     NULL);
+	ret = get_state_obj_export_owner_refs(state, &obj, &export, NULL);
 
 	if (!ret || obj == NULL) {
 		LogDebug(COMPONENT_NFS_CB, "Stale file");
 		goto out;
 	}
 
-	op_ctx->ctx_export = export;
-	op_ctx->fsal_export = export->fsal_export;
+	init_op_context_simple(&op_context, export, export->fsal_export);
+	used_ctx = true;
 
 	if (eval_deleg_revoke(state)) {
 		if (str_valid)
@@ -1414,10 +1389,8 @@ static void delegrevoke_check(void *ctx)
 	if (obj != NULL)
 		obj->obj_ops->put_ref(obj);
 
-	if (export != NULL)
-		put_gsh_export(export);
-
-	op_ctx = save_ctx;
+	if (used_ctx)
+		release_op_context();
 }
 
 static void delegrecall_task(void *ctx)
@@ -1426,32 +1399,25 @@ static void delegrecall_task(void *ctx)
 	struct state_t *state;
 	struct fsal_obj_handle *obj;
 	bool   ret = false;
-	struct req_op_context *save_ctx, req_ctx = {0};
+	struct req_op_context op_context;
 	struct gsh_export *export = NULL;
 
 	state = nfs4_State_Get_Pointer(deleg_ctx->drc_stateid.other);
 
 	if (state == NULL) {
-		LogDebug(COMPONENT_NFS_CB, "Delgation is already returned");
+		LogDebug(COMPONENT_NFS_CB, "Delegation is already returned");
 		free_delegrecall_context(deleg_ctx);
 		return;
 	}
 
-	/* op_ctx may be used by state_del_locked and others */
-	save_ctx = op_ctx;
-	op_ctx = &req_ctx;
-
-	ret = get_state_obj_export_owner_refs(state, &obj,
-					     &export,
-					     NULL);
+	ret = get_state_obj_export_owner_refs(state, &obj, &export, NULL);
 	if (!ret || obj == NULL) {
 		LogDebug(COMPONENT_NFS_CB,
-			 "Delgation recall skipped due to stale file");
+			 "Delegation recall skipped due to stale file");
 		goto out;
 	}
 
-	op_ctx->ctx_export = export;
-	op_ctx->fsal_export = export->fsal_export;
+	init_op_context_simple(&op_context, export, export->fsal_export);
 
 	/* state_del_locked is called from deleg_revoke, take the lock. */
 	STATELOCK_lock(obj);
@@ -1460,12 +1426,9 @@ static void delegrecall_task(void *ctx)
 
 	/* Release the obj ref and export ref. */
 	obj->obj_ops->put_ref(obj);
-	put_gsh_export(export);
-	op_ctx->ctx_export = NULL;
-	op_ctx->fsal_export = NULL;
+	release_op_context();
 out:
 	dec_state_t_ref(state);
-	op_ctx = save_ctx;
 }
 
 static int schedule_delegrecall_task(struct delegrecall_context *ctx,
@@ -1506,7 +1469,7 @@ state_status_t delegrecall_impl(struct fsal_obj_handle *obj)
 	struct state_t *state;
 	state_owner_t *owner;
 	struct delegrecall_context *drc_ctx;
-	struct req_op_context *save_ctx = op_ctx, req_ctx = {0};
+	struct req_op_context op_context;
 
 	LogDebug(COMPONENT_FSAL_UP,
 		 "FSAL_UP_DELEG: obj %p type %u",
@@ -1538,7 +1501,7 @@ state_status_t delegrecall_impl(struct fsal_obj_handle *obj)
 
 		drc_ctx = gsh_malloc(sizeof(struct delegrecall_context));
 
-		/* Get references on the owner and the the export. The
+		/* Get references on the owner and the export. The
 		 * export reference we will hold while we perform the recall.
 		 * The owner reference will be used to get access to the
 		 * clientid and reserve the lease.
@@ -1552,10 +1515,12 @@ state_status_t delegrecall_impl(struct fsal_obj_handle *obj)
 			continue;
 		}
 
-		/* op_ctx may be used by state_del_locked and others */
-		op_ctx = &req_ctx;
-		op_ctx->ctx_export = drc_ctx->drc_exp;
-		op_ctx->fsal_export = drc_ctx->drc_exp->fsal_export;
+		/* Get a ref to the drc_ctx->drc_exp and initialize op_context
+		 * in case state_del_locks and others need it.
+		 */
+		get_gsh_export_ref(drc_ctx->drc_exp);
+		init_op_context_simple(&op_context, drc_ctx->drc_exp,
+				       drc_ctx->drc_exp->fsal_export);
 
 		drc_ctx->drc_clid = owner->so_owner.so_nfs4_owner.so_clientrec;
 		COPY_STATEID(&drc_ctx->drc_stateid, state);
@@ -1573,25 +1538,25 @@ state_status_t delegrecall_impl(struct fsal_obj_handle *obj)
 		PTHREAD_MUTEX_lock(&drc_ctx->drc_clid->cid_mutex);
 		if (!reserve_lease(drc_ctx->drc_clid)) {
 			PTHREAD_MUTEX_unlock(&drc_ctx->drc_clid->cid_mutex);
-			put_gsh_export(drc_ctx->drc_exp);
 			dec_client_id_ref(drc_ctx->drc_clid);
 			gsh_free(drc_ctx);
+			release_op_context();
 			continue;
 		}
 		PTHREAD_MUTEX_unlock(&drc_ctx->drc_clid->cid_mutex);
 
 		delegrecall_one(obj, state, drc_ctx);
+		release_op_context();
 	}
 	STATELOCK_unlock(obj);
 
-	op_ctx = save_ctx;
 	return rc;
 }
 
 /**
  * @brief Recall a delegation
  *
- * @param[in] handle Handle on which the delegation is held
+ * @param[in] handle The handle on which the delegation is held
  *
  * @return STATE_SUCCESS or errors.
  */
@@ -1612,7 +1577,7 @@ state_status_t delegrecall(const struct fsal_up_vector *vec,
 							       &obj, NULL));
 	if (rc != STATE_SUCCESS) {
 		LogDebug(COMPONENT_FSAL_UP,
-			 "FSAL_UP_DELEG: cache inode get failed, rc %d", rc);
+			 "FSAL_UP_DELEG: create_handle failed, rc %d", rc);
 		/* Not an error. Expecting some nodes will not have it
 		 * in cache in a cluster. */
 		return rc;
@@ -1666,7 +1631,7 @@ cbgetattr_state handle_getattr_response(struct cbgetattr_context *cbg_ctx,
 					rpc_call_t *call)
 {
 	fattr4 attr;
-	struct attrlist rsp_attr;
+	struct fsal_attrlist rsp_attr;
 	int rc = 0;
 	enum cbgetattr_state cb_state;
 	struct fsal_obj_handle *obj = cbg_ctx->obj;
@@ -1674,11 +1639,11 @@ cbgetattr_state handle_getattr_response(struct cbgetattr_context *cbg_ctx,
 	nfs_cb_resop4 *cbr = NULL;
 	CB_GETATTR4res *res = NULL;
 	const struct fsal_up_vector *event_func;
-	struct attrlist up_attr = {0, };
+	struct fsal_attrlist up_attr = {0, };
 	uint32_t upflags = 0;
 	struct gsh_buffdesc key;
 	fsal_status_t fsal_status = {0,};
-	struct root_op_context root_ctx;
+	struct req_op_context op_context;
 	struct timespec	timeo = { .tv_sec = time(NULL) + 5,
 				  .tv_nsec = 0 };
 
@@ -1711,7 +1676,7 @@ cbgetattr_state handle_getattr_response(struct cbgetattr_context *cbg_ctx,
 		obj->state_hdl->file.cbgetattr.modified = true;
 	}
 
-	/* everytime we need to update change number to let
+	/* every time we need to update change number to let
 	 * other clients know that file contents could have
 	 * been modified even if the filesize remains same
 	 */
@@ -1720,9 +1685,10 @@ cbgetattr_state handle_getattr_response(struct cbgetattr_context *cbg_ctx,
 
 	event_func = (cbg_ctx->ctx_export->fsal_export->up_ops);
 
-	init_root_op_context(&root_ctx, cbg_ctx->ctx_export,
-			 cbg_ctx->ctx_export->fsal_export,
-			 0, 0, UNKNOWN_REQUEST);
+	/* Get a ref to the cbg_ctx->ctx_export and initialize op_context */
+	get_gsh_export_ref(cbg_ctx->ctx_export);
+	init_op_context_simple(&op_context, cbg_ctx->ctx_export,
+			       cbg_ctx->ctx_export->fsal_export);
 
 	/* @todo : log message if event_func is NULL */
 	obj->obj_ops->handle_to_key(obj, &key);
@@ -1739,7 +1705,7 @@ cbgetattr_state handle_getattr_response(struct cbgetattr_context *cbg_ctx,
 	fsal_status = event_func->update(event_func, &key,
 					 &up_attr, upflags);
 
-	release_root_op_context();
+	release_op_context();
 
 	if (FSAL_IS_ERROR(fsal_status)) {
 		cb_state = CB_GETATTR_FAILED;
@@ -1795,7 +1761,7 @@ static void cbgetattr_completion_func(rpc_call_t *call)
 	switch (call->cbt.v_u.v4.res.status) {
 	case NFS4_OK:
 		LogDebug(COMPONENT_NFS_CB,
-			 "CB_GEATTR succeded for client(%s)",
+			 "CB_GEATTR succeeded for client(%s)",
 			 cbg_ctx->clid->gsh_client->hostaddr_str);
 		*cbgetattr_state = handle_getattr_response(cbg_ctx,
 							   call);
@@ -1823,7 +1789,7 @@ out:
  * @brief Send CB_GETATTR to write_delegated client.
  *
  * This function sends a CB_GETATTR, the caller has to lock
- * cache_entry->state_lock before calling this function.
+ * cache_entry->st_lock before calling this function.
  *
  * @param[in] obj The file holding delegation
  * @param[in] cbgetattr_context
@@ -1837,11 +1803,12 @@ int send_cbgetattr(struct fsal_obj_handle *obj,
 	int ret = 0;
 	nfs_cb_argop4 argop;
 	bitmap4	*cb_attr_req;
-	struct root_op_context root_ctx;
+	struct req_op_context op_context;
 
-	init_root_op_context(&root_ctx, p_cargs->ctx_export,
-			     p_cargs->ctx_export->fsal_export,
-			     0, 0, UNKNOWN_REQUEST);
+	/* Get a ref to the p_cargs->ctx_export and initialize op_context */
+	get_gsh_export_ref(p_cargs->ctx_export);
+	init_op_context_simple(&op_context, p_cargs->ctx_export,
+			       p_cargs->ctx_export->fsal_export);
 
 	LogDebug(COMPONENT_FSAL_UP, "Sending CB_GETATTR to client %s",
 		 p_cargs->clid->gsh_client->hostaddr_str);
@@ -1871,11 +1838,13 @@ int send_cbgetattr(struct fsal_obj_handle *obj,
 
 	ret = nfs_rpc_cb_single(p_cargs->clid, &argop, NULL,
 				cbgetattr_completion_func, p_cargs);
-	if (ret == 0)
-		return 0;
+
 	LogDebug(COMPONENT_FSAL_UP, "CB_GETATTR nfs_rpc_cb_single returned %d",
 		 ret);
 
+	if (ret == 0)
+		goto out_exp;
+
 out:
 	nfs4_freeFH(&argop.nfs_cb_argop4_u.opcbgetattr.fh);
 
@@ -1883,7 +1852,10 @@ out:
 		p_cargs->clid->gsh_client->hostaddr_str);
 
 	free_cbgetattr_context(p_cargs);
-	release_root_op_context();
+
+out_exp:
+
+	release_op_context();
 	return ret;
 }
 
@@ -1896,7 +1868,7 @@ out:
  * otherwise mark it as CB_GETATTR_FAILED.
  *
  * @param[in] obj file being operated on
- * @param[in] client client holding delegation
+ * @param[in] client   The client holding delegation
  * @param[in] ctx_exp current export reference
  *
  * @return: 0, success
@@ -1995,7 +1967,7 @@ void up_ready_wait(struct fsal_up_vector *up_ops)
 /** Release a mdcache entry if otherwise unused
  *
  * @param[in] vec    Up ops vector
- * @param[in] handle Handle being conditionally released
+ * @param[in] handle The handle being conditionally released
  * @param[in] flags  Unused, for future expansion
  */
 static fsal_status_t try_release(const struct fsal_up_vector *vec,
