@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: LGPL-3.0-or-later
 /*
  * vim:noexpandtab:shiftwidth=8:tabstop=8:
  *
@@ -292,8 +293,6 @@ static void _9p_enqueue_req(struct _9p_request_data *reqdata)
 			/* ! SPIN LOCKED */
 			pthread_spin_unlock(&_9p_req_st.reqs.sp);
 	}
-
-	return;
 }
 
 /**
@@ -303,15 +302,11 @@ static void _9p_enqueue_req(struct _9p_request_data *reqdata)
  */
 static void _9p_execute(struct _9p_request_data *req9p)
 {
-	struct req_op_context req_ctx;
-	struct export_perms export_perms;
+	struct req_op_context op_context;
 
-	memset(&req_ctx, 0, sizeof(struct req_op_context));
-	memset(&export_perms, 0, sizeof(struct export_perms));
-	op_ctx = &req_ctx;
-	op_ctx->caller_addr = (sockaddr_t *)&req9p->pconn->addrpeer;
-	op_ctx->req_type = _9P_REQUEST;
-	op_ctx->export_perms = &export_perms;
+	init_op_context(&op_context, NULL, NULL,
+			(sockaddr_t *)&req9p->pconn->addrpeer,
+			0, 0, _9P_REQUEST);
 
 	if (req9p->pconn->trans_type == _9P_TCP)
 		_9p_tcp_process_request(req9p);
@@ -319,7 +314,7 @@ static void _9p_execute(struct _9p_request_data *req9p)
 	else if (req9p->pconn->trans_type == _9P_RDMA)
 		_9p_rdma_process_request(req9p);
 #endif
-	op_ctx = NULL;
+	release_op_context();
 }				/* _9p_execute */
 
 /**
@@ -527,7 +522,7 @@ void DispatchWork9P(struct _9p_request_data *req)
 
 void *_9p_socket_thread(void *Arg)
 {
-	long int tcp_sock = (long int)Arg;
+	long tcp_sock = (long)Arg;
 	int rc = -1;
 	struct pollfd fds[1];
 	int fdcount = 1;
@@ -687,7 +682,7 @@ void *_9p_socket_thread(void *Arg)
 		tag = *(u16 *) (_9pmsg + _9P_HDR_SIZE + _9P_TYPE_SIZE);
 		_9p_AddFlushHook(req, tag, sequence++);
 		LogFullDebug(COMPONENT_9P,
-			     "Request tag is %d\n", tag);
+			     "Request tag is %d", tag);
 
 		/* Message was OK push it */
 		DispatchWork9P(req);
@@ -948,7 +943,7 @@ void *_9p_dispatcher_thread(void *Arg)
 {
 	int _9p_socket;
 	int rc = 0;
-	long int newsock = -1;
+	long newsock = -1;
 	pthread_attr_t attr_thr;
 	pthread_t tcp_thrid;
 
