@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: LGPL-3.0-or-later
 /*
  * vim:noexpandtab:shiftwidth=8:tabstop=8:
  *
@@ -76,6 +77,7 @@
 #include "nfs_init.h"
 #include <urcu-bp.h>
 #include "conf_url.h"
+#include "FSAL/fsal_localfs.h"
 
 /**
  * @brief init_complete used to indicate if ganesha is during
@@ -89,7 +91,7 @@ struct _nfs_health nfs_health_;
 
 static struct _nfs_health healthstats;
 
-/* ServerEpoch is ServerBootTime unless overriden by -E command line option */
+/* ServerEpoch is ServerBootTime unless overridden by -E command line option */
 struct timespec nfs_ServerBootTime;
 time_t nfs_ServerEpoch;
 
@@ -151,13 +153,8 @@ struct config_error_type err_type;
 void reread_config(void)
 {
 	int status = 0;
-	int i;
 	config_file_t config_struct;
 
-	/* Clear out the flag indicating component was set from environment. */
-	for (i = COMPONENT_ALL; i < COMPONENT_COUNT; i++)
-		LogComponents[i].comp_env_set = false;
-
 	/* If no configuration file is given, then the caller must want to
 	 * reparse the configuration file from startup.
 	 */
@@ -316,7 +313,9 @@ void nfs_print_param_config(void)
 	printf("NFS_Core_Param\n{\n");
 
 	printf("\tNFS_Port = %u ;\n", nfs_param.core_param.port[P_NFS]);
+#ifdef _USE_NFS3
 	printf("\tMNT_Port = %u ;\n", nfs_param.core_param.port[P_MNT]);
+#endif
 	printf("\tNFS_Program = %u ;\n", nfs_param.core_param.program[P_NFS]);
 	printf("\tMNT_Program = %u ;\n", nfs_param.core_param.program[P_NFS]);
 	printf("\tDRC_TCP_Npart = %u ;\n", nfs_param.core_param.drc.tcp.npart);
@@ -352,8 +351,7 @@ void nfs_print_param_config(void)
 	printf("\tDrop_Delay_Errors = %s ;\n",
 	       nfs_param.core_param.drop_delay_errors ? "true" : "false");
 
-	printf("\tEnable UDP = %s ;\n", nfs_param.core_param.enable_UDP ?
-	       "true" : "false");
+	printf("\tEnable UDP = %u ;\n", nfs_param.core_param.enable_UDP);
 
 	printf("}\n\n");
 }
@@ -391,7 +389,9 @@ int nfs_set_param_from_conf(config_file_t parse_tree,
 		return -1;
 	}
 
-	/* Worker paramters: ip/name hash table and expiration for each entry */
+	/* Worker parameters: ip/name hash table and expiration
+	 * for each entry
+	 */
 	(void) load_config_from_parse(parse_tree,
 				      &nfs_ip_name,
 				      NULL,
@@ -877,7 +877,7 @@ static void lower_my_caps(void)
 
 	/* Print newly set capabilities (same as what CLI "getpcaps" displays */
 	cap_text = cap_to_text(cap, &capstrlen);
-	LogEvent(COMPONENT_INIT, "currenty set capabilities are: %s",
+	LogEvent(COMPONENT_INIT, "currently set capabilities are: %s",
 		 cap_text);
 	cap_free(cap_text);
 	cap_free(cap);
@@ -1010,7 +1010,7 @@ bool nfs_health(void)
 
 	/* Consider healthy and making progress if we have dequeued some
 	 * requests or there is one or less to dequeue.  Don't check
-	 * enqueue_diff == 0 here, as there will be suprious warnings during
+	 * enqueue_diff == 0 here, as there will be spurious warnings during
 	 * times of low traffic, when an enqueue happens to coincide with the
 	 * heartbeat firing.
 	 */
