@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: LGPL-3.0-or-later
 /*
  * vim:noexpandtab:shiftwidth=8:tabstop=8:
  *
@@ -54,6 +55,7 @@
 #include "mount.h"
 #include "nlm4.h"
 #include "rquota.h"
+#include "nfsacl.h"
 #include "nfs_init.h"
 #include "nfs_core.h"
 #include "nfs_exports.h"
@@ -65,9 +67,14 @@
 #include "gsh_lttng/nfs_rpc.h"
 #endif
 
-#define NFS_pcp nfs_param.core_param
-#define NFS_options NFS_pcp.core_options
-#define NFS_program NFS_pcp.program
+#ifdef __APPLE__
+#include <sys/socketvar.h>
+#include <netinet/tcp_var.h>
+#define TCP_KEEPIDLE TCPCTL_KEEPIDLE
+#endif
+
+#define NFS_options nfs_param.core_param.core_options
+#define NFS_program nfs_param.core_param.program
 
 /**
  * TI-RPC event channels.  Each channel is a thread servicing an event
@@ -117,13 +124,26 @@ void nfs_rpc_dispatch_dummy(struct svc_req *req)
 		 __func__);
 }
 
-const char *tags[] = {
+const char *tags[P_COUNT] = {
 	"NFS",
+#ifdef _USE_NFS3
 	"MNT",
+#endif
+#ifdef _USE_NLM
 	"NLM",
+#ifdef _USE_RQUOTA
 	"RQUOTA",
+#endif
+#endif
+#ifdef USE_NFSACL3
+	"NFSACL",
+#endif
+#ifdef RPC_VSOCK
 	"NFS_VSOCK",
+#endif
+#ifdef _USE_NFS_RDMA
 	"NFS_RDMA",
+#endif
 };
 
 typedef struct proto_data {
@@ -182,8 +202,10 @@ static void unregister(const rpcprog_t prog, const rpcvers_t vers1,
 static void unregister_rpc(void)
 {
 	if ((NFS_options & CORE_OPTION_NFSV3) != 0) {
+#ifdef _USE_NFS3
 		unregister(NFS_program[P_NFS], NFS_V3, NFS_V4);
 		unregister(NFS_program[P_MNT], MOUNT_V1, MOUNT_V3);
+#endif
 	} else {
 		unregister(NFS_program[P_NFS], NFS_V4, NFS_V4);
 	}
@@ -191,23 +213,35 @@ static void unregister_rpc(void)
 	if (nfs_param.core_param.enable_NLM)
 		unregister(NFS_program[P_NLM], 1, NLM4_VERS);
 #endif /* _USE_NLM */
+#ifdef _USE_RQUOTA
 	if (nfs_param.core_param.enable_RQUOTA) {
 		unregister(NFS_program[P_RQUOTA], RQUOTAVERS, EXT_RQUOTAVERS);
 	}
+#endif
+
+#ifdef USE_NFSACL3
+	if (nfs_param.core_param.enable_NFSACL) {
+		unregister(NFS_program[P_NFSACL], NFSACL_V3, NFSACL_V3);
+	}
+#endif
 }
 
 static inline bool nfs_protocol_enabled(protos p)
 {
+#ifdef _USE_NFS3
 	bool nfsv3 = NFS_options & CORE_OPTION_NFSV3;
+#endif
 
 	switch (p) {
 	case P_NFS:
 		return true;
 
+#ifdef _USE_NFS3
 	case P_MNT: /* valid only for NFSv3 environments */
 		if (nfsv3)
 			return true;
 		break;
+#endif
 
 #ifdef _USE_NLM
 	case P_NLM: /* valid only for NFSv3 environments */
@@ -216,10 +250,20 @@ static inline bool nfs_protocol_enabled(protos p)
 		break;
 #endif
 
+#ifdef _USE_RQUOTA
 	case P_RQUOTA:
 		if (nfs_param.core_param.enable_RQUOTA)
 			return true;
 		break;
+#endif
+
+#ifdef USE_NFSACL3
+	case P_NFSACL: /* valid only for NFSv3 environments */
+		if (nfsv3 && nfs_param.core_param.enable_NFSACL) {
+			return true;
+		}
+		break;
+#endif
 
 	default:
 		break;
@@ -231,7 +275,7 @@ static inline bool nfs_protocol_enabled(protos p)
 /**
  * @brief Close transports and file descriptors used for RPC services.
  *
- * So that restarting the NFS server wont encounter issues of "Address
+ * So that restarting the NFS server won't encounter issues of "Address
  * Already In Use" - this has occurred even though we set the
  * SO_REUSEADDR option when restarting the server with a single export
  * (i.e.: a small config) & no logging at all, making the restart very
@@ -279,6 +323,7 @@ static enum xprt_stat nfs_rpc_dispatch_udp_NFS(SVCXPRT *xprt)
 	return SVC_RECV(xprt);
 }
 
+#ifdef _USE_NFS3
 static enum xprt_stat nfs_rpc_dispatch_udp_MNT(SVCXPRT *xprt)
 {
 	LogFullDebug(COMPONENT_DISPATCH,
@@ -290,7 +335,9 @@ static enum xprt_stat nfs_rpc_dispatch_udp_MNT(SVCXPRT *xprt)
 	xprt->xp_dispatch.process_cb = nfs_rpc_valid_MNT;
 	return SVC_RECV(xprt);
 }
+#endif
 
+#ifdef _USE_NLM
 static enum xprt_stat nfs_rpc_dispatch_udp_NLM(SVCXPRT *xprt)
 {
 	LogFullDebug(COMPONENT_DISPATCH,
@@ -302,7 +349,9 @@ static enum xprt_stat nfs_rpc_dispatch_udp_NLM(SVCXPRT *xprt)
 	xprt->xp_dispatch.process_cb = nfs_rpc_valid_NLM;
 	return SVC_RECV(xprt);
 }
+#endif
 
+#ifdef _USE_RQUOTA
 static enum xprt_stat nfs_rpc_dispatch_udp_RQUOTA(SVCXPRT *xprt)
 {
 	LogFullDebug(COMPONENT_DISPATCH,
@@ -314,14 +363,39 @@ static enum xprt_stat nfs_rpc_dispatch_udp_RQUOTA(SVCXPRT *xprt)
 	xprt->xp_dispatch.process_cb = nfs_rpc_valid_RQUOTA;
 	return SVC_RECV(xprt);
 }
+#endif
+
+#ifdef USE_NFSACL3
+static enum xprt_stat nfs_rpc_dispatch_udp_NFSACL(SVCXPRT *xprt)
+{
+	LogFullDebug(COMPONENT_DISPATCH,
+		     "NFSACL UDP request for SVCXPRT %p fd %d",
+		     xprt, xprt->xp_fd);
+	xprt->xp_dispatch.process_cb = nfs_rpc_valid_NFSACL;
+	return SVC_RECV(xprt);
+}
+#endif
 
 const svc_xprt_fun_t udp_dispatch[] = {
 	nfs_rpc_dispatch_udp_NFS,
+#ifdef _USE_NFS3
 	nfs_rpc_dispatch_udp_MNT,
+#endif
+#ifdef _USE_NLM
 	nfs_rpc_dispatch_udp_NLM,
+#endif
+#ifdef _USE_RQUOTA
 	nfs_rpc_dispatch_udp_RQUOTA,
+#endif
+#ifdef USE_NFSACL3
+	nfs_rpc_dispatch_udp_NFSACL,
+#endif
+#ifdef RPC_VSOCK
 	NULL,
+#endif
+#ifdef _USE_NFS_RDMA
 	NULL,
+#endif
 };
 
 static enum xprt_stat nfs_rpc_dispatch_tcp_NFS(SVCXPRT *xprt)
@@ -333,6 +407,7 @@ static enum xprt_stat nfs_rpc_dispatch_tcp_NFS(SVCXPRT *xprt)
 	return nfs_rpc_tcp_user_data(xprt);
 }
 
+#ifdef _USE_NFS3
 static enum xprt_stat nfs_rpc_dispatch_tcp_MNT(SVCXPRT *xprt)
 {
 	LogFullDebug(COMPONENT_DISPATCH,
@@ -341,7 +416,9 @@ static enum xprt_stat nfs_rpc_dispatch_tcp_MNT(SVCXPRT *xprt)
 	xprt->xp_dispatch.process_cb = nfs_rpc_valid_MNT;
 	return nfs_rpc_tcp_user_data(xprt);
 }
+#endif
 
+#ifdef _USE_NLM
 static enum xprt_stat nfs_rpc_dispatch_tcp_NLM(SVCXPRT *xprt)
 {
 	LogFullDebug(COMPONENT_DISPATCH,
@@ -350,7 +427,9 @@ static enum xprt_stat nfs_rpc_dispatch_tcp_NLM(SVCXPRT *xprt)
 	xprt->xp_dispatch.process_cb = nfs_rpc_valid_NLM;
 	return nfs_rpc_tcp_user_data(xprt);
 }
+#endif
 
+#ifdef _USE_RQUOTA
 static enum xprt_stat nfs_rpc_dispatch_tcp_RQUOTA(SVCXPRT *xprt)
 {
 	LogFullDebug(COMPONENT_DISPATCH,
@@ -359,7 +438,20 @@ static enum xprt_stat nfs_rpc_dispatch_tcp_RQUOTA(SVCXPRT *xprt)
 	xprt->xp_dispatch.process_cb = nfs_rpc_valid_RQUOTA;
 	return nfs_rpc_tcp_user_data(xprt);
 }
+#endif
 
+#ifdef USE_NFSACL3
+static enum xprt_stat nfs_rpc_dispatch_tcp_NFSACL(SVCXPRT *xprt)
+{
+	LogFullDebug(COMPONENT_DISPATCH,
+		     "NFSACL TCP request on SVCXPRT %p fd %d",
+		     xprt, xprt->xp_fd);
+	xprt->xp_dispatch.process_cb = nfs_rpc_valid_NFSACL;
+	return nfs_rpc_tcp_user_data(xprt);
+}
+#endif
+
+#ifdef RPC_VSOCK
 static enum xprt_stat nfs_rpc_dispatch_tcp_VSOCK(SVCXPRT *xprt)
 {
 	LogFullDebug(COMPONENT_DISPATCH,
@@ -368,14 +460,28 @@ static enum xprt_stat nfs_rpc_dispatch_tcp_VSOCK(SVCXPRT *xprt)
 	xprt->xp_dispatch.process_cb = nfs_rpc_valid_NFS;
 	return nfs_rpc_tcp_user_data(xprt);
 }
+#endif
 
-const svc_xprt_fun_t tcp_dispatch[] = {
+const svc_xprt_fun_t tcp_dispatch[P_COUNT] = {
 	nfs_rpc_dispatch_tcp_NFS,
+#ifdef _USE_NFS3
 	nfs_rpc_dispatch_tcp_MNT,
+#endif
+#ifdef _USE_NLM
 	nfs_rpc_dispatch_tcp_NLM,
+#endif
+#ifdef _USE_RQUOTA
 	nfs_rpc_dispatch_tcp_RQUOTA,
+#endif
+#ifdef USE_NFSACL3
+	nfs_rpc_dispatch_tcp_NFSACL,
+#endif
+#ifdef RPC_VSOCK
 	nfs_rpc_dispatch_tcp_VSOCK,
+#endif
+#ifdef _USE_NFS_RDMA
 	NULL,
+#endif
 };
 
 void Create_udp(protos prot)
@@ -499,6 +605,14 @@ static int Bind_sockets_V6(void)
 	protos p;
 	int    rc = 0;
 
+	if (isInfo(COMPONENT_DISPATCH)) {
+		char str[LOG_BUFF_LEN] = "\0";
+		struct display_buffer dbuf = {sizeof(str), str, str};
+
+		display_sockaddr(&dbuf, &nfs_param.core_param.bind_addr);
+		LogInfo(COMPONENT_DISPATCH, "Binding to address %s", str);
+	}
+
 	for (p = P_NFS; p < P_COUNT; p++) {
 		if (nfs_protocol_enabled(p)) {
 
@@ -534,6 +648,21 @@ static int Bind_sockets_V6(void)
 					return -1;
 				}
 
+				if (isInfo(COMPONENT_DISPATCH)) {
+					char str[LOG_BUFF_LEN] = "\0";
+					struct display_buffer dbuf = {
+							sizeof(str), str, str};
+
+					display_sockaddr(
+					    &dbuf,
+					    (sockaddr_t *)
+					    pdatap->bindaddr_udp6.addr.buf);
+
+					LogInfo(COMPONENT_DISPATCH,
+						"Binding UDP socket to address %s for %s",
+						str, tags[p]);
+				}
+
 				rc = bind(udp_socket[p],
 					  (struct sockaddr *)
 					  pdatap->bindaddr_udp6.addr.buf,
@@ -573,6 +702,21 @@ static int Bind_sockets_V6(void)
 				return -1;
 			}
 
+			if (isInfo(COMPONENT_DISPATCH)) {
+				char str[LOG_BUFF_LEN] = "\0";
+				struct display_buffer dbuf = {
+						sizeof(str), str, str};
+
+				display_sockaddr(
+				    &dbuf,
+				    (sockaddr_t *)
+				    pdatap->bindaddr_tcp6.addr.buf);
+
+				LogInfo(COMPONENT_DISPATCH,
+					"Binding TCP socket to address %s for %s",
+					str, tags[p]);
+			}
+
 			rc = bind(tcp_socket[p],
 				  (struct sockaddr *)
 				   pdatap->bindaddr_tcp6.addr.buf,
@@ -598,6 +742,14 @@ static int Bind_sockets_V4(void)
 	protos p;
 	int    rc = 0;
 
+	if (isInfo(COMPONENT_DISPATCH)) {
+		char str[LOG_BUFF_LEN] = "\0";
+		struct display_buffer dbuf = {sizeof(str), str, str};
+
+		display_sockaddr(&dbuf, &nfs_param.core_param.bind_addr);
+		LogInfo(COMPONENT_DISPATCH, "Binding to address %s", str);
+	}
+
 	for (p = P_NFS; p < P_COUNT; p++) {
 		if (nfs_protocol_enabled(p)) {
 
@@ -634,6 +786,21 @@ static int Bind_sockets_V4(void)
 					return -1;
 				}
 
+				if (isInfo(COMPONENT_DISPATCH)) {
+					char str[LOG_BUFF_LEN] = "\0";
+					struct display_buffer dbuf = {
+							sizeof(str), str, str};
+
+					display_sockaddr(
+					    &dbuf,
+					    (sockaddr_t *)
+					    pdatap->bindaddr_udp6.addr.buf);
+
+					LogInfo(COMPONENT_DISPATCH,
+						"Binding UDP socket to address %s for %s",
+						str, tags[p]);
+				}
+
 				rc = bind(udp_socket[p],
 					  (struct sockaddr *)
 					  pdatap->bindaddr_udp6.addr.buf,
@@ -651,7 +818,7 @@ static int Bind_sockets_V4(void)
 			       sizeof(pdatap->sinaddr_tcp));
 			pdatap->sinaddr_tcp.sin_family = AF_INET;
 			/* all interfaces */
-			pdatap->sinaddr_udp.sin_addr.s_addr =
+			pdatap->sinaddr_tcp.sin_addr.s_addr =
 			    ((struct sockaddr_in *)
 			    &nfs_param.core_param.bind_addr)->sin_addr.s_addr;
 			pdatap->sinaddr_tcp.sin_port =
@@ -673,6 +840,21 @@ static int Bind_sockets_V4(void)
 				return -1;
 			}
 
+			if (isInfo(COMPONENT_DISPATCH)) {
+				char str[LOG_BUFF_LEN] = "\0";
+				struct display_buffer dbuf = {
+						sizeof(str), str, str};
+
+				display_sockaddr(
+				    &dbuf,
+				    (sockaddr_t *)
+				    pdatap->bindaddr_tcp6.addr.buf);
+
+				LogInfo(COMPONENT_DISPATCH,
+					"Binding TCP socket to address %s for %s",
+					str, tags[p]);
+			}
+
 			rc = bind(tcp_socket[p],
 				  (struct sockaddr *)
 				  pdatap->bindaddr_tcp6.addr.buf,
@@ -840,6 +1022,18 @@ static int alloc_socket_setopts(int p)
 	return 0;
 }
 
+static bool enable_udp_listener(protos prot)
+{
+	if (nfs_param.core_param.enable_UDP & UDP_LISTENER_ALL)
+		return true;
+#ifdef _USE_NFS3
+	if (prot == P_MNT &&
+	    (nfs_param.core_param.enable_UDP & UDP_LISTENER_MOUNT))
+		return true;
+#endif
+	return false;
+}
+
 /**
  * @brief Allocate the tcp and udp sockets for the nfs daemon
  * using V4 interfaces
@@ -847,7 +1041,7 @@ static int alloc_socket_setopts(int p)
 static int Allocate_sockets_V4(int p)
 {
 	udp_socket[p] = tcp_socket[p] = -1;
-	if (nfs_param.core_param.enable_UDP) {
+	if (enable_udp_listener(p)) {
 		udp_socket[p] = socket(AF_INET,
 				       SOCK_DGRAM,
 				       IPPROTO_UDP);
@@ -933,7 +1127,7 @@ static void Allocate_sockets(void)
 			if (v6disabled)
 				goto try_V4;
 
-			if (nfs_param.core_param.enable_UDP) {
+			if (enable_udp_listener(p)) {
 				udp_socket[p] = socket(AF_INET6,
 						       SOCK_DGRAM,
 						       IPPROTO_UDP);
@@ -966,18 +1160,31 @@ static void Allocate_sockets(void)
 					       SOCK_STREAM,
 					       IPPROTO_TCP);
 
-			/* We fail with LogFatal here on error because it
-			 * shouldn't be that we have managed to create a
-			 * V6 based udp socket and have failed for the tcp
-			 * sock. If it were a case of V6 being disabled,
-			 * then we would have encountered that case with
-			 * the first udp sock create and would have moved
-			 * on to create the V4 sockets.
-			 */
-			if (tcp_socket[p] == -1)
-				LogFatal(COMPONENT_DISPATCH,
-					 "Cannot allocate a tcp socket for %s, error %d(%s)",
-					 tags[p], errno, strerror(errno));
+			if (tcp_socket[p] == -1) {
+				/* We fail with LogFatal here on error because
+				 * it shouldn't be that we have managed to
+				 * create a V6 based udp socket and have failed
+				 * for the tcp sock. If it were a case of V6
+				 * being disabled, then we would have
+				 * encountered that case with the first udp
+				 * sock create and would have moved on to
+				 * create the V4 sockets.
+				 */
+				if (enable_udp_listener(p)
+				    || errno != EAFNOSUPPORT) {
+					LogFatal(COMPONENT_DISPATCH,
+						 "Cannot allocate a tcp socket for %s, error %d(%s)",
+						 tags[p], errno,
+						 strerror(errno));
+				}
+
+				v6disabled = true;
+				LogWarn(COMPONENT_DISPATCH,
+				    "System may not have V6 intfs configured error %d(%s)",
+				    errno, strerror(errno));
+
+				goto try_V4;
+			}
 
 try_V4:
 			if (v6disabled) {
@@ -1036,64 +1243,62 @@ void Clean_RPC(void)
 		nfs_rpc_dispatch_dummy, netconfig)
 
 #ifdef RPCBIND
-static bool __Register_program(protos prot, int flag, int vers)
+static bool __Register_program(protos prot, int vers)
 {
-	if ((NFS_options & flag) != 0) {
-		if (nfs_param.core_param.enable_UDP) {
-			LogInfo(COMPONENT_DISPATCH, "Registering %s V%d/UDP",
-				tags[prot], (int)vers);
-
-			/* XXXX fix svc_register! */
-			if (!UDP_REGISTER(prot, vers, netconfig_udpv4)) {
-				LogMajor(COMPONENT_DISPATCH,
-					 "Cannot register %s V%d on UDP",
-					 tags[prot], (int)vers);
-				return false;
-			}
-
-			if (!v6disabled && netconfig_udpv6) {
-				LogInfo(COMPONENT_DISPATCH,
-					"Registering %s V%d/UDPv6",
-					tags[prot], (int)vers);
-				if (!UDP_REGISTER(prot, vers,
-						  netconfig_udpv6)) {
-					LogMajor(COMPONENT_DISPATCH,
-						 "Cannot register %s V%d on UDPv6",
-						 tags[prot], (int)vers);
-					return false;
-				}
-			}
-		}
-
-#ifndef _NO_TCP_REGISTER
-		LogInfo(COMPONENT_DISPATCH, "Registering %s V%d/TCP",
+	if (enable_udp_listener(prot)) {
+		LogInfo(COMPONENT_DISPATCH, "Registering %s V%d/UDP",
 			tags[prot], (int)vers);
 
-		if (!TCP_REGISTER(prot, vers, netconfig_tcpv4)) {
+		/* XXXX fix svc_register! */
+		if (!UDP_REGISTER(prot, vers, netconfig_udpv4)) {
 			LogMajor(COMPONENT_DISPATCH,
-				 "Cannot register %s V%d on TCP", tags[prot],
-				 (int)vers);
+				 "Cannot register %s V%d on UDP",
+				 tags[prot], (int)vers);
 			return false;
 		}
 
-		if (!v6disabled && netconfig_tcpv6) {
-			LogInfo(COMPONENT_DISPATCH, "Registering %s V%d/TCPv6",
+		if (!v6disabled && netconfig_udpv6) {
+			LogInfo(COMPONENT_DISPATCH,
+				"Registering %s V%d/UDPv6",
 				tags[prot], (int)vers);
-			if (!TCP_REGISTER(prot, vers, netconfig_tcpv6)) {
+			if (!UDP_REGISTER(prot, vers,
+					  netconfig_udpv6)) {
 				LogMajor(COMPONENT_DISPATCH,
-					 "Cannot register %s V%d on TCPv6",
+					 "Cannot register %s V%d on UDPv6",
 					 tags[prot], (int)vers);
 				return false;
 			}
 		}
-#endif				/* _NO_TCP_REGISTER */
 	}
+
+#ifndef _NO_TCP_REGISTER
+	LogInfo(COMPONENT_DISPATCH, "Registering %s V%d/TCP",
+		tags[prot], (int)vers);
+
+	if (!TCP_REGISTER(prot, vers, netconfig_tcpv4)) {
+		LogMajor(COMPONENT_DISPATCH,
+			 "Cannot register %s V%d on TCP", tags[prot],
+			 (int)vers);
+		return false;
+	}
+
+	if (!v6disabled && netconfig_tcpv6) {
+		LogInfo(COMPONENT_DISPATCH, "Registering %s V%d/TCPv6",
+			tags[prot], (int)vers);
+		if (!TCP_REGISTER(prot, vers, netconfig_tcpv6)) {
+			LogMajor(COMPONENT_DISPATCH,
+				 "Cannot register %s V%d on TCPv6",
+				 tags[prot], (int)vers);
+			return false;
+		}
+	}
+#endif				/* _NO_TCP_REGISTER */
 	return true;
 }
 
-static void Register_program(protos prot, int flag, int vers)
+static inline void Register_program(protos prot, int vers)
 {
-	if (!__Register_program(prot, flag, vers))
+	if (!__Register_program(prot, vers))
 		Fatal();
 }
 #endif /* RPCBIND */
@@ -1141,8 +1346,8 @@ void nfs_Init_svc(void)
 	    nfs_param.core_param.rpc.max_send_buffer_size;
 	svc_params.channels = N_EVENT_CHAN;
 	svc_params.idle_timeout = nfs_param.core_param.rpc.idle_timeout_s;
-	svc_params.ioq_thrd_max = /* max ioq worker threads */
-		nfs_param.core_param.rpc.ioq_thrd_max;
+	svc_params.ioq_thrd_min = nfs_param.core_param.rpc.ioq_thrd_min;
+	svc_params.ioq_thrd_max = nfs_param.core_param.rpc.ioq_thrd_max;
 	/* GSS ctx cache tuning, expiration */
 	svc_params.gss_ctx_hash_partitions =
 		nfs_param.core_param.rpc.gss.ctx_hash_partitions;
@@ -1252,26 +1457,32 @@ void nfs_Init_svc(void)
 	 */
 #ifdef _USE_NFS3
 	if (NFS_options & CORE_OPTION_NFSV3) {
-		Register_program(P_NFS, CORE_OPTION_NFSV3, NFS_V3);
-		Register_program(P_MNT, CORE_OPTION_NFSV3, MOUNT_V1);
-		Register_program(P_MNT, CORE_OPTION_NFSV3, MOUNT_V3);
+		Register_program(P_NFS, NFS_V3);
+		Register_program(P_MNT, MOUNT_V1);
+		Register_program(P_MNT, MOUNT_V3);
+#ifdef _USE_NLM
+		if (nfs_param.core_param.enable_NLM)
+			Register_program(P_NLM, NLM4_VERS);
+#endif /* _USE_NLM */
+#ifdef USE_NFSACL3
+		if (nfs_param.core_param.enable_NFSACL)
+			Register_program(P_NFSACL, NFSACL_V3);
+#endif
 	}
 #endif /* _USE_NFS3 */
+
 	/* v4 registration is optional */
 	if (NFS_options & CORE_OPTION_NFSV4)
-		__Register_program(P_NFS, CORE_OPTION_NFSV4, NFS_V4);
-#ifdef _USE_NLM
-	if (nfs_param.core_param.enable_NLM)
-		Register_program(P_NLM, CORE_OPTION_NFSV3, NLM4_VERS);
-#endif /* _USE_NLM */
+		__Register_program(P_NFS, NFS_V4);
+
+#ifdef _USE_RQUOTA
 	if (nfs_param.core_param.enable_RQUOTA &&
 	    (NFS_options & CORE_OPTION_ALL_NFS_VERS)) {
-		Register_program(P_RQUOTA, CORE_OPTION_ALL_VERS, RQUOTAVERS);
-		Register_program(P_RQUOTA, CORE_OPTION_ALL_VERS,
-				 EXT_RQUOTAVERS);
+		Register_program(P_RQUOTA, RQUOTAVERS);
+		Register_program(P_RQUOTA, EXT_RQUOTAVERS);
 	}
+#endif
 #endif	/* RPCBIND */
-
 }
 
 /**
@@ -1339,15 +1550,7 @@ static struct svc_req *alloc_nfs_request(SVCXPRT *xprt, XDR *xdrs)
 	reqdata->svc.rq_xdrs = xdrs;
 	reqdata->svc.rq_refcnt = 1;
 
-#if HAVE_BLKIN
-	blkin_init_new_trace(&reqdata->svc.bl_trace, "nfs-ganesha",
-			&xprt->blkin.endp);
-#endif
-
-#if defined(HAVE_BLKIN)
-	BLKIN_TIMESTAMP(
-		&reqdata->svc.bl_trace, &xprt->blkin.endp, "pre-recv");
-#endif
+	TAILQ_INIT_ENTRY(reqdata, dupes);
 
 	return &reqdata->svc;
 }
@@ -1357,17 +1560,6 @@ static void free_nfs_request(struct svc_req *req, enum xprt_stat stat)
 	nfs_request_t *reqdata = container_of(req, nfs_request_t, svc);
 	SVCXPRT *xprt = reqdata->svc.rq_xprt;
 
-#if defined(HAVE_BLKIN)
-	BLKIN_TIMESTAMP(
-		&reqdata->svc.bl_trace, &xprt->blkin.endp, "post-recv");
-
-	BLKIN_KEYVAL_INTEGER(
-		&reqdata->svc.bl_trace,
-		&reqdata->xprt->blkin.endp,
-		"rq-xid",
-		reqdata->svc.rq_xid);
-#endif
-
 	if (unlikely(stat > XPRT_DESTROYED)) {
 		LogInfo(COMPONENT_DISPATCH,
 			"SVC_DECODE on %p fd %d returned unknown %u",
@@ -1379,10 +1571,8 @@ static void free_nfs_request(struct svc_req *req, enum xprt_stat stat)
 		struct display_buffer dspbuf = {
 					sizeof(addrbuf), addrbuf, addrbuf};
 
-		if (copy_xprt_addr(&addr, xprt) == 1)
-			display_sockaddr(&dspbuf, &addr);
-		else
-			display_cat(&dspbuf, "<unresolved>");
+		copy_xprt_addr(&addr, xprt);
+		display_sockaddr(&dspbuf, &addr);
 
 		LogDebug(COMPONENT_DISPATCH,
 			 "SVC_DECODE on %p fd %d (%s) xid=%" PRIu32
