@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: LGPL-3.0-or-later
 /*
  * vim:noexpandtab:shiftwidth=8:tabstop=8:
  *
@@ -260,12 +261,11 @@ const nfs_function_desc_t nfs3_func_desc[] = {
 	 .xdr_decode_func = (xdrproc_t) xdr_COMMIT3args,
 	 .xdr_encode_func = (xdrproc_t) xdr_COMMIT3res,
 	 .funcname = "NFS3_COMMIT",
-	 .dispatch_behaviour = (MAKES_WRITE | NEEDS_CRED | SUPPORTS_GSS)
-	 }
+	 .dispatch_behaviour = (MAKES_WRITE | NEEDS_CRED | SUPPORTS_GSS)}
 };
 #endif /* _USE_NFS3 */
 
-/* Remeber that NFSv4 manages authentication though junction crossing, and
+/* Remember that NFSv4 manages authentication though junction crossing, and
  * so does it for RO FS management (for each operation) */
 const nfs_function_desc_t nfs4_func_desc[] = {
 	{
@@ -284,6 +284,7 @@ const nfs_function_desc_t nfs4_func_desc[] = {
 	 .dispatch_behaviour = CAN_BE_DUP}
 };
 
+#ifdef _USE_NFS3
 const nfs_function_desc_t mnt1_func_desc[] = {
 	{
 	 .service_function = mnt_Null,
@@ -373,11 +374,13 @@ const nfs_function_desc_t mnt3_func_desc[] = {
 	 .funcname = "MNT_EXPORT",
 	 .dispatch_behaviour = NOTHING_SPECIAL}
 };
+#endif
+
+#ifdef _USE_NLM
 
 #define nlm4_Unsupported nlm_Null
 #define nlm4_Unsupported_Free nlm_Null_Free
 
-#ifdef _USE_NLM
 const nfs_function_desc_t nlm4_func_desc[] = {
 	[NLMPROC4_NULL] = {
 			   .service_function = nlm_Null,
@@ -560,6 +563,7 @@ const nfs_function_desc_t nlm4_func_desc[] = {
 };
 #endif /* _USE_NLM */
 
+#ifdef _USE_RQUOTA
 const nfs_function_desc_t rquota1_func_desc[] = {
 	[0] = {
 	       .service_function = rquota_Null,
@@ -659,16 +663,43 @@ const nfs_function_desc_t rquota2_func_desc[] = {
 				       .funcname = "RQUOTA_EXT_GETACTIVEQUOTA",
 				       .dispatch_behaviour = NEEDS_CRED}
 };
+#endif
+
+#ifdef USE_NFSACL3
+const nfs_function_desc_t nfsacl_func_desc[] = {
+	[0] = {
+	       .service_function = nfsacl_Null,
+	       .free_function = nfsacl_Null_Free,
+	       .xdr_decode_func = (xdrproc_t) xdr_void,
+	       .xdr_encode_func = (xdrproc_t) xdr_void,
+	       .funcname = "NFSACL_NULL",
+	       .dispatch_behaviour = NOTHING_SPECIAL},
+	[NFSACLPROC_GETACL] = {
+				 .service_function = nfsacl_getacl,
+				 .free_function = nfsacl_getacl_Free,
+				 .xdr_decode_func =
+				 (xdrproc_t) xdr_getaclargs,
+				 .xdr_encode_func =
+				 (xdrproc_t) xdr_getaclres,
+				 .funcname = "NFSACL_GETACL",
+				 .dispatch_behaviour = NEEDS_CRED},
+	[NFSACLPROC_SETACL] = {
+				 .service_function = nfsacl_setacl,
+				 .free_function = nfsacl_setacl_Free,
+				 .xdr_decode_func =
+				 (xdrproc_t) xdr_setaclargs,
+				 .xdr_encode_func =
+				 (xdrproc_t) xdr_setaclres,
+				 .funcname = "NFSACL_SETACL",
+				 .dispatch_behaviour = NEEDS_CRED}
+};
+#endif
 
 void auth_failure(nfs_request_t *reqdata, enum auth_stat auth_rc)
 {
 	svcerr_auth(&reqdata->svc, auth_rc);
 	/* nb, a no-op when req is uncacheable */
-	if (nfs_dupreq_delete(&reqdata->svc) != DUPREQ_SUCCESS) {
-		LogCrit(COMPONENT_DISPATCH,
-			"Attempt to delete duplicate request failed on line %d",
-			__LINE__);
-	}
+	nfs_dupreq_delete(reqdata, NFS_REQ_AUTH_ERR);
 }
 
 void free_args(nfs_request_t *reqdata)
@@ -690,7 +721,7 @@ void free_args(nfs_request_t *reqdata)
 	}
 
 	/* Finalize the request. */
-	nfs_dupreq_rele(&reqdata->svc, reqdesc);
+	nfs_dupreq_rele(reqdata);
 
 	SetClientIP(NULL);
 
@@ -700,25 +731,18 @@ void free_args(nfs_request_t *reqdata)
 		op_ctx->client = NULL;
 	}
 
-	if (op_ctx->ctx_export != NULL) {
-		put_gsh_export(op_ctx->ctx_export);
-		op_ctx->ctx_export = NULL;
-	}
-
 	clean_credentials();
-	op_ctx = NULL;
+	release_op_context();
 
 #ifdef USE_LTTNG
 	tracepoint(nfs_rpc, end, reqdata);
 #endif
 }
 
-void complete_request(nfs_request_t *reqdata,
-		      int rc,
-		      dupreq_status_t dpq_status)
+enum nfs_req_result complete_request(nfs_request_t *reqdata,
+				     enum nfs_req_result rc)
 {
 	SVCXPRT *xprt = reqdata->svc.rq_xprt;
-	nfs_res_t *res_nfs = reqdata->res_nfs;
 	const nfs_function_desc_t *reqdesc = reqdata->funcdesc;
 
 	/* NFSv4 stats are handled in nfs4_compound() */
@@ -743,56 +767,47 @@ void complete_request(nfs_request_t *reqdata,
 		 * will be removed later.  We only remove a reply that is
 		 * normally cached that has been dropped.
 		 */
-		if (nfs_dupreq_delete(&reqdata->svc)
-		    != DUPREQ_SUCCESS) {
-			LogCrit(COMPONENT_DISPATCH,
-				"Attempt to delete duplicate request failed on line %d",
-				__LINE__);
-		}
-		return;
-	} else {
-		LogFullDebug(COMPONENT_DISPATCH,
-			     "Before svc_sendreply on socket %d", xprt->xp_fd);
+		nfs_dupreq_delete(reqdata, rc);
+		return rc;
+	}
 
-		reqdata->svc.rq_msg.RPCM_ack.ar_results.where = res_nfs;
-		reqdata->svc.rq_msg.RPCM_ack.ar_results.proc =
-					reqdesc->xdr_encode_func;
+	LogFullDebug(COMPONENT_DISPATCH,
+		     "Before svc_sendreply on socket %d", xprt->xp_fd);
+
+	reqdata->svc.rq_msg.RPCM_ack.ar_results.where = reqdata->res_nfs;
+	reqdata->svc.rq_msg.RPCM_ack.ar_results.proc =
+				reqdesc->xdr_encode_func;
 
 #ifdef USE_LTTNG
-		tracepoint(nfs_rpc, before_reply, __func__, __LINE__, xprt);
+	tracepoint(nfs_rpc, before_reply, __func__, __LINE__, xprt);
 #endif
-		if (svc_sendreply(&reqdata->svc) >= XPRT_DIED) {
-			LogDebug(COMPONENT_DISPATCH,
-				 "NFS DISPATCHER: FAILURE: Error while calling svc_sendreply on a new request. rpcxid=%"
-				 PRIu32
-				 " socket=%d function:%s client:%s program:%"
-				 PRIu32
-				 " nfs version:%" PRIu32
-				 " proc:%" PRIu32
-				 " errno: %d",
-				 reqdata->svc.rq_msg.rm_xid,
-				 xprt->xp_fd,
-				 reqdesc->funcname,
-				 op_ctx->client->hostaddr_str,
-				 reqdata->svc.rq_msg.cb_prog,
-				 reqdata->svc.rq_msg.cb_vers,
-				 reqdata->svc.rq_msg.cb_proc,
-				 errno);
-			SVC_DESTROY(xprt);
-			/* We failed to send the response, but the
-			 * request is complete, so we should mark
-			 * the same in our DRC.
-			 */
-		}
+	if (svc_sendreply(&reqdata->svc) >= XPRT_DIED) {
+		LogDebug(COMPONENT_DISPATCH,
+			 "NFS DISPATCHER: FAILURE: Error while calling svc_sendreply on a new request. rpcxid=%"
+			 PRIu32
+			 " socket=%d function:%s client:%s program:%"
+			 PRIu32
+			 " nfs version:%" PRIu32
+			 " proc:%" PRIu32
+			 " errno: %d",
+			 reqdata->svc.rq_msg.rm_xid,
+			 xprt->xp_fd,
+			 reqdesc->funcname,
+			 op_ctx->client->hostaddr_str,
+			 reqdata->svc.rq_msg.cb_prog,
+			 reqdata->svc.rq_msg.cb_vers,
+			 reqdata->svc.rq_msg.cb_proc,
+			 errno);
+		SVC_DESTROY(xprt);
+		rc = NFS_REQ_XPRT_DIED;
+	}
 
-		LogFullDebug(COMPONENT_DISPATCH,
-			     "After svc_sendreply on socket %d", xprt->xp_fd);
-
-	}			/* rc == NFS_REQ_DROP */
+	LogFullDebug(COMPONENT_DISPATCH,
+		     "After svc_sendreply on socket %d", xprt->xp_fd);
 
 	/* Finish any request not already deleted */
-	if (dpq_status == DUPREQ_SUCCESS)
-		(void) nfs_dupreq_finish(&reqdata->svc, res_nfs);
+	nfs_dupreq_finish(reqdata, rc);
+	return rc;
 }
 
 void complete_request_instrumentation(nfs_request_t *reqdata)
@@ -800,13 +815,6 @@ void complete_request_instrumentation(nfs_request_t *reqdata)
 #ifdef USE_LTTNG
 	tracepoint(nfs_rpc, op_end, reqdata);
 #endif
-
-#if defined(HAVE_BLKIN)
-	BLKIN_TIMESTAMP(
-		&reqdata->svc.bl_trace,
-		&reqdata->svc.rq_xprt->blkin.endp,
-		"nfs_rpc_process_request-post-service");
-#endif
 }
 
 /** @brief Completion of async RPC dispatch
@@ -822,17 +830,69 @@ void nfs_rpc_complete_async_request(nfs_request_t *reqdata,
 				    enum nfs_req_result rc)
 {
 	complete_request_instrumentation(reqdata);
-	complete_request(reqdata, rc, DUPREQ_SUCCESS);
+	rc = complete_request(reqdata, rc);
 	free_args(reqdata);
 }
 
+enum nfs_req_result process_dupreq(nfs_request_t *reqdata,
+				   const char *client_ip)
+{
+	enum xprt_stat xprt_rc;
+
+	/* Found the request in the dupreq cache.
+	 * Send cached reply. */
+	LogFullDebug(COMPONENT_DISPATCH,
+		     "DUP: DupReq Cache Hit: using previous reply, rpcxid=%"
+		     PRIu32,
+		     reqdata->svc.rq_msg.rm_xid);
+
+	LogFullDebug(COMPONENT_DISPATCH,
+		     "Before svc_sendreply on socket %d (dup req)",
+		     reqdata->svc.rq_xprt->xp_fd);
+
+	reqdata->svc.rq_msg.RPCM_ack.ar_results.where = reqdata->res_nfs;
+	reqdata->svc.rq_msg.RPCM_ack.ar_results.proc =
+				reqdata->funcdesc->xdr_encode_func;
+
+#ifdef USE_LTTNG
+	tracepoint(nfs_rpc, before_reply, __func__, __LINE__,
+		   reqdata->svc.rq_xprt);
+#endif
+	xprt_rc = svc_sendreply(&reqdata->svc);
+
+	if (xprt_rc >= XPRT_DIED) {
+		LogDebug(COMPONENT_DISPATCH,
+			 "NFS DISPATCHER: FAILURE: Error while calling svc_sendreply on a duplicate request. rpcxid=%"
+			 PRIu32
+			 " socket=%d function:%s client:%s program:%"
+			 PRIu32
+			 " nfs version:%" PRIu32
+			 " proc:%" PRIu32
+			 " errno: %d",
+			 reqdata->svc.rq_msg.rm_xid,
+			 reqdata->svc.rq_xprt->xp_fd,
+			 reqdata->funcdesc->funcname,
+			 client_ip,
+			 reqdata->svc.rq_msg.cb_prog,
+			 reqdata->svc.rq_msg.cb_vers,
+			 reqdata->svc.rq_msg.cb_proc,
+			 errno);
+		svcerr_systemerr(&reqdata->svc);
+		return NFS_REQ_XPRT_DIED;
+	}
+
+	return NFS_REQ_OK;
+}
+
 /**
  * @brief Main RPC dispatcher routine
  *
  * @param[in,out] reqdata	NFS request
+ * @param[in]     retry         True if a dupe request is coming back in
  *
  */
-static enum xprt_stat nfs_rpc_process_request(nfs_request_t *reqdata)
+static enum xprt_stat nfs_rpc_process_request(nfs_request_t *reqdata,
+					      bool retry)
 {
 	const char *client_ip = "<unknown client>";
 	const char *progname = "unknown";
@@ -840,11 +900,8 @@ static enum xprt_stat nfs_rpc_process_request(nfs_request_t *reqdata)
 	nfs_arg_t *arg_nfs = &reqdata->arg_nfs;
 	SVCXPRT *xprt = reqdata->svc.rq_xprt;
 	XDR *xdrs = reqdata->svc.rq_xdrs;
-	nfs_res_t *res_nfs;
 	dupreq_status_t dpq_status;
-	struct timespec timer_start;
-	enum auth_stat auth_rc;
-	enum xprt_stat xprt_rc;
+	enum auth_stat auth_rc = AUTH_OK;
 	int port;
 	int rc = NFS_REQ_OK;
 #ifdef _USE_NFS3
@@ -852,17 +909,13 @@ static enum xprt_stat nfs_rpc_process_request(nfs_request_t *reqdata)
 #endif /* _USE_NFS3 */
 	bool no_dispatch = false;
 
+	if (retry)
+		goto retry_after_drc_suspend;
+
 #ifdef USE_LTTNG
 	tracepoint(nfs_rpc, start, reqdata);
 #endif
 
-#if defined(HAVE_BLKIN)
-	BLKIN_TIMESTAMP(
-		&reqdata->svc.bl_trace,
-		&reqdata->svc.rq_xprt->blkin.endp,
-		"nfs_rpc_process_request-start");
-#endif
-
 	LogFullDebug(COMPONENT_DISPATCH,
 		     "About to authenticate Prog=%" PRIu32
 		     ", vers=%" PRIu32
@@ -955,12 +1008,9 @@ static enum xprt_stat nfs_rpc_process_request(nfs_request_t *reqdata)
 
 	/* set up the request context
 	 */
-	op_ctx = &reqdata->req_ctx;
-	op_ctx->creds = &reqdata->req_user_credentials;
-	op_ctx->caller_addr = (sockaddr_t *)svc_getrpccaller(xprt);
-	op_ctx->nfs_vers = reqdata->svc.rq_msg.cb_vers;
-	op_ctx->req_type = NFS_REQUEST;
-	op_ctx->export_perms = &reqdata->req_export_perms;
+	init_op_context(&reqdata->op_context, NULL, NULL,
+			(sockaddr_t *)svc_getrpccaller(xprt),
+			reqdata->svc.rq_msg.cb_vers, 0, NFS_REQUEST);
 
 	/* Set up initial export permissions that don't allow anything. */
 	export_check_access();
@@ -969,8 +1019,7 @@ static enum xprt_stat nfs_rpc_process_request(nfs_request_t *reqdata)
 	 * we measure all time stats as intervals (elapsed nsecs) from
 	 * server boot time.  This gets high precision with simple 64 bit math.
 	 */
-	now(&timer_start);
-	op_ctx->start_time = timespec_diff(&nfs_ServerBootTime, &timer_start);
+	now(&op_ctx->start_time);
 
 	/* Initialized user_credentials */
 	init_credentials();
@@ -984,8 +1033,8 @@ static enum xprt_stat nfs_rpc_process_request(nfs_request_t *reqdata)
 	 * this, we should sprint a buffer once, in when we're setting up
 	 * xprt private data. */
 
-	port = get_port(op_ctx->caller_addr);
 	op_ctx->client = get_gsh_client(op_ctx->caller_addr, false);
+
 	if (op_ctx->client == NULL) {
 		LogDebug(COMPONENT_DISPATCH,
 			 "Cannot get client block for Program %" PRIu32
@@ -1010,17 +1059,11 @@ static enum xprt_stat nfs_rpc_process_request(nfs_request_t *reqdata)
 			 reqdata->svc.rq_msg.rm_xid);
 	}
 
-#if defined(HAVE_BLKIN)
-	BLKIN_TIMESTAMP(
-		&reqdata->svc.bl_trace,
-		&reqdata->svc.rq_xprt->blkin.endp,
-		"nfs_rpc_process_request-have-clientid");
-#endif
 	/* If req is uncacheable, or if req is v41+, nfs_dupreq_start will do
 	 * nothing but allocate a result object and mark the request (ie, the
 	 * path is short, lockless, and does no hash/search). */
-	dpq_status = nfs_dupreq_start(reqdata, &reqdata->svc);
-	res_nfs = reqdata->res_nfs;
+	dpq_status = nfs_dupreq_start(reqdata);
+
 	if (dpq_status == DUPREQ_SUCCESS) {
 		/* A new request, continue processing it. */
 		LogFullDebug(COMPONENT_DISPATCH,
@@ -1028,73 +1071,33 @@ static enum xprt_stat nfs_rpc_process_request(nfs_request_t *reqdata)
 	} else {
 		switch (dpq_status) {
 		case DUPREQ_EXISTS:
-			/* Found the request in the dupreq cache.
-			 * Send cached reply. */
-			LogFullDebug(COMPONENT_DISPATCH,
-				     "DUP: DupReq Cache Hit: using previous reply, rpcxid=%"
-				     PRIu32,
-				     reqdata->svc.rq_msg.rm_xid);
-
-			LogFullDebug(COMPONENT_DISPATCH,
-				     "Before svc_sendreply on socket %d (dup req)",
-				     xprt->xp_fd);
-
-			reqdata->svc.rq_msg.RPCM_ack.ar_results.where =
-						res_nfs;
-			reqdata->svc.rq_msg.RPCM_ack.ar_results.proc =
-						reqdesc->xdr_encode_func;
-
-#ifdef USE_LTTNG
-			tracepoint(nfs_rpc, before_reply, __func__, __LINE__,
-				   xprt);
-#endif
-			xprt_rc = svc_sendreply(&reqdata->svc);
-			if (xprt_rc >= XPRT_DIED) {
-				LogDebug(COMPONENT_DISPATCH,
-					 "NFS DISPATCHER: FAILURE: Error while calling svc_sendreply on a duplicate request. rpcxid=%"
-					 PRIu32
-					 " socket=%d function:%s client:%s program:%"
-					 PRIu32
-					 " nfs version:%" PRIu32
-					 " proc:%" PRIu32
-					 " errno: %d",
-					 reqdata->svc.rq_msg.rm_xid,
-					 xprt->xp_fd,
-					 reqdesc->funcname,
-					 client_ip,
-					 reqdata->svc.rq_msg.cb_prog,
-					 reqdata->svc.rq_msg.cb_vers,
-					 reqdata->svc.rq_msg.cb_proc,
-					 errno);
-				svcerr_systemerr(&reqdata->svc);
-			}
+			(void) process_dupreq(reqdata, client_ip);
 			break;
 
 			/* Another thread owns the request */
 		case DUPREQ_BEING_PROCESSED:
 			LogFullDebug(COMPONENT_DISPATCH,
-				     "DUP: Request xid=%" PRIu32
+				     "Suspending DUP: Request xid=%" PRIu32
 				     " is already being processed; the active thread will reply",
 				     reqdata->svc.rq_msg.rm_xid);
+			/* The request is suspended, don't touch the request in
+			 * any way because the resume may already be scheduled
+			 * and running on nother thread. The xp_resume_cb has
+			 * already been set up before we started processing
+			 * ops on this request at all.
+			 */
+			suspend_op_context();
+			return XPRT_SUSPEND;
+
+		case DUPREQ_DROP:
+			LogFullDebug(COMPONENT_DISPATCH,
+				     "DUP: Request xid=%" PRIu32
+				     " is already being processed and has too many dupes queued",
+				     reqdata->svc.rq_msg.rm_xid);
 			/* Free the arguments */
 			/* Ignore the request, send no error */
 			break;
 
-			/* something is very wrong with
-			 * the duplicate request cache */
-		case DUPREQ_ERROR:
-			LogCrit(COMPONENT_DISPATCH,
-				"DUP: Did not find the request in the duplicate request cache and couldn't add the request.");
-			svcerr_systemerr(&reqdata->svc);
-			break;
-
-			/* oom */
-		case DUPREQ_INSERT_MALLOC_ERROR:
-			LogCrit(COMPONENT_DISPATCH,
-				"DUP: Cannot process request, not enough memory available!");
-			svcerr_systemerr(&reqdata->svc);
-			break;
-
 		default:
 			LogCrit(COMPONENT_DISPATCH,
 				"DUP: Unknown duplicate request cache status. This should never be reached!");
@@ -1105,6 +1108,14 @@ static enum xprt_stat nfs_rpc_process_request(nfs_request_t *reqdata)
 		goto freeargs;
 	}
 
+retry_after_drc_suspend:
+	/* If we come here on a retry after drc suspend, then we already did
+	 * the stuff above.
+	 */
+
+	/* We need the port below. */
+	port = get_port(op_ctx->caller_addr);
+
 	/* Don't waste time for null or invalid ops
 	 * null op code in all valid protos == 0
 	 * and invalid protos all point to invalid_funcdesc
@@ -1115,7 +1126,11 @@ static enum xprt_stat nfs_rpc_process_request(nfs_request_t *reqdata)
 	    || reqdata->svc.rq_msg.cb_proc == NFSPROC_NULL)
 		goto null_op;
 	/* Get the export entry */
-	if (reqdata->svc.rq_msg.cb_prog == NFS_program[P_NFS]) {
+	if (reqdata->svc.rq_msg.cb_prog == NFS_program[P_NFS]
+#ifdef USE_NFSACL3
+	    || reqdata->svc.rq_msg.cb_prog == NFS_program[P_NFSACL]
+#endif
+	    ) {
 		/* The NFSv3 functions' arguments always begin with the file
 		 * handle (but not the NULL function).  This hook is used to
 		 * get the fhandle with the arguments and so determine the
@@ -1134,13 +1149,13 @@ static enum xprt_stat nfs_rpc_process_request(nfs_request_t *reqdata)
 					client_ip);
 
 				/* Bad handle, report to client */
-				res_nfs->res_getattr3.status =
-				    NFS3ERR_BADHANDLE;
+				reqdata->res_nfs->res_getattr3.status =
+							NFS3ERR_BADHANDLE;
 				rc = NFS_REQ_OK;
 				goto req_error;
 			}
 
-			op_ctx->ctx_export = get_gsh_export(exportid);
+			set_op_context_export(get_gsh_export(exportid));
 
 			if (op_ctx->ctx_export == NULL) {
 				LogInfoAlt(COMPONENT_DISPATCH, COMPONENT_EXPORT,
@@ -1148,16 +1163,15 @@ static enum xprt_stat nfs_rpc_process_request(nfs_request_t *reqdata)
 					client_ip, exportid);
 
 				/* Bad export, report to client */
-				res_nfs->res_getattr3.status = NFS3ERR_STALE;
+				reqdata->res_nfs->res_getattr3.status =
+								NFS3ERR_STALE;
 				rc = NFS_REQ_OK;
 				goto req_error;
 			}
 
-			op_ctx->fsal_export = op_ctx->ctx_export->fsal_export;
-
 			LogMidDebugAlt(COMPONENT_DISPATCH, COMPONENT_EXPORT,
 				    "Found export entry for path=%s as exportid=%d",
-				    op_ctx_export_path(op_ctx->ctx_export),
+				    op_ctx_export_path(op_ctx),
 				    op_ctx->ctx_export->export_id);
 		}
 #endif /* _USE_NFS3 */
@@ -1213,12 +1227,6 @@ static enum xprt_stat nfs_rpc_process_request(nfs_request_t *reqdata)
 			exportid = nlm4_FhandleToExportId(pfh3);
 
 			if (exportid < 0) {
-				LogInfo(COMPONENT_DISPATCH,
-					"NLM4 Request from client %s has badly formed handle",
-					client_ip);
-				op_ctx->ctx_export = NULL;
-				op_ctx->fsal_export = NULL;
-
 				/* We need to send a NLM4_STALE_FH response
 				 * (NLM doesn't have an error code for
 				 * BADHANDLE), but we don't know how to do that
@@ -1226,8 +1234,12 @@ static enum xprt_stat nfs_rpc_process_request(nfs_request_t *reqdata)
 				 * routine to let it know what to do since it
 				 * can respond to ASYNC calls.
 				 */
+				LogInfo(COMPONENT_DISPATCH,
+					"NLM4 Request from client %s has badly formed handle",
+					client_ip);
+
 			} else {
-				op_ctx->ctx_export = get_gsh_export(exportid);
+				set_op_context_export(get_gsh_export(exportid));
 
 				if (op_ctx->ctx_export == NULL) {
 					LogInfoAlt(COMPONENT_DISPATCH,
@@ -1244,23 +1256,20 @@ static enum xprt_stat nfs_rpc_process_request(nfs_request_t *reqdata)
 					 * to let it know what to do since it
 					 * can respond to ASYNC calls.
 					 */
-					op_ctx->fsal_export = NULL;
 				} else {
-					op_ctx->fsal_export =
-					    op_ctx->ctx_export->fsal_export;
-
 					LogMidDebugAlt(COMPONENT_DISPATCH,
 						COMPONENT_EXPORT,
 						"Found export entry for dirname=%s as exportid=%d",
-						op_ctx_export_path(
-							op_ctx->ctx_export),
+						ctx_export_path(op_ctx),
 						op_ctx->ctx_export->export_id);
 				}
 			}
 		}
 #endif /* _USE_NLM */
+#ifdef _USE_NFS3
 	} else if (reqdata->svc.rq_msg.cb_prog == NFS_program[P_MNT]) {
 		progname = "MNT";
+#endif /* _USE_NFS3 */
 	}
 
 	/* Only do access check if we have an export. */
@@ -1274,14 +1283,14 @@ static enum xprt_stat nfs_rpc_process_request(nfs_request_t *reqdata)
 
 		export_check_access();
 
-		if ((reqdata->req_export_perms.options &
+		if ((op_ctx->export_perms.options &
 		     EXPORT_OPTION_ACCESS_MASK) == 0) {
 			LogInfoAlt(COMPONENT_DISPATCH, COMPONENT_EXPORT,
 				"Client %s is not allowed to access Export_Id %d %s, vers=%"
 				PRIu32 ", proc=%" PRIu32,
 				client_ip,
 				op_ctx->ctx_export->export_id,
-				op_ctx_export_path(op_ctx->ctx_export),
+				op_ctx_export_path(op_ctx),
 				reqdata->svc.rq_msg.cb_vers,
 				reqdata->svc.rq_msg.cb_proc);
 
@@ -1289,7 +1298,7 @@ static enum xprt_stat nfs_rpc_process_request(nfs_request_t *reqdata)
 			goto freeargs;
 		}
 
-		if ((reqdata->req_export_perms.options &
+		if ((op_ctx->export_perms.options &
 		     EXPORT_OPTION_NFSV3) == 0) {
 			LogInfoAlt(COMPONENT_DISPATCH, COMPONENT_EXPORT,
 				"%s Version %" PRIu32
@@ -1297,7 +1306,7 @@ static enum xprt_stat nfs_rpc_process_request(nfs_request_t *reqdata)
 				progname,
 				reqdata->svc.rq_msg.cb_vers,
 				op_ctx->ctx_export->export_id,
-				op_ctx_export_path(op_ctx->ctx_export),
+				op_ctx_export_path(op_ctx),
 				client_ip);
 
 			auth_failure(reqdata, AUTH_FAILED);
@@ -1306,10 +1315,10 @@ static enum xprt_stat nfs_rpc_process_request(nfs_request_t *reqdata)
 
 		/* Check transport type */
 		if (((xprt_type == XPRT_UDP)
-		     && ((reqdata->req_export_perms.options &
+		     && ((op_ctx->export_perms.options &
 			  EXPORT_OPTION_UDP) == 0))
 		    || ((xprt_type == XPRT_TCP)
-			&& ((reqdata->req_export_perms.options &
+			&& ((op_ctx->export_perms.options &
 			     EXPORT_OPTION_TCP) == 0))) {
 			LogInfoAlt(COMPONENT_DISPATCH, COMPONENT_EXPORT,
 				"%s Version %" PRIu32
@@ -1318,7 +1327,7 @@ static enum xprt_stat nfs_rpc_process_request(nfs_request_t *reqdata)
 				reqdata->svc.rq_msg.cb_vers,
 				xprt_type_to_str(xprt_type),
 				op_ctx->ctx_export->export_id,
-				op_ctx_export_path(op_ctx->ctx_export),
+				op_ctx_export_path(op_ctx),
 				client_ip);
 
 			auth_failure(reqdata, AUTH_FAILED);
@@ -1334,7 +1343,7 @@ static enum xprt_stat nfs_rpc_process_request(nfs_request_t *reqdata)
 				progname,
 				reqdata->svc.rq_msg.cb_vers,
 				op_ctx->ctx_export->export_id,
-				op_ctx_export_path(op_ctx->ctx_export),
+				op_ctx_export_path(op_ctx),
 				client_ip);
 
 			auth_failure(reqdata, AUTH_TOOWEAK);
@@ -1344,13 +1353,13 @@ static enum xprt_stat nfs_rpc_process_request(nfs_request_t *reqdata)
 		/* Check if client is using a privileged port,
 		 * but only for NFS protocol */
 		if ((reqdata->svc.rq_msg.cb_prog == NFS_program[P_NFS])
-		    && (reqdata->req_export_perms.options &
+		    && (op_ctx->export_perms.options &
 			EXPORT_OPTION_PRIVILEGED_PORT)
 		    && (port >= IPPORT_RESERVED)) {
 			LogInfoAlt(COMPONENT_DISPATCH, COMPONENT_EXPORT,
 				"Non-reserved Port %d is not allowed on Export_Id %d %s for client %s",
 				port, op_ctx->ctx_export->export_id,
-				op_ctx_export_path(op_ctx->ctx_export),
+				op_ctx_export_path(op_ctx),
 				client_ip);
 
 			auth_failure(reqdata, AUTH_TOOWEAK);
@@ -1364,7 +1373,7 @@ static enum xprt_stat nfs_rpc_process_request(nfs_request_t *reqdata)
 	 */
 	if (op_ctx->ctx_export != NULL
 	    && (reqdesc->dispatch_behaviour & MAKES_IO)
-	    && !(reqdata->req_export_perms.options & EXPORT_OPTION_RW_ACCESS)) {
+	    && !(op_ctx->export_perms.options & EXPORT_OPTION_RW_ACCESS)) {
 		/* Request of type MDONLY_RO were rejected at the
 		 * nfs_rpc_dispatcher level.
 		 * This is done by replying EDQUOT
@@ -1378,7 +1387,8 @@ static enum xprt_stat nfs_rpc_process_request(nfs_request_t *reqdata)
 				LogDebugAlt(COMPONENT_DISPATCH,
 					    COMPONENT_EXPORT,
 					    "Returning NFS3ERR_DQUOT because request is on an MD Only export");
-				res_nfs->res_getattr3.status = NFS3ERR_DQUOT;
+				reqdata->res_nfs->res_getattr3.status =
+								NFS3ERR_DQUOT;
 				rc = NFS_REQ_OK;
 				break;
 #endif /* _USE_NFS3 */
@@ -1396,7 +1406,7 @@ static enum xprt_stat nfs_rpc_process_request(nfs_request_t *reqdata)
 		}
 	} else if (op_ctx->ctx_export != NULL
 		   && (reqdesc->dispatch_behaviour & MAKES_WRITE)
-		   && (reqdata->req_export_perms.options
+		   && (op_ctx->export_perms.options
 		       & (EXPORT_OPTION_WRITE_ACCESS
 			| EXPORT_OPTION_MD_WRITE_ACCESS)) == 0) {
 		if (reqdata->svc.rq_msg.cb_prog == NFS_program[P_NFS])
@@ -1406,7 +1416,8 @@ static enum xprt_stat nfs_rpc_process_request(nfs_request_t *reqdata)
 				LogDebugAlt(COMPONENT_DISPATCH,
 					    COMPONENT_EXPORT,
 					    "Returning NFS3ERR_ROFS because request is on a Read Only export");
-				res_nfs->res_getattr3.status = NFS3ERR_ROFS;
+				reqdata->res_nfs->res_getattr3.status =
+								NFS3ERR_ROFS;
 				rc = NFS_REQ_OK;
 				break;
 #endif /* _USE_NFS3 */
@@ -1423,14 +1434,14 @@ static enum xprt_stat nfs_rpc_process_request(nfs_request_t *reqdata)
 			rc = NFS_REQ_DROP;
 		}
 	} else if (op_ctx->ctx_export != NULL
-		   && (reqdata->req_export_perms.options
+		   && (op_ctx->export_perms.options
 		       & (EXPORT_OPTION_READ_ACCESS
 			 | EXPORT_OPTION_MD_READ_ACCESS)) == 0) {
 		LogInfoAlt(COMPONENT_DISPATCH, COMPONENT_EXPORT,
 			"Client %s is not allowed to access Export_Id %d %s, vers=%"
 			PRIu32 ", proc=%" PRIu32,
 			client_ip, op_ctx->ctx_export->export_id,
-			op_ctx_export_path(op_ctx->ctx_export),
+			op_ctx_export_path(op_ctx),
 			reqdata->svc.rq_msg.cb_vers,
 			reqdata->svc.rq_msg.cb_proc);
 		auth_failure(reqdata, AUTH_TOOWEAK);
@@ -1440,7 +1451,7 @@ static enum xprt_stat nfs_rpc_process_request(nfs_request_t *reqdata)
 		if (reqdesc->dispatch_behaviour & NEEDS_CRED) {
 			/* If we don't have an export, don't squash */
 			if (op_ctx->fsal_export == NULL) {
-				reqdata->req_export_perms.options &=
+				op_ctx->export_perms.options &=
 					~EXPORT_OPTION_SQUASH_TYPES;
 			} else if (nfs_req_creds(&reqdata->svc) != NFS4_OK) {
 				LogInfoAlt(COMPONENT_DISPATCH, COMPONENT_EXPORT,
@@ -1485,37 +1496,17 @@ static enum xprt_stat nfs_rpc_process_request(nfs_request_t *reqdata)
 			    ? op_ctx->ctx_export->export_id : -1));
 #endif
 
-#if defined(HAVE_BLKIN)
-		BLKIN_TIMESTAMP(
-			&reqdata->svc.bl_trace,
-			&reqdata->svc.rq_xprt->blkin.endp,
-			"nfs_rpc_process_request-pre-service");
-
-		BLKIN_KEYVAL_STRING(
-			&reqdata->svc.bl_trace,
-			&reqdata->svc.rq_xprt->blkin.endp,
-			"op-name",
-			reqdesc->funcname
-			);
-
-		BLKIN_KEYVAL_INTEGER(
-			&reqdata->svc.bl_trace,
-			&reqdata->svc.rq_xprt->blkin.endp,
-			"export-id",
-			(op_ctx->ctx_export != NULL)
-			? op_ctx->ctx_export->export_id : -1);
-#endif
 		rc = reqdesc->service_function(arg_nfs, &reqdata->svc,
-					res_nfs);
+					       reqdata->res_nfs);
 
 		if (rc == NFS_REQ_ASYNC_WAIT) {
 			/* The request is suspended, don't touch the request in
 			 * any way because the resume may already be scheduled
 			 * and running on nother thread. The xp_resume_cb has
-			 * already been set up before we started proecessing
+			 * already been set up before we started processing
 			 * ops on this request at all.
 			 */
-			op_ctx = NULL;
+			suspend_op_context();
 			return XPRT_SUSPEND;
 		}
 
@@ -1526,15 +1517,101 @@ static enum xprt_stat nfs_rpc_process_request(nfs_request_t *reqdata)
  req_error:
 #endif /* _USE_NFS3 */
 
-	complete_request(reqdata, rc, dpq_status);
+	rc = complete_request(reqdata, rc);
 
  freeargs:
 
 	free_args(reqdata);
 
+	/* Make sure no-one called init_op_context() without calling
+	 * release_op_context() */
+	assert(op_ctx == NULL);
+	/* Make sure we return to ntirpc without op_ctx set, or saved_op_ctx can
+	 * point to freed memory */
+	op_ctx = NULL;
 	return SVC_STAT(xprt);
 }
 
+/** @brief Resume a duplicate request
+ *
+ * When a duplicate request comes in while still processing the original request
+ * we suspend it, and queue it in the dupreq_entry_t. Now the original request
+ * has been finished and we are ready to respond to the duplicate request.
+ *
+ */
+enum xprt_stat drc_resume(struct svc_req *req)
+{
+	nfs_request_t *reqdata = container_of(req, nfs_request_t, svc);
+	int rc = NFS_REQ_OK;
+
+	/* Restore the op_ctx */
+	resume_op_context(&reqdata->op_context);
+
+	/* Get the result of sending the response. Note that we are ONLY here
+	 * if this request came in as a dupe while we were processing the
+	 * original reqyest and it has now completed. So either we just sent a
+	 * response and there is no need to respond to this dupe OR something
+	 * went wrong that warrants further attention.
+	 */
+	rc = nfs_dupreq_reply_rc(reqdata);
+
+	server_stats_nfs_done(reqdata, rc, true);
+
+	switch (rc) {
+	case NFS_REQ_OK:
+	case NFS_REQ_ERROR:
+		/* In these cases, we don't need to respond */
+		LogFullDebug(COMPONENT_DISPATCH,
+			     "Suspended DUP: Request xid=%" PRIu32
+			     " was processed and replied to",
+			     reqdata->svc.rq_msg.rm_xid);
+		break;
+
+	case NFS_REQ_DROP:
+		/* The original request was dropped for some reason, we actually
+		 * need to process now... Resubmit this request.
+		 */
+		return nfs_rpc_process_request(reqdata, true);
+
+	case NFS_REQ_REPLAY:
+	case NFS_REQ_ASYNC_WAIT:
+		/* These cases can never happen... */
+		break;
+
+	case NFS_REQ_XPRT_DIED:
+		/* The attempt to send a response failed because the XPRT died
+		 * so now we need to actually resend the response here.
+		 */
+		rc = process_dupreq(reqdata, op_ctx->client != NULL
+					     ? op_ctx->client->hostaddr_str
+					     : "<unknown client>");
+
+		/* And now we need to try and finish this request. If we had
+		 * another XPRT_DIED and there's yet another duplicate request
+		 * queued, we will be back around with that request shortly...
+		 */
+		nfs_dupreq_finish(reqdata, rc);
+		break;
+
+	case NFS_REQ_AUTH_ERR:
+		/* The original request failed due to an auth error. This retry
+		 * will probably also similarly fail, but we will re-submit
+		 * anyway.
+		 */
+		return nfs_rpc_process_request(reqdata, true);
+	}
+
+	free_args(reqdata);
+
+	/* Make sure no-one called init_op_context() without calling
+	 * release_op_context() */
+	assert(op_ctx == NULL);
+	/* Make sure we return to ntirpc without op_ctx set, or saved_op_ctx can
+	 * point to freed memory */
+	op_ctx = NULL;
+	return SVC_STAT(reqdata->svc.rq_xprt);
+}
+
 /**
  * @brief Report Invalid Program number
  *
@@ -1614,6 +1691,18 @@ enum xprt_stat nfs_rpc_valid_NFS(struct svc_req *req)
 
 	reqdata->funcdesc = &invalid_funcdesc;
 
+#ifdef USE_NFSACL3
+	if (req->rq_msg.cb_prog == NFS_program[P_NFSACL]) {
+		if (req->rq_msg.cb_vers == NFSACL_V3 && CORE_OPTION_NFSV3) {
+			if (req->rq_msg.cb_proc <= NFSACLPROC_SETACL) {
+				reqdata->funcdesc =
+					&nfsacl_func_desc[req->rq_msg.cb_proc];
+				return nfs_rpc_process_request(reqdata, false);
+			}
+		}
+	}
+#endif /* USE_NFSACL3 */
+
 	if (req->rq_msg.cb_prog != NFS_program[P_NFS]) {
 		return nfs_rpc_noprog(reqdata);
 	}
@@ -1622,7 +1711,7 @@ enum xprt_stat nfs_rpc_valid_NFS(struct svc_req *req)
 		if (req->rq_msg.cb_proc <= NFSPROC4_COMPOUND) {
 			reqdata->funcdesc =
 				&nfs4_func_desc[req->rq_msg.cb_proc];
-			return nfs_rpc_process_request(reqdata);
+			return nfs_rpc_process_request(reqdata, false);
 		}
 		return nfs_rpc_noproc(reqdata);
 	}
@@ -1632,7 +1721,7 @@ enum xprt_stat nfs_rpc_valid_NFS(struct svc_req *req)
 		if (req->rq_msg.cb_proc <= NFSPROC3_COMMIT) {
 			reqdata->funcdesc =
 				&nfs3_func_desc[req->rq_msg.cb_proc];
-			return nfs_rpc_process_request(reqdata);
+			return nfs_rpc_process_request(reqdata, false);
 		}
 		return nfs_rpc_noproc(reqdata);
 	}
@@ -1655,6 +1744,7 @@ enum xprt_stat nfs_rpc_valid_NFS(struct svc_req *req)
 	return nfs_rpc_novers(reqdata, lo_vers, hi_vers);
 }
 
+#ifdef _USE_NLM
 enum xprt_stat nfs_rpc_valid_NLM(struct svc_req *req)
 {
 	nfs_request_t *reqdata =
@@ -1662,23 +1752,23 @@ enum xprt_stat nfs_rpc_valid_NLM(struct svc_req *req)
 
 	reqdata->funcdesc = &invalid_funcdesc;
 
-#ifdef _USE_NLM
 	if (req->rq_msg.cb_prog == NFS_program[P_NLM]
 	     && (NFS_options & CORE_OPTION_NFSV3)) {
 		if (req->rq_msg.cb_vers == NLM4_VERS) {
 			if (req->rq_msg.cb_proc <= NLMPROC4_FREE_ALL) {
 				reqdata->funcdesc =
 					&nlm4_func_desc[req->rq_msg.cb_proc];
-				return nfs_rpc_process_request(reqdata);
+				return nfs_rpc_process_request(reqdata, false);
 			}
 			return nfs_rpc_noproc(reqdata);
 		}
 		return nfs_rpc_novers(reqdata, NLM4_VERS, NLM4_VERS);
 	}
-#endif /* _USE_NLM */
 	return nfs_rpc_noprog(reqdata);
 }
+#endif /* _USE_NLM */
 
+#ifdef _USE_NFS3
 enum xprt_stat nfs_rpc_valid_MNT(struct svc_req *req)
 {
 	nfs_request_t *reqdata =
@@ -1700,7 +1790,7 @@ enum xprt_stat nfs_rpc_valid_MNT(struct svc_req *req)
 			if (req->rq_msg.cb_proc <= MOUNTPROC3_EXPORT) {
 				reqdata->funcdesc =
 					&mnt3_func_desc[req->rq_msg.cb_proc];
-				return nfs_rpc_process_request(reqdata);
+				return nfs_rpc_process_request(reqdata, false);
 			}
 			return nfs_rpc_noproc(reqdata);
 		}
@@ -1709,7 +1799,7 @@ enum xprt_stat nfs_rpc_valid_MNT(struct svc_req *req)
 			    && req->rq_msg.cb_proc != MOUNTPROC2_MNT) {
 				reqdata->funcdesc =
 					&mnt1_func_desc[req->rq_msg.cb_proc];
-				return nfs_rpc_process_request(reqdata);
+				return nfs_rpc_process_request(reqdata, false);
 			}
 			return nfs_rpc_noproc(reqdata);
 		}
@@ -1717,7 +1807,9 @@ enum xprt_stat nfs_rpc_valid_MNT(struct svc_req *req)
 	}
 	return nfs_rpc_noprog(reqdata);
 }
+#endif
 
+#ifdef _USE_RQUOTA
 enum xprt_stat nfs_rpc_valid_RQUOTA(struct svc_req *req)
 {
 	nfs_request_t *reqdata =
@@ -1730,7 +1822,7 @@ enum xprt_stat nfs_rpc_valid_RQUOTA(struct svc_req *req)
 			if (req->rq_msg.cb_proc <= RQUOTAPROC_SETACTIVEQUOTA) {
 				reqdata->funcdesc =
 					&rquota2_func_desc[req->rq_msg.cb_proc];
-				return nfs_rpc_process_request(reqdata);
+				return nfs_rpc_process_request(reqdata, false);
 			}
 			return nfs_rpc_noproc(reqdata);
 		}
@@ -1738,7 +1830,7 @@ enum xprt_stat nfs_rpc_valid_RQUOTA(struct svc_req *req)
 			if (req->rq_msg.cb_proc <= RQUOTAPROC_SETACTIVEQUOTA) {
 				reqdata->funcdesc =
 					&rquota1_func_desc[req->rq_msg.cb_proc];
-				return nfs_rpc_process_request(reqdata);
+				return nfs_rpc_process_request(reqdata, false);
 			}
 			return nfs_rpc_noproc(reqdata);
 		}
@@ -1746,3 +1838,27 @@ enum xprt_stat nfs_rpc_valid_RQUOTA(struct svc_req *req)
 	}
 	return nfs_rpc_noprog(reqdata);
 }
+#endif
+
+#ifdef USE_NFSACL3
+enum xprt_stat nfs_rpc_valid_NFSACL(struct svc_req *req)
+{
+	nfs_request_t *reqdata =
+			container_of(req, struct nfs_request, svc);
+
+	reqdata->funcdesc = &invalid_funcdesc;
+
+	if (req->rq_msg.cb_prog == NFS_program[P_NFSACL]) {
+		if (req->rq_msg.cb_vers == NFSACL_V3) {
+			if (req->rq_msg.cb_proc <= NFSACLPROC_SETACL) {
+				reqdata->funcdesc =
+					&nfsacl_func_desc[req->rq_msg.cb_proc];
+				return nfs_rpc_process_request(reqdata, false);
+			}
+			return nfs_rpc_noproc(reqdata);
+		}
+		return nfs_rpc_novers(reqdata, NFSACL_V3, NFSACL_V3);
+	}
+	return nfs_rpc_noprog(reqdata);
+}
+#endif
