@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: LGPL-3.0-or-later
 /*
  * vim:noexpandtab:shiftwidth=8:tabstop=8:
  *
@@ -51,7 +52,7 @@ int _9p_getattr(struct _9p_request_data *req9p, u32 *plenout, char *preply)
 	u32 *fid = NULL;
 	u64 *request_mask = NULL;
 	fsal_status_t fsal_status;
-	struct attrlist attrs;
+	struct fsal_attrlist attrs;
 
 	struct _9p_fid *pfid = NULL;
 
@@ -136,7 +137,6 @@ int _9p_getattr(struct _9p_request_data *req9p, u32 *plenout, char *preply)
 	} else
 		mode = 0;
 
-	/** @todo this is racy, use cache_inode_lock_trust_attrs */
 	uid =
 	    (*request_mask & _9P_GETATTR_UID) ?
 		(u32) attrs.owner :
