@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: LGPL-3.0-or-later
 /*
  * vim:noexpandtab:shiftwidth=8:tabstop=8:
  *
@@ -64,7 +65,7 @@ int _9p_lcreate(struct _9p_request_data *req9p, u32 *plenout, char *preply)
 	fsal_status_t fsal_status;
 	fsal_openflags_t openflags = 0;
 
-	struct attrlist sattr;
+	struct fsal_attrlist sattr;
 	fsal_verifier_t verifier;
 	enum fsal_create_mode createmode = FSAL_UNCHECKED;
 
@@ -94,8 +95,7 @@ int _9p_lcreate(struct _9p_request_data *req9p, u32 *plenout, char *preply)
 	}
 
 	_9p_init_opctx(pfid, req9p);
-	if ((op_ctx->export_perms->options &
-				 EXPORT_OPTION_WRITE_ACCESS) == 0)
+	if ((op_ctx->export_perms.options & EXPORT_OPTION_WRITE_ACCESS) == 0)
 		return _9p_rerror(req9p, msgtag, EROFS, plenout, preply);
 
 	if (*name_len >= sizeof(file_name)) {
