@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: LGPL-3.0-or-later
 /*
  * vim:noexpandtab:shiftwidth=8:tabstop=8:
  *
@@ -55,7 +56,7 @@
 char *strtype[] = { "RDLOCK", "WRLOCK", "UNLOCK" };
 
 /*
- * Remonder:
+ * Reminder:
  * LOCK_SUCCESS = 0
  * LOCK_BLOCKED = 1
  * LOCK_ERROR   = 2
@@ -87,7 +88,7 @@ int _9p_lock(struct _9p_request_data *req9p, u32 *plenout, char *preply)
 	char name[MAXNAMLEN+1];
 
 	struct addrinfo hints, *result;
-	struct sockaddr_storage client_addr;
+	sockaddr_t client_addr;
 
 	struct _9p_fid *pfid = NULL;
 
