@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: LGPL-3.0-or-later
 /*
  * vim:noexpandtab:shiftwidth=8:tabstop=8:
  *
@@ -55,7 +56,7 @@
  * @param[in]  req     SVC request related to this call
  * @param[out] res     Structure to contain the result of the call
  *
- * @retval NFS_REQ_OK if successfull
+ * @retval NFS_REQ_OK if successful
  * @retval NFS_REQ_DROP if failed but retryable
  * @retval NFS_REQ_FAILED if failed and not retryable
  */
@@ -64,21 +65,15 @@ int nfs3_getattr(nfs_arg_t *arg, struct svc_req *req, nfs_res_t *res)
 {
 	struct fsal_obj_handle *obj = NULL;
 	int rc = NFS_REQ_OK;
-	struct attrlist attrs;
+	struct fsal_attrlist *attrs;
 	fsal_status_t status;
 
-	if (isDebug(COMPONENT_NFSPROTO)) {
-		char str[LEN_FH_STR];
+	attrs = &res->res_getattr3.GETATTR3res_u.resok.obj_attributes;
 
-		nfs_FhandleToStr(req->rq_msg.cb_vers,
-				 &(arg->arg_getattr3.object),
-				 NULL, str);
-		LogDebug(COMPONENT_NFSPROTO,
-			 "REQUEST PROCESSING: Calling NFS3_GETATTR handle: %s",
-			 str);
-	}
+	LogNFS3_Operation(COMPONENT_NFSPROTO, req, &arg->arg_getattr3.object,
+			  "");
 
-	fsal_prepare_attrs(&attrs, ATTRS_NFS3);
+	fsal_prepare_attrs(attrs, ATTRS_NFS3);
 
 	obj = nfs3_FhandleToCache(&arg->arg_getattr3.object,
 				    &res->res_getattr3.status,
@@ -92,7 +87,7 @@ int nfs3_getattr(nfs_arg_t *arg, struct svc_req *req, nfs_res_t *res)
 		goto out;
 	}
 
-	status = obj->obj_ops->getattrs(obj, &attrs);
+	status = obj->obj_ops->getattrs(obj, attrs);
 
 	if (FSAL_IS_ERROR(status)) {
 		res->res_getattr3.status = nfs3_Errno_status(status);
@@ -104,9 +99,7 @@ int nfs3_getattr(nfs_arg_t *arg, struct svc_req *req, nfs_res_t *res)
 		goto out;
 	}
 
-	nfs3_FSALattr_To_Fattr(
-			obj, &attrs,
-			&res->res_getattr3.GETATTR3res_u.resok.obj_attributes);
+	nfs3_Fixup_FSALattr(obj, attrs);
 
 	res->res_getattr3.status = NFS3_OK;
 
@@ -115,8 +108,9 @@ int nfs3_getattr(nfs_arg_t *arg, struct svc_req *req, nfs_res_t *res)
 
  out:
 
-	/* Done with the attrs */
-	fsal_release_attrs(&attrs);
+	/* Done with the attrs (NFSv3 doesn't use ANY of the reffed attributes)
+	 */
+	fsal_release_attrs(attrs);
 
 	/* return references */
 	if (obj)
