@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: LGPL-3.0-or-later
 /*
  * vim:noexpandtab:shiftwidth=8:tabstop=8:
  *
@@ -71,7 +72,7 @@ int nfs3_symlink(nfs_arg_t *arg, struct svc_req *req, nfs_res_t *res)
 	pre_op_attr pre_parent;
 	fsal_status_t fsal_status;
 	int rc = NFS_REQ_OK;
-	struct attrlist sattr, attrs;
+	struct fsal_attrlist sattr, attrs;
 	SYMLINK3resfail *resfail = &res->res_symlink3.SYMLINK3res_u.resfail;
 	SYMLINK3resok *resok = &res->res_symlink3.SYMLINK3res_u.resok;
 
@@ -109,10 +110,10 @@ int nfs3_symlink(nfs_arg_t *arg, struct svc_req *req, nfs_res_t *res)
 	/* if quota support is active, then we should check is the
 	 * FSAL allows inode creation or not
 	 */
-	fsal_status =
-	    op_ctx->fsal_export->exp_ops.check_quota(op_ctx->fsal_export,
-						   op_ctx->ctx_export->fullpath,
-						   FSAL_QUOTA_INODES);
+	fsal_status = op_ctx->fsal_export->exp_ops.check_quota(
+							op_ctx->fsal_export,
+							CTX_FULLPATH(op_ctx),
+							FSAL_QUOTA_INODES);
 
 	if (FSAL_IS_ERROR(fsal_status)) {
 		res->res_symlink3.status = NFS3ERR_DQUOT;
