@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: LGPL-3.0-or-later
 /*
  * vim:noexpandtab:shiftwidth=8:tabstop=8:
  *
@@ -49,7 +50,7 @@
 /**
  * @brief NFS4_OP_ACCESS, checks for file's accessibility.
  *
- * This function impelments the NFS4_OP_ACCESS operation, which checks
+ * This function implements the NFS4_OP_ACCESS operation, which checks
  * for file's accessibility.
  *
  * @param[in]     op   Arguments for nfs4_op
@@ -69,6 +70,13 @@ enum nfs_req_result nfs4_op_access(struct nfs_argop4 *op, compound_data_t *data,
 			       ACCESS4_MODIFY | ACCESS4_EXTEND |
 			       ACCESS4_DELETE | ACCESS4_EXECUTE);
 
+	/* xattrs are a v4.2+ feature */
+	if (data->minorversion >= 2) {
+		max_access |= ACCESS4_XAREAD |
+			      ACCESS4_XAWRITE |
+			      ACCESS4_XALIST;
+	}
+
 	/* initialize output */
 	res_ACCESS4->ACCESS4res_u.resok4.supported = 0;
 	res_ACCESS4->ACCESS4res_u.resok4.access = 0;
@@ -105,7 +113,7 @@ enum nfs_req_result nfs4_op_access(struct nfs_argop4 *op, compound_data_t *data,
  * @brief Free memory allocated for ACCESS result
  *
  * This function frees any memory allocated for the result of the
- * NFS4_OP_ACCESS operatino.
+ * NFS4_OP_ACCESS operation.
  *
  * @param[in,out] resp nfs4_op results
  */
