@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: LGPL-3.0-or-later
 /*
  * vim:noexpandtab:shiftwidth=8:tabstop=8:
  *
@@ -46,7 +47,7 @@ static const char *close_tag = "CLOSE";
 /**
  * @brief Clean up the current layouts
  *
- * @note state_lock MUST be held for write
+ * @note st_lock MUST be held
  *
  * @param[in] data	Current compound data
  */
@@ -130,9 +131,9 @@ void cleanup_layouts(compound_data_t *data)
 
 /**
  *
- * Brief Implemtation of NFS4_OP_CLOSE
+ * Brief Implementation of NFS4_OP_CLOSE
  *
- * This function implemtats the NFS4_OP_CLOSE
+ * This function implements the NFS4_OP_CLOSE
  * operation.
  *
  * @param[in]     op   Arguments for nfs4_op
@@ -142,8 +143,8 @@ void cleanup_layouts(compound_data_t *data)
  * @return per RFC5661, p. 362
  */
 
-int nfs4_op_close(struct nfs_argop4 *op, compound_data_t *data,
-		  struct nfs_resop4 *resp)
+enum nfs_req_result nfs4_op_close(struct nfs_argop4 *op, compound_data_t *data,
+				  struct nfs_resop4 *resp)
 {
 	/* Short alias for arguments */
 	CLOSE4args * const arg_CLOSE4 = &op->nfs_argop4_u.opclose;
@@ -159,6 +160,8 @@ int nfs4_op_close(struct nfs_argop4 *op, compound_data_t *data,
 	struct glist_head *glist = NULL;
 	/* Secondary safe iterator to continue traversal on delete */
 	struct glist_head *glistn = NULL;
+	struct fsal_obj_handle *state_obj;
+	bool ok;
 
 	LogDebug(COMPONENT_STATE,
 		 "Entering NFS v4 CLOSE handler ----------------------------");
@@ -171,7 +174,7 @@ int nfs4_op_close(struct nfs_argop4 *op, compound_data_t *data,
 	res_CLOSE4->status = nfs4_sanity_check_FH(data, REGULAR_FILE, false);
 
 	if (res_CLOSE4->status != NFS4_OK)
-		return res_CLOSE4->status;
+		return NFS_REQ_ERROR;
 
 	/* Check stateid correctness and get pointer to state */
 	nfs_status = nfs4_Check_Stateid(&arg_CLOSE4->open_stateid,
@@ -188,11 +191,24 @@ int nfs4_op_close(struct nfs_argop4 *op, compound_data_t *data,
 	if (nfs_status != NFS4_OK && nfs_status != NFS4ERR_REPLAY) {
 		res_CLOSE4->status = nfs_status;
 		LogDebug(COMPONENT_STATE, "CLOSE failed nfs4_Check_Stateid");
-		return res_CLOSE4->status;
+		return NFS_REQ_ERROR;
 	}
 
-	if (state_found == NULL) {
+	/* We hold the state, but not its object handle. Its object
+	 * handle could be freed as soon as the state gets deleted from
+	 * the hashtable.
+	 *
+	 * If there are multiple threads trying to delete the state at
+	 * the same time, the object handle could be NULL here.
+	 *
+	 * Get a ref on the object handle and the open owner.
+	 */
+	ok = get_state_obj_export_owner_refs(state_found, &state_obj, NULL,
+					     &open_owner);
+	if (!ok) {
 		/* Assume this is a replayed close */
+		if (state_found)
+			dec_state_t_ref(state_found);
 		res_CLOSE4->status = NFS4_OK;
 		memcpy(res_CLOSE4->CLOSE4res_u.open_stateid.other,
 		       arg_CLOSE4->open_stateid.other,
@@ -207,16 +223,9 @@ int nfs4_op_close(struct nfs_argop4 *op, compound_data_t *data,
 		LogDebug(COMPONENT_STATE,
 			 "CLOSE failed nfs4_Check_Stateid must have already been closed. But treating it as replayed close and returning NFS4_OK");
 
-		return res_CLOSE4->status;
+		return NFS_REQ_OK;
 	}
 
-	open_owner = get_state_owner_ref(state_found);
-
-	if (open_owner == NULL) {
-		/* Unexpected, but something just went stale. */
-		res_CLOSE4->status = NFS4ERR_STALE;
-		goto out3;
-	}
 
 	PTHREAD_MUTEX_lock(&open_owner->so_mutex);
 
@@ -225,7 +234,7 @@ int nfs4_op_close(struct nfs_argop4 *op, compound_data_t *data,
 		if (!Check_nfs4_seqid(open_owner,
 				      arg_CLOSE4->seqid,
 				      op,
-				      data->current_obj,
+				      state_obj,
 				      resp,
 				      close_tag)) {
 			/* Response is all setup for us and LogDebug
@@ -238,7 +247,7 @@ int nfs4_op_close(struct nfs_argop4 *op, compound_data_t *data,
 
 	PTHREAD_MUTEX_unlock(&open_owner->so_mutex);
 
-	PTHREAD_RWLOCK_wrlock(&data->current_obj->state_hdl->state_lock);
+	STATELOCK_lock(state_obj);
 
 	/* Check is held locks remain */
 	glist_for_each(glist, &state_found->state_data.share.share_lockstates) {
@@ -250,12 +259,11 @@ int nfs4_op_close(struct nfs_argop4 *op, compound_data_t *data,
 			/* Is this actually what we want to do, rather
 			 * than freeing all locks on close?
 			 * Especially since the next thing we do is
-			 * go through ane release any lock states.
+			 * go through and release any lock states.
 			 */
 			res_CLOSE4->status = NFS4ERR_LOCKS_HELD;
 
-			PTHREAD_RWLOCK_unlock(
-				&data->current_obj->state_hdl->state_lock);
+			STATELOCK_unlock(state_obj);
 			LogDebug(COMPONENT_STATE,
 				 "NFS4 Close with existing locks");
 			goto out;
@@ -303,16 +311,10 @@ int nfs4_op_close(struct nfs_argop4 *op, compound_data_t *data,
 	if (data->minorversion > 0)
 		cleanup_layouts(data);
 
-	/* Fill in the clientid for NFSv4.0 */
-	if (data->minorversion == 0) {
-		op_ctx->clientid =
-		    &open_owner->so_owner.so_nfs4_owner.so_clientid;
-	}
-
 	if (data->minorversion == 0)
 		op_ctx->clientid = NULL;
 
-	PTHREAD_RWLOCK_unlock(&data->current_obj->state_hdl->state_lock);
+	STATELOCK_unlock(state_obj);
 	res_CLOSE4->status = NFS4_OK;
 
 	if (isFullDebug(COMPONENT_STATE) && isFullDebug(COMPONENT_MEMLEAKS)) {
@@ -325,18 +327,15 @@ int nfs4_op_close(struct nfs_argop4 *op, compound_data_t *data,
 	/* Save the response in the open owner */
 	if (data->minorversion == 0) {
 		Copy_nfs4_state_req(open_owner, arg_CLOSE4->seqid, op,
-				    data->current_obj, resp, close_tag);
+				    state_obj, resp, close_tag);
 	}
 
  out2:
-
 	dec_state_owner_ref(open_owner);
-
- out3:
-
+	state_obj->obj_ops->put_ref(state_obj);
 	dec_state_t_ref(state_found);
 
-	return res_CLOSE4->status;
+	return nfsstat4_to_nfs_req_result(res_CLOSE4->status);
 }				/* nfs4_op_close */
 
 /**
