@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: LGPL-3.0-or-later
 /*
  * vim:noexpandtab:shiftwidth=8:tabstop=8:
  *
@@ -108,7 +109,7 @@ enum nfs_req_result nfs4_op_destroy_clientid(struct nfs_argop4 *op,
 	}
 
 	/* ref +1 */
-	if (client_record == NULL) {
+	if (found == NULL) {
 		/* Fine.  We're done. */
 		res_DESTROY_CLIENTID4->dcr_status = NFS4ERR_STALE_CLIENTID;
 		goto out;
