@@ -1191,9 +1191,8 @@ static void open4_ex(OPEN4args *arg,
 	if (res_OPEN4->status != NFS4_OK) {
 		/* Cleanup state on error */
 		if (*new_state)
-			(*file_state)
-				->state_exp->exp_ops.free_state(
-					(*file_state)->state_exp, *file_state);
+			op_ctx->fsal_export->exp_ops.free_state(
+					op_ctx->fsal_export, *file_state);
 		else if (*file_state != NULL)
 			dec_state_t_ref(*file_state);
 		*file_state = NULL;
