@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: LGPL-3.0-or-later
 /*
  * Vim:noexpandtab:shiftwidth=8:tabstop=8:
  *
@@ -40,6 +41,7 @@
 #include "nfs_proto_tools.h"
 #include "nfs_file_handle.h"
 #include "export_mgr.h"
+#include "pnfs_utils.h"
 
 /**
  *
@@ -111,15 +113,19 @@ enum nfs_req_result nfs4_op_restorefh(struct nfs_argop4 *op,
 
 	data->currentFH.nfs_fh4_len = data->savedFH.nfs_fh4_len;
 
-	if (op_ctx->ctx_export != NULL)
-		put_gsh_export(op_ctx->ctx_export);
+	/* Restore the export information and release any old export reference
+	 * and any old fsal_pnfs_ds reference.
+	 */
+	set_op_context_export(data->saved_export);
+	op_ctx->export_perms = data->saved_export_perms;
 
-	/* Restore the export information */
-	op_ctx->ctx_export = data->saved_export;
-	if (op_ctx->ctx_export != NULL)
-		op_ctx->fsal_export = op_ctx->ctx_export->fsal_export;
-
-	*op_ctx->export_perms = data->saved_export_perms;
+	/* If saved_pnfs_ds is present, get an additional reference and restore
+	 * it.
+	 */
+	if (data->saved_pnfs_ds != NULL) {
+		op_ctx->ctx_pnfs_ds = data->saved_pnfs_ds;
+		pnfs_ds_get_ref(op_ctx->ctx_pnfs_ds);
+	}
 
 	/* No need to call nfs4_SetCompoundExport or nfs4_MakeCred
 	 * because we are restoring saved information, and the
@@ -137,7 +143,6 @@ enum nfs_req_result nfs4_op_restorefh(struct nfs_argop4 *op,
 	if (data->current_ds != NULL) {
 		data->current_ds = data->saved_ds;
 		data->current_filetype = data->saved_filetype;
-		ds_handle_get_ref(data->current_ds);
 	}
 
 	LogHandleNFS4("RESTORE FH: Current FH ", &data->currentFH);
