@@ -337,13 +337,13 @@ bool pseudo_mount_export(struct gsh_export *export)
 	int rc;
 	bool result = false;
 
+	LOG_EXPORT(NIV_DEBUG, "Attempt pseudofs mount for", export, false);
+
 	/* skip exports that aren't for NFS v4
 	 * Also, nothing to actually do for Pseudo Root
 	 * (defer checking pseudopath for Pseudo Root until we have refstr.
 	 */
-	if ((export->export_perms.options & EXPORT_OPTION_NFSV4) == 0
-	    || export->pseudopath == NULL
-	    || export->export_id == 0)
+	if (!export_can_be_mounted(export) || export->export_id == 0)
 		return true;
 
 	/* Initialize state and it's op_context.
@@ -495,6 +495,9 @@ bool pseudo_mount_export(struct gsh_export *export)
 
 	PTHREAD_RWLOCK_unlock(&export->lock);
 
+	/* Set this outside the export->lock, protected by EXPORT_ADMIN_LOCK */
+	export->is_mounted = true;
+
 	LogDebug(COMPONENT_EXPORT,
 		 "BUILDING PSEUDOFS: Export_Id %d Path %s Pseudo Path %s junction %p",
 		 export->export_id,
@@ -574,7 +577,7 @@ void pseudo_unmount_export(struct gsh_export *export)
 		assert(junction_inode == NULL && mounted_on_export == NULL);
 
 		LogDebug(COMPONENT_EXPORT,
-			 "Unmount of export %d unnecessary it should be pseudo root",
+			 "Unmount of export %d unnecessary",
 			 export->export_id);
 
 		PTHREAD_RWLOCK_unlock(&export->lock);
@@ -627,6 +630,10 @@ void pseudo_unmount_export(struct gsh_export *export)
 	/* Release the export lock */
 	PTHREAD_RWLOCK_unlock(&export->lock);
 
+	/* Clear this outside the export->lock, protected by EXPORT_ADMIN_LOCK
+	 */
+	export->is_mounted = false;
+
 	if (is_export_pseudo(mounted_on_export) && junction_inode != NULL) {
 		char *pseudo_path = gsh_strdup(ref_pseudopath->gr_val);
 
