@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: LGPL-3.0-or-later
 /*
  * Copyright IBM Corporation, 2010
  *  Contributor: Aneesh Kumar K.v  <aneesh.kumar@linux.vnet.ibm.com>
@@ -73,7 +74,7 @@ int nlm4_Cancel(nfs_arg_t *args, struct svc_req *req, nfs_res_t *res)
 
 	copy_netobj(&res->res_nlm4test.cookie, &arg->cookie);
 
-	if (nfs_in_grace()) {
+	if (!nfs_get_grace_status(false)) {
 		res->res_nlm4.stat.stat = NLM4_DENIED_GRACE_PERIOD;
 		LogDebug(COMPONENT_NLM,
 			 "REQUEST RESULT: NLM4_CANCEL %s",
@@ -101,14 +102,14 @@ int nlm4_Cancel(nfs_arg_t *args, struct svc_req *req, nfs_res_t *res)
 		LogDebug(COMPONENT_NLM,
 			 "REQUEST RESULT: nlm4_Unlock %s",
 			 lock_result_str(res->res_nlm4.stat.stat));
-		return NFS_REQ_OK;
+		goto out;
 	}
 
 	state_status = state_cancel(obj, nlm_owner, &lock);
 	if (state_status != STATE_SUCCESS) {
 		/* Cancel could fail in the FSAL and make a bit of a mess,
 		 * especially if we are in out of memory situation. Such an
-		 * error is logged by Cache Inode.
+		 * error is already logged.
 		 */
 		res->res_nlm4test.test_stat.stat =
 		    nlm_convert_state_error(state_status);
@@ -120,11 +121,13 @@ int nlm4_Cancel(nfs_arg_t *args, struct svc_req *req, nfs_res_t *res)
 	dec_nsm_client_ref(nsm_client);
 	dec_nlm_client_ref(nlm_client);
 	dec_state_owner_ref(nlm_owner);
-	obj->obj_ops.put_ref(obj);
+	obj->obj_ops->put_ref(obj);
 
 	LogDebug(COMPONENT_NLM,
 		 "REQUEST RESULT: NLM4_CANCEL %s",
 		 lock_result_str(res->res_nlm4.stat.stat));
+out:
+	nfs_put_grace_status();
 	return NFS_REQ_OK;
 }				/* nlm4_Cancel */
 
@@ -170,9 +173,7 @@ int nlm4_Cancel_Message(nfs_arg_t *args, struct svc_req *req, nfs_res_t *res)
 	LogDebug(COMPONENT_NLM,
 		 "REQUEST PROCESSING: Calling nlm_Cancel_Message");
 
-	nsm_client = get_nsm_client(CARE_NO_MONITOR,
-				    req->rq_xprt,
-				    arg->alock.caller_name);
+	nsm_client = get_nsm_client(CARE_NO_MONITOR, arg->alock.caller_name);
 
 	if (nsm_client != NULL)
 		nlm_client = get_nlm_client(CARE_NO_MONITOR,
