@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: LGPL-3.0-or-later
 /*
  * Copyright IBM Corporation, 2010
  *  Contributor: M. Mohan Kumar <mohan@in.ibm.com>
@@ -78,9 +79,8 @@ int nlm4_Granted_Res(nfs_arg_t *args, struct svc_req *req, nfs_res_t *res)
 	 * We take an export reference even if the export is stale because
 	 * we want to properly clean up the cookie_entry.
 	 */
-	op_ctx->ctx_export = cookie_entry->sce_lock_entry->sle_export;
-	op_ctx->fsal_export = op_ctx->ctx_export->fsal_export;
-	get_gsh_export_ref(op_ctx->ctx_export);
+	get_gsh_export_ref(cookie_entry->sce_lock_entry->sle_export);
+	set_op_context_export(cookie_entry->sce_lock_entry->sle_export);
 
 	/* If the client returned an error or the export has gone stale,
 	 * release the grant to properly clean up cookie_entry.
@@ -95,7 +95,7 @@ int nlm4_Granted_Res(nfs_arg_t *args, struct svc_req *req, nfs_res_t *res)
 		state_status = state_release_grant(cookie_entry);
 		if (state_status != STATE_SUCCESS) {
 			LogDebug(COMPONENT_NLM,
-				 "cache_inode_release_grant failed");
+				 "state_release_grant failed");
 		}
 	} else {
 		state_complete_grant(cookie_entry);
