@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: LGPL-3.0-or-later
 /*
  * Copyright IBM Corporation, 2010
  *  Contributor: Aneesh Kumar K.v  <aneesh.kumar@linux.vnet.ibm.com>
@@ -165,7 +166,7 @@ static void nlm4_send_grant_msg(state_async_queue_t *arg)
 	state_cookie_entry_t *cookie_entry;
 	state_nlm_async_data_t *nlm_arg =
 	    &arg->state_async_data.state_nlm_async_data;
-	struct root_op_context root_op_context;
+	struct req_op_context op_context;
 	struct gsh_export *export;
 	nlm4_testargs *nlm_async_grant;
 
@@ -230,14 +231,12 @@ static void nlm4_send_grant_msg(state_async_queue_t *arg)
 	export = cookie_entry->sce_lock_entry->sle_export;
 	get_gsh_export_ref(export);
 
-	init_root_op_context(&root_op_context,
-			     export, export->fsal_export,
-			     NFS_V3, 0, NFS_REQUEST);
+	init_op_context(&op_context, export, export->fsal_export, NULL,
+			NFS_V3, 0, NFS_REQUEST);
 
 	state_status = state_release_grant(cookie_entry);
 
-	release_root_op_context();
-	put_gsh_export(export);
+	release_op_context();
 
 	if (state_status != STATE_SUCCESS) {
 		/* Huh? */
@@ -288,7 +287,13 @@ int nlm_process_parameters(struct svc_req *req, bool exclusive,
 		return NLM4_STALE_FH;
 	}
 
-	*ppnsm_client = get_nsm_client(care, ptr_svc, alock->caller_name);
+	if ((*ppobj)->type != REGULAR_FILE) {
+		LogWarn(COMPONENT_NLM,
+			"NLM operation on non-REGULAR_FILE");
+		return NLM4_FAILED;
+	}
+
+	*ppnsm_client = get_nsm_client(care, alock->caller_name);
 
 	if (*ppnsm_client == NULL) {
 		/* If NSM Client is not found, and we don't care (such as
@@ -437,7 +442,13 @@ int nlm_process_share_parms(struct svc_req *req, nlm4_share *share,
 		return NLM4_STALE_FH;
 	}
 
-	*ppnsm_client = get_nsm_client(care, ptr_svc, share->caller_name);
+	if ((*ppobj)->type != REGULAR_FILE) {
+		LogWarn(COMPONENT_NLM,
+			"NLM operation on non-REGULAR_FILE");
+		return NLM4_FAILED;
+	}
+
+	*ppnsm_client = get_nsm_client(care, share->caller_name);
 
 	if (*ppnsm_client == NULL) {
 		/* If NSM Client is not found, and we don't care (for unshare),
