@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: LGPL-3.0-or-later
 /*
  * Copyright CEA/DAM/DIF  (2008)
  * contributeur : Philippe DENIEL   philippe.deniel@cea.fr
@@ -81,6 +82,9 @@ uint64_t clientid_verifier;
  */
 pool_t *client_id_pool;
 
+
+static uint64_t num_confirmed_client_ids;
+
 /**
  * @brief Return the NFSv4 status for the client id error code
  *
@@ -189,13 +193,10 @@ int display_client_id_rec(struct display_buffer *dspbuf,
 	if (b_left <= 0)
 		return b_left;
 
-	if (clientid->cid_client_record != NULL) {
-		b_left = display_client_record(dspbuf,
-					       clientid->cid_client_record);
+	b_left = display_client_record(dspbuf, clientid->cid_client_record);
 
-		if (b_left <= 0)
-			return b_left;
-	}
+	if (b_left <= 0)
+		return b_left;
 
 	if (clientid->cid_lease_reservations > 0)
 		delta = 0;
@@ -233,9 +234,6 @@ int display_client_id_rec(struct display_buffer *dspbuf,
 int display_clientid_name(struct display_buffer *dspbuf,
 			  nfs_client_id_t *clientid)
 {
-	if (clientid->cid_client_record == NULL)
-		return display_start(dspbuf);
-
 	return display_opaque_value(
 		dspbuf,
 		clientid->cid_client_record->cr_client_val,
@@ -317,8 +315,8 @@ void free_client_id(nfs_client_id_t *clientid)
 {
 	assert(atomic_fetch_int32_t(&clientid->cid_refcount) == 0);
 
-	if (clientid->cid_client_record != NULL)
-		dec_client_record_ref(clientid->cid_client_record);
+	/* This is where we finally let go of the client record. */
+	dec_client_record_ref(clientid->cid_client_record);
 
 #ifdef _HAVE_GSSAPI
 	if (clientid->cid_credential.flavor == RPCSEC_GSS) {
@@ -477,39 +475,25 @@ int compare_client_id(struct gsh_buffdesc *buff1, struct gsh_buffdesc *buff2)
 /**
  * @brief Displays the client_id stored from the hash table
  *
- * @param[in]  buff Buffer to display
- * @param[out] str  Output string
- *
- * @return Number of character written.
- *
- */
-int display_client_id_key(struct gsh_buffdesc *buff, char *str)
+ * @param[in]  dspbuf display buffer to display into
+ * @param[in]  buff   Buffer to display
+  */
+int display_client_id_key(struct display_buffer *dspbuf,
+			  struct gsh_buffdesc *buff)
 {
-	struct display_buffer dspbuf = {DISPLAY_CLIENTID_SIZE, str, str};
-	int rc;
-
-	rc = display_clientid(&dspbuf, *((clientid4 *) (buff->addr)));
-	assert(rc >= 0);
-
-	return display_buffer_len(&dspbuf);
+	return display_clientid(dspbuf, *((clientid4 *) (buff->addr)));
 }
 
 /**
  * @brief Displays the client record from a hash table
  *
- * @param[in]  buff Buffer to display
- * @param[out] str  Output string
- *
- * @return Number of character written.
- *
+ * @param[in]  dspbuf display buffer to display into
+ * @param[in]  buff   Buffer to display
  */
-int display_client_id_val(struct gsh_buffdesc *buff, char *str)
+int display_client_id_val(struct display_buffer *dspbuf,
+			  struct gsh_buffdesc *buff)
 {
-	struct display_buffer dspbuf = {HASHTABLE_DISPLAY_STRLEN, str, str};
-
-	display_client_id_rec(&dspbuf, buff->addr);
-
-	return display_buffer_len(&dspbuf);
+	return display_client_id_rec(dspbuf, buff->addr);
 }
 
 /**
@@ -597,10 +581,10 @@ nfs_client_id_t *create_client_id(clientid4 clientid,
  *
  * @param[in] clientid the client id record
  *
- * @retval CLIENT_ID_SUCCESS if successfull.
- * @retval CLIENT_ID_INSERT_MALLOC_ERROR if an error occured during
+ * @retval CLIENT_ID_SUCCESS if successful.
+ * @retval CLIENT_ID_INSERT_MALLOC_ERROR if an error occurred during
  *         the insertion process
- * @retval CLIENT_ID_NETDB_ERROR if an error occured during the netdb
+ * @retval CLIENT_ID_NETDB_ERROR if an error occurred during the netdb
  *         query (via gethostbyaddr).
  */
 
@@ -610,6 +594,18 @@ clientid_status_t nfs_client_id_insert(nfs_client_id_t *clientid)
 	struct gsh_buffdesc buffdata;
 	int rc;
 
+	if (nfs_param.nfsv4_param.max_client_ids &&
+	    atomic_fetch_uint64_t(&num_confirmed_client_ids) >
+		 nfs_param.nfsv4_param.max_client_ids) {
+		LogDebug(COMPONENT_CLIENTID,
+			 "Max client_id limit reached - clientid %"
+			 PRIx64, clientid->cid_clientid);
+
+		/* Free the clientid record and return */
+		free_client_id(clientid);
+
+		return CLIENT_ID_INSERT_MALLOC_ERROR;
+	}
 	/* Create key from cid_clientid */
 	buffkey.addr = &clientid->cid_clientid;
 	buffkey.len = sizeof(clientid->cid_clientid);
@@ -677,14 +673,14 @@ int remove_confirmed_client_id(nfs_client_id_t *clientid)
 		return rc;
 	}
 
-	if (clientid->cid_client_record != NULL)
-		clientid->cid_client_record->cr_confirmed_rec = NULL;
+	clientid->cid_client_record->cr_confirmed_rec = NULL;
 
 	/* Set this up so this client id record will be freed. */
 	clientid->cid_confirmed = EXPIRED_CLIENT_ID;
 
 	/* Release hash table reference to the unconfirmed record */
 	(void)dec_client_id_ref(clientid);
+	atomic_dec_uint64_t(&num_confirmed_client_ids);
 
 	return rc;
 }
@@ -721,8 +717,7 @@ int remove_unconfirmed_client_id(nfs_client_id_t *clientid)
 	/* XXX prevents calling remove_confirmed before removed_confirmed,
 	 * if we failed to maintain the invariant that the cases are
 	 * disjoint */
-	if (clientid->cid_client_record != NULL)
-		clientid->cid_client_record->cr_unconfirmed_rec = NULL;
+	clientid->cid_client_record->cr_unconfirmed_rec = NULL;
 
 	/* Set this up so this client id record will be freed. */
 	clientid->cid_confirmed = EXPIRED_CLIENT_ID;
@@ -737,14 +732,14 @@ int remove_unconfirmed_client_id(nfs_client_id_t *clientid)
  * @brief Confirm a client id record.
  *
  * @param[in] clientid  The client id record
- * @param[in] component Component ID for logging
+ * @param[in] component The component ID for logging
  *
- * @retval CLIENT_ID_SUCCESS if successfull.
+ * @retval CLIENT_ID_SUCCESS if successful.
  * @retval CLIENT_ID_INVALID_ARGUMENT if unable to find record in
  *         unconfirmed table
  * @retval CLIENT_ID_INSERT_MALLOC_ERROR if unable to insert record
  *         into confirmed table
- * @retval CLIENT_ID_NETDB_ERROR if an error occured during the netdb
+ * @retval CLIENT_ID_NETDB_ERROR if an error occurred during the netdb
  *         query (via gethostbyaddr).
  */
 clientid_status_t nfs_client_id_confirm(nfs_client_id_t *clientid,
@@ -808,6 +803,7 @@ clientid_status_t nfs_client_id_confirm(nfs_client_id_t *clientid,
 
 		return CLIENT_ID_INSERT_MALLOC_ERROR;
 	}
+	atomic_inc_uint64_t(&num_confirmed_client_ids);
 
 	/* Add the clientid as the confirmed entry for the client
 	   record */
@@ -878,15 +874,13 @@ bool nfs_client_id_expire(nfs_client_id_t *clientid, bool make_stale)
 	struct gsh_buffdesc old_key;
 	struct gsh_buffdesc old_value;
 	hash_table_t *ht_expire;
-	nfs_client_record_t *record;
 	char str[LOG_BUFF_LEN] = "\0";
 	struct display_buffer dspbuf = {sizeof(str), str, str};
 	bool str_valid = false;
-	struct root_op_context root_op_context;
+	struct req_op_context op_context;
 
-	/* Initialize req_ctx */
-	init_root_op_context(&root_op_context, NULL, NULL,
-			     0, 0, UNKNOWN_REQUEST);
+	/* Initialize op_context */
+	init_op_context_simple(&op_context, NULL, NULL);
 
 	PTHREAD_MUTEX_lock(&clientid->cid_mutex);
 	if (clientid->cid_confirmed == EXPIRED_CLIENT_ID) {
@@ -897,7 +891,7 @@ bool nfs_client_id_expire(nfs_client_id_t *clientid, bool make_stale)
 		}
 
 		PTHREAD_MUTEX_unlock(&clientid->cid_mutex);
-		release_root_op_context();
+		release_op_context();
 		return false;
 	}
 
@@ -912,21 +906,12 @@ bool nfs_client_id_expire(nfs_client_id_t *clientid, bool make_stale)
 	else
 		ht_expire = ht_unconfirmed_client_id;
 
-	/* Need to clean up the client record. */
-	record = clientid->cid_client_record;
-	clientid->cid_client_record = NULL;
+	/* Detach the clientid record from the client record */
+	if (clientid->cid_client_record->cr_confirmed_rec == clientid)
+		clientid->cid_client_record->cr_confirmed_rec = NULL;
 
-	if (record != NULL) {
-		/* Detach the clientid record from the client record */
-		if (record->cr_confirmed_rec == clientid)
-			record->cr_confirmed_rec = NULL;
-
-		if (record->cr_unconfirmed_rec == clientid)
-			record->cr_unconfirmed_rec = NULL;
-
-		/* the linkage was removed, update refcount */
-		dec_client_record_ref(record);
-	}
+	if (clientid->cid_client_record->cr_unconfirmed_rec == clientid)
+		clientid->cid_client_record->cr_unconfirmed_rec = NULL;
 
 	if (make_stale) {
 		/* Keep clientid hashed, but mark it as stale */
@@ -956,7 +941,8 @@ bool nfs_client_id_expire(nfs_client_id_t *clientid, bool make_stale)
 				"Could not remove expired clientid %" PRIx64
 				" error=%s", clientid->cid_clientid,
 				hash_table_err_to_str(rc));
-		}
+		} else if (ht_expire == ht_confirmed_client_id)
+			atomic_dec_uint64_t(&num_confirmed_client_ids);
 	}
 
 	/* Traverse the client's lock owners, and release all
@@ -984,9 +970,8 @@ bool nfs_client_id_expire(nfs_client_id_t *clientid, bool make_stale)
 		/* Move owner to end of list in case it doesn't get
 		 * freed when we decrement the refcount.
 		 */
-		glist_del(&owner->so_owner.so_nfs4_owner.so_perclient);
-		glist_add_tail(&clientid->cid_lockowners,
-			       &owner->so_owner.so_nfs4_owner.so_perclient);
+		glist_move_tail(&clientid->cid_lockowners,
+				&owner->so_owner.so_nfs4_owner.so_perclient);
 
 		/* Hold a reference to the owner while we drop the cid_mutex. */
 		held = hold_state_owner(owner);
@@ -1050,9 +1035,8 @@ bool nfs_client_id_expire(nfs_client_id_t *clientid, bool make_stale)
 		/* Move owner to end of list in case it doesn't get
 		 * freed when we decrement the refcount.
 		 */
-		glist_del(&owner->so_owner.so_nfs4_owner.so_perclient);
-		glist_add_tail(&clientid->cid_openowners,
-			       &owner->so_owner.so_nfs4_owner.so_perclient);
+		glist_move_tail(&clientid->cid_openowners,
+				&owner->so_owner.so_nfs4_owner.so_perclient);
 
 		/* Hold a reference to the owner while we drop the cid_mutex. */
 		held = hold_state_owner(owner);
@@ -1152,7 +1136,7 @@ bool nfs_client_id_expire(nfs_client_id_t *clientid, bool make_stale)
 			     str);
 	}
 
-	release_root_op_context();
+	release_op_context();
 	return true;
 }
 
@@ -1208,7 +1192,7 @@ clientid_status_t nfs_client_id_get(hash_table_t *ht, clientid4 clientid,
 		pclientid = buffval.addr;
 
 		if (pclientid->cid_confirmed == STALE_CLIENT_ID) {
-			/* Stale client becuse of ip detach and attach to
+			/* Stale client because of ip detach and attach to
 			 * same node */
 			dec_client_id_ref(pclientid);
 			status = CLIENT_ID_STALE;
@@ -1266,8 +1250,8 @@ static hash_parameter_t cid_confirmed_hash_param = {
 	.hash_func_rbt = client_id_rbt_hash_func,
 	.hash_func_both = NULL,
 	.compare_key = compare_client_id,
-	.key_to_str = display_client_id_key,
-	.val_to_str = display_client_id_val,
+	.display_key = display_client_id_key,
+	.display_val = display_client_id_val,
 	.ht_name = "Confirmed Client ID",
 	.flags = HT_FLAG_CACHE,
 	.ht_log_component = COMPONENT_CLIENTID,
@@ -1279,67 +1263,12 @@ static hash_parameter_t cid_unconfirmed_hash_param = {
 	.hash_func_rbt = client_id_rbt_hash_func,
 	.hash_func_both = NULL,
 	.compare_key = compare_client_id,
-	.key_to_str = display_client_id_key,
-	.val_to_str = display_client_id_val,
+	.display_key = display_client_id_key,
+	.display_val = display_client_id_val,
 	.ht_name = "Unconfirmed Client ID",
 	.flags = HT_FLAG_CACHE,
 	.ht_log_component = COMPONENT_CLIENTID,
 };
-
-static hash_parameter_t cr_hash_param = {
-	.index_size = PRIME_STATE,
-	.hash_func_key = client_record_value_hash_func,
-	.hash_func_rbt = client_record_rbt_hash_func,
-	.hash_func_both = NULL,
-	.compare_key = compare_client_record,
-	.key_to_str = display_client_record_key,
-	.val_to_str = display_client_record_val,
-	.ht_name = "Client Record",
-	.flags = HT_FLAG_CACHE,
-	.ht_log_component = COMPONENT_CLIENTID,
-};
-
-/**
- * @brief Init the hashtable for Client Id cache.
- *
- * Perform all the required initialization for hashtable Client Id cache
- *
- * @return 0 if successful, -1 otherwise
- */
-int nfs_Init_client_id(void)
-{
-	ht_confirmed_client_id =
-		hashtable_init(&cid_confirmed_hash_param);
-
-	if (ht_confirmed_client_id == NULL) {
-		LogCrit(COMPONENT_INIT,
-			"NFS CLIENT_ID: Cannot init Client Id cache");
-		return -1;
-	}
-
-	ht_unconfirmed_client_id =
-		hashtable_init(&cid_unconfirmed_hash_param);
-
-	if (ht_unconfirmed_client_id == NULL) {
-		LogCrit(COMPONENT_INIT,
-			"NFS CLIENT_ID: Cannot init Client Id cache");
-		return -1;
-	}
-
-	ht_client_record = hashtable_init(&cr_hash_param);
-
-	if (ht_client_record == NULL) {
-		LogCrit(COMPONENT_INIT,
-			"NFS CLIENT_ID: Cannot init Client Record cache");
-		return -1;
-	}
-
-	client_id_pool =
-	    pool_basic_init("NFS4 Client ID Pool", sizeof(nfs_client_id_t));
-
-	return CLIENT_ID_SUCCESS;
-}
-
 int display_clientid(struct display_buffer *dspbuf, clientid4 clientid)
 {
 	int b_left = display_buffer_remain(dspbuf);
@@ -1421,7 +1350,7 @@ int display_client_record(struct display_buffer *dspbuf,
 /**
  * @brief Increment the refcount on a client owner record
  *
- * @param[in] record Record on which to take a reference
+ * @param[in] record The record on which to take a reference
  */
 
 int32_t inc_client_record_ref(nfs_client_record_t *record)
@@ -1455,7 +1384,7 @@ void free_client_record(nfs_client_record_t *record)
 /**
  * @brief Decrement the refcount on a client owner record
  *
- * @param[in] record Record on which to release a reference
+ * @param[in] record The record on which to release a reference
  */
 
 int32_t dec_client_record_ref(nfs_client_record_t *record)
@@ -1479,7 +1408,7 @@ int32_t dec_client_record_ref(nfs_client_record_t *record)
 
 	if (refcount > 0) {
 		LogFullDebug(COMPONENT_CLIENTID,
-			     "Decrement refcount refcount now=%" PRId32 " {%s}",
+			     "Decrement refcount now=%" PRId32 " {%s}",
 			     refcount, str);
 
 		return refcount;
@@ -1626,35 +1555,25 @@ int compare_client_record(struct gsh_buffdesc *buff1,
 /**
  * @brief Displays the client_record stored in the buffer.
  *
- * @param[in]  buff Buffer to display
- * @param[out] str  output string
- *
- * @return The number of character written.
+ * @param[in]  dspbuf display buffer to display into
+ * @param[in]  buff   Buffer to display
  */
-int display_client_record_key(struct gsh_buffdesc *buff, char *str)
+int display_client_record_key(struct display_buffer *dspbuf,
+			      struct gsh_buffdesc *buff)
 {
-	struct display_buffer dspbuf = {HASHTABLE_DISPLAY_STRLEN, str, str};
-
-	display_client_record(&dspbuf, buff->addr);
-
-	return display_buffer_len(&dspbuf);
+	return display_client_record(dspbuf, buff->addr);
 }
 
 /**
  * @brief Displays the client_record stored in the buffer.
  *
- * @param[in]  buff Buffer to display
- * @param[out] str  output string
- *
- * @return The number of character written.
+ * @param[in]  dspbuf display buffer to display into
+ * @param[in]  buff   Buffer to display
  */
-int display_client_record_val(struct gsh_buffdesc *buff, char *str)
+int display_client_record_val(struct display_buffer *dspbuf,
+			      struct gsh_buffdesc *buff)
 {
-	struct display_buffer dspbuf = {HASHTABLE_DISPLAY_STRLEN, str, str};
-
-	display_client_record(&dspbuf, buff->addr);
-
-	return display_buffer_len(&dspbuf);
+	return display_client_record(dspbuf, buff->addr);
 }
 
 /**
@@ -1816,4 +1735,59 @@ nfs41_foreach_client_callback(bool(*cb) (nfs_client_id_t *cl, void *state),
 	}
 }
 
+static hash_parameter_t cr_hash_param = {
+	.index_size = PRIME_STATE,
+	.hash_func_key = client_record_value_hash_func,
+	.hash_func_rbt = client_record_rbt_hash_func,
+	.hash_func_both = NULL,
+	.compare_key = compare_client_record,
+	.display_key = display_client_record_key,
+	.display_val = display_client_record_val,
+	.ht_name = "Client Record",
+	.flags = HT_FLAG_CACHE,
+	.ht_log_component = COMPONENT_CLIENTID,
+};
+
+/**
+ * @brief Init the hashtable for Client Id cache.
+ *
+ * Perform all the required initialization for hashtable Client Id cache
+ *
+ * @return 0 if successful, -1 otherwise
+ */
+int nfs_Init_client_id(void)
+{
+	ht_confirmed_client_id =
+		hashtable_init(&cid_confirmed_hash_param);
+
+	if (ht_confirmed_client_id == NULL) {
+		LogCrit(COMPONENT_INIT,
+			"NFS CLIENT_ID: Cannot init Client Id cache");
+		return -1;
+	}
+
+	ht_unconfirmed_client_id =
+		hashtable_init(&cid_unconfirmed_hash_param);
+
+	if (ht_unconfirmed_client_id == NULL) {
+		LogCrit(COMPONENT_INIT,
+			"NFS CLIENT_ID: Cannot init Client Id cache");
+		return -1;
+	}
+
+	ht_client_record = hashtable_init(&cr_hash_param);
+
+	if (ht_client_record == NULL) {
+		LogCrit(COMPONENT_INIT,
+			"NFS CLIENT_ID: Cannot init Client Record cache");
+		return -1;
+	}
+
+	client_id_pool =
+	    pool_basic_init("NFS4 Client ID Pool", sizeof(nfs_client_id_t));
+
+	return CLIENT_ID_SUCCESS;
+}
+
+
 /** @} */
