@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: LGPL-3.0-or-later
 /*
  * vim:noexpandtab:shiftwidth=8:tabstop=8:
  *
@@ -48,17 +49,13 @@ hash_table_t *ht_nfs4_owner;
 /**
  * @brief Display an NFSv4 owner key
  *
- * @param[in]  buff Key to display
- * @param[out] str  Output buffer
- *
- * @return Length of output string.
+ * @param[in]  dspbuf display buffer to display into
+ * @param[in]  buff   Key to display
  */
-int display_nfs4_owner_key(struct gsh_buffdesc *buff, char *str)
+int display_nfs4_owner_key(struct display_buffer *dspbuf,
+			   struct gsh_buffdesc *buff)
 {
-	struct display_buffer dspbuf = {HASHTABLE_DISPLAY_STRLEN, str, str};
-
-	display_nfs4_owner(&dspbuf, buff->addr);
-	return display_buffer_len(&dspbuf);
+	return display_nfs4_owner(dspbuf, buff->addr);
 }
 
 /**
@@ -136,7 +133,7 @@ int display_nfs4_owner(struct display_buffer *dspbuf, state_owner_t *owner)
 
 	if (texpire != 0) {
 		b_left = display_printf(dspbuf,
-					" cached(expires in %d secs)",
+					" cached(expires in %ld secs)",
 					texpire - time(NULL));
 
 		if (b_left <= 0)
@@ -150,17 +147,13 @@ int display_nfs4_owner(struct display_buffer *dspbuf, state_owner_t *owner)
 /**
  * @brief Display owner from hash table
  *
- * @param[in]  buff Buffer
- * @param[out] str  Output buffer
- *
- * @return Length of the output string.
+ * @param[in]  dspbuf display buffer to display into
+ * @param[in]  buff   Buffer
  */
-int display_nfs4_owner_val(struct gsh_buffdesc *buff, char *str)
+int display_nfs4_owner_val(struct display_buffer *dspbuf,
+			   struct gsh_buffdesc *buff)
 {
-	struct display_buffer dspbuf = {HASHTABLE_DISPLAY_STRLEN, str, str};
-
-	display_nfs4_owner(&dspbuf, buff->addr);
-	return display_buffer_len(&dspbuf);
+	return display_nfs4_owner(dspbuf, buff->addr);
 }
 
 /**
@@ -319,7 +312,7 @@ uint64_t nfs4_owner_rbt_hash_func(hash_parameter_t *hparam,
 /**
  * @brief Free an NFS4 owner object
  *
- * @param[in] owner Owner to remove
+ * @param[in] owner The owner to remove
  */
 
 void free_nfs4_owner(state_owner_t *owner)
@@ -347,8 +340,8 @@ static hash_parameter_t nfs4_owner_param = {
 	.hash_func_key = nfs4_owner_value_hash_func,
 	.hash_func_rbt = nfs4_owner_rbt_hash_func,
 	.compare_key = compare_nfs4_owner_key,
-	.key_to_str = display_nfs4_owner_key,
-	.val_to_str = display_nfs4_owner_val,
+	.display_key = display_nfs4_owner_key,
+	.display_val = display_nfs4_owner_val,
 	.flags = HT_FLAG_CACHE,
 };
 
@@ -638,7 +631,7 @@ void Copy_nfs4_denied(LOCK4denied *denied_dst, LOCK4denied *denied_src)
  * operations for which it guarantees At-Most Once Semantics.
  *
  * @param[in,out] owner The owner to hold the operation
- * @param[in]     seqid Seqid of this operation
+ * @param[in]     seqid The seqid of this operation
  * @param[in]     args  Arguments of operation to copy
  * @param[in]     data  Compound data
  * @param[in]     resp  Response to copy
@@ -696,8 +689,8 @@ void Copy_nfs4_state_req(state_owner_t *owner, seqid4 seqid, nfs_argop4 *args,
  * In either case, on a false return, the caller should send the
  * resulting response back to the client.
  *
- * @param[in]  owner Owner to check
- * @param[in]  seqid Seqid to check
+ * @param[in]  owner The owner to check
+ * @param[in]  seqid The seqid to check
  * @param[in]  args  Arguments of operation
  * @param[in]  data  Compound data
  * @param[out] resp  Cached request, if replay
