@@ -1,3 +1,22 @@
+// SPDX-License-Identifier: LGPL-3.0-or-later
+/*
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 3 of the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ *
+ */
+
 #include "config.h"
 #include "log.h"
 #include "nfs_core.h"
@@ -720,6 +739,14 @@ out:
 	gsh_free(cval);
 }
 
+int rados_kv_get_nodeid(char **pnodeid)
+{
+	/* return the nodeid if we have one */
+	if (rados_kv_param.nodeid)
+		*pnodeid = gsh_strdup(rados_kv_param.nodeid);
+	return 0;
+}
+
 struct nfs4_recovery_backend rados_kv_backend = {
 	.recovery_init = rados_kv_init,
 	.recovery_shutdown = rados_kv_shutdown,
@@ -728,6 +755,7 @@ struct nfs4_recovery_backend rados_kv_backend = {
 	.add_clid = rados_kv_add_clid,
 	.rm_clid = rados_kv_rm_clid,
 	.add_revoke_fh = rados_kv_add_revoke_fh,
+	.get_nodeid = rados_kv_get_nodeid,
 };
 
 void rados_kv_backend_init(struct nfs4_recovery_backend **backend)
