@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: LGPL-3.0-or-later
 /*
  * vim:noexpandtab:shiftwidth=8:tabstop=8:
  *
@@ -59,7 +60,7 @@ struct fridgethr *state_poll_fridge;
  * We use this wrapper so we can void rewriting stuff.  We can change
  * this later.
  *
- * @param[in] ctx Thread context, containing arguments.
+ * @param[in] ctx Thread fridge context, containing arguments.
  */
 
 static void state_blocked_lock_caller(struct fridgethr_context *ctx)
@@ -67,23 +68,21 @@ static void state_blocked_lock_caller(struct fridgethr_context *ctx)
 	state_block_data_t *block = ctx->arg;
 	struct gsh_export *export;
 	bool set_op_ctx = false;
-	struct root_op_context root_ctx;
+	struct req_op_context op_context;
 
 	export = block->sbd_lock_entry->sle_export;
 	if (export_ready(export)) {
 		get_gsh_export_ref(export);
 		/* Initialize a root context, need to get a valid export. */
-		init_root_op_context(&root_ctx, export,
-				 export->fsal_export, 0, 0, UNKNOWN_REQUEST);
+		init_op_context(&op_context, export, export->fsal_export,
+				NULL, 0, 0, UNKNOWN_REQUEST);
 		set_op_ctx = true;
 	}
 
 	process_blocked_lock_upcall(block);
 
-	if (set_op_ctx) {
-		put_gsh_export(export);
-		release_root_op_context();
-	}
+	if (set_op_ctx)
+		release_op_context();
 }
 
 /**
@@ -92,7 +91,7 @@ static void state_blocked_lock_caller(struct fridgethr_context *ctx)
  * We use this wrapper so we can avoid having to rewrite every async
  * func.  Later on we might want to remove it.
  *
- * @param[in] ctx Thread context, containing arguments.
+ * @param[in] ctx Thread fridge context, containing arguments.
  */
 static void state_async_func_caller(struct fridgethr_context *ctx)
 {
