@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: LGPL-3.0-or-later
 /*
  * vim:noexpandtab:shiftwidth=8:tabstop=8:
  *
@@ -54,7 +55,7 @@
  * each segment returned by FSAL_layoutget to an existing state of
  * type STATE_TYPE_LAYOUT.
  *
- * @note state_lock must be held for write.
+ * @note st_lock must be held.
  *
  * @param[in] state           The layout state.
  * @param[in] segment         Layout segment itself granted by the FSAL
@@ -105,7 +106,7 @@ state_status_t state_add_segment(state_t *state, struct pnfs_segment *segment,
  *
  * This function must be called with the mutex lock held.
  *
- * @param[in] segment Segment to delete
+ * @param[in] segment The segment to delete
  *
  * @return State status.
  */
@@ -122,7 +123,7 @@ state_status_t state_delete_segment(state_layout_segment_t *segment)
  * This function finds a layout corresponding to a given file,
  * clientid, and layout type if one exists.
  *
- * @note state_lock MUST be held for read
+ * @note st_lock MUST be held
  *
  * @param[in]  obj    File
  * @param[in]  owner  The state owner.  This must be a clientid owner.
@@ -166,7 +167,7 @@ void revoke_owner_layouts(state_owner_t *client_owner)
 {
 	state_t *state, *first;
 	struct fsal_obj_handle *obj;
-	struct gsh_export *saved_export = op_ctx->ctx_export;
+	struct saved_export_context saved;
 	struct gsh_export *export;
 	int errcnt = 0;
 	struct glist_head *glist, *glistn;
@@ -190,7 +191,7 @@ void revoke_owner_layouts(state_owner_t *client_owner)
 
 		/* We set first to the first state we look in this iteration.
 		 * If the current state matches the first state, it implies
-		 * that went through the entire list without droping the lock
+		 * that went through the entire list without dropping the lock
 		 * guarding the list. So nothing more left to process.
 		 */
 		if (first == NULL)
@@ -201,8 +202,7 @@ void revoke_owner_layouts(state_owner_t *client_owner)
 		/* Move entry to end of list to handle errors and skipping of
 		 * non-layout states.
 		 */
-		glist_del(&state->state_owner_list);
-		glist_add_tail(
+		glist_move_tail(
 			&client_owner->so_owner.so_nfs4_owner.so_state_list,
 			&state->state_owner_list);
 
@@ -219,14 +219,14 @@ void revoke_owner_layouts(state_owner_t *client_owner)
 
 		inc_state_t_ref(state);
 
-		/* Set up the op_context with the proper export */
-		op_ctx->ctx_export = export;
-		op_ctx->fsal_export = export->fsal_export;
+		/* Get a ref to the proper export and add it to op_ctx */
+		get_gsh_export_ref(export);
+		save_op_context_export_and_set_export(&saved, export);
 
 		PTHREAD_MUTEX_unlock(&client_owner->so_mutex);
 		so_mutex_held = false;
 
-		PTHREAD_RWLOCK_wrlock(&obj->state_hdl->state_lock);
+		STATELOCK_lock(obj);
 
 		(void) nfs4_return_one_state(obj,
 					     LAYOUTRETURN4_FILE,
@@ -243,15 +243,15 @@ void revoke_owner_layouts(state_owner_t *client_owner)
 				"Layout state not destroyed during lease expiry.");
 		}
 
-		PTHREAD_RWLOCK_unlock(&obj->state_hdl->state_lock);
+		STATELOCK_unlock(obj);
 
 		/* Release the reference taken above */
 		obj->obj_ops->put_ref(obj);
-		put_gsh_export(export);
 		dec_state_t_ref(state);
+		restore_op_context_export(&saved);
 
 		if (errcnt < STATE_ERR_MAX) {
-			/* Loop again, but since we droped the so_mutex, we
+			/* Loop again, but since we dropped the so_mutex, we
 			 * must restart.
 			 */
 			goto again;
@@ -274,11 +274,6 @@ void revoke_owner_layouts(state_owner_t *client_owner)
 			 "Could not complete cleanup of layouts for client owner %s",
 			 str);
 	}
-
-	op_ctx->ctx_export = saved_export;
-
-	if (saved_export != NULL)
-		op_ctx->fsal_export = op_ctx->ctx_export->fsal_export;
 }
 
 /** @} */
