@@ -1294,7 +1294,6 @@ bool hold_state_owner(state_owner_t *owner)
  */
 void state_wipe_file(struct fsal_obj_handle *obj)
 {
-	bool release;
 	/*
 	 * currently, only REGULAR files can have state; byte range locks and
 	 * stateid (for v4).  In the future, 4.1, directories could have
@@ -1306,7 +1305,7 @@ void state_wipe_file(struct fsal_obj_handle *obj)
 
 	STATELOCK_lock(obj);
 
-	release = state_lock_wipe(obj->state_hdl);
+	state_lock_wipe(obj->state_hdl);
 #ifdef _USE_NLM
 	state_share_wipe(obj->state_hdl);
 #endif /* _USE_NLM */
@@ -1317,11 +1316,6 @@ void state_wipe_file(struct fsal_obj_handle *obj)
 #ifdef DEBUG_SAL
 	dump_all_states();
 #endif
-
-	if (release) {
-		/* Drop the ref for the lock_list */
-		obj->obj_ops->put_ref(obj);
-	}
 }
 
 #ifdef DEBUG_SAL
