@@ -94,8 +94,8 @@ namespace {
       status = test_file->obj_ops->close2(test_file, test_file_state);
       EXPECT_EQ(0, status.major);
 
-      test_file_state->state_exp->exp_ops.free_state(test_file_state->state_exp,
-						     test_file_state);
+      op_ctx->fsal_export->exp_ops.free_state(op_ctx->fsal_export,
+					      test_file_state);
       EXPECT_EQ(0, status.major);
 
       status = fsal_remove(test_root, TEST_FILE);
