@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: LGPL-3.0-or-later
 // -*- mode:C; tab-width:8; c-basic-offset:2; indent-tabs-mode:t -*-
 // vim: ts=8 sw=2 smarttab
 /*
@@ -81,7 +82,7 @@ namespace {
       fsal_status_t status;
       char fname[NAMELEN];
       struct fsal_obj_handle *obj;
-      struct attrlist attrs_out;
+      struct fsal_attrlist attrs_out;
 
       MknodeEmptyLatencyTest::SetUp();
 
