@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: LGPL-3.0-or-later
 // -*- mode:C; tab-width:8; c-basic-offset:2; indent-tabs-mode:t -*-
 // vim: ts=8 sw=2 smarttab
 /*
@@ -73,7 +74,7 @@ namespace {
 
     virtual void SetUp() {
       fsal_status_t status;
-      struct attrlist attrs_out;
+      struct fsal_attrlist attrs_out;
 
       gtest::GaneshaFSALBaseTest::SetUp();
 
@@ -156,7 +157,7 @@ namespace {
   }
 
   enum fsal_dir_result trc_populate_dirent(const char *name,
-      struct fsal_obj_handle *obj, struct attrlist *attrs, void *dir_state,
+      struct fsal_obj_handle *obj, struct fsal_attrlist *attrs, void *dir_state,
       fsal_cookie_t cookie) {
     rd_state_t *st = (rd_state_t*)dir_state;
     struct gsh_buffdesc fh_desc;
