@@ -1,3 +1,4 @@
+/* SPDX-License-Identifier: LGPL-3.0-or-later */
 /*
  * Copyright © 2012 Linux Box Corporation
  *
@@ -87,8 +88,6 @@
 
 #if ((ATOMIC_GCC_VERSION) >= 40700)
 #define GCC_ATOMIC_FUNCTIONS 1
-#elif defined(__APPLE__) && defined(__x86_64__)
-#include "atomic_x86_64.h"
 #elif ((ATOMIC_GCC_VERSION) >= 40100)
 #define GCC_SYNC_FUNCTIONS 1
 #else
@@ -1503,7 +1502,7 @@ static inline size_t atomic_postdec_size_t(size_t *var)
  * This function atomic clears the bits indicated.
  *
  * @param[in,out] var  Pointer to the value to modify
- * @param[in]     bits Bits to clear
+ * @param[in]     bits The bits to clear
  *
  * @return The value after clearing.
  */
@@ -1526,7 +1525,7 @@ static inline uint64_t atomic_clear_uint64_t_bits(uint64_t *var, uint64_t bits)
  * This function atomic clears the bits indicated.
  *
  * @param[in,out] var  Pointer to the value to modify
- * @param[in]     bits Bits to set
+ * @param[in]     bits The bits to set
  *
  * @return The value after setting.
  */
@@ -1549,7 +1548,7 @@ static inline uint64_t atomic_set_uint64_t_bits(uint64_t *var, uint64_t bits)
  * This function atomic clears the bits indicated.
  *
  * @param[in,out] var  Pointer to the value to modify
- * @param[in]     bits Bits to clear
+ * @param[in]     bits The bits to clear
  *
  * @return The value after clearing.
  */
@@ -1572,7 +1571,7 @@ static inline uint32_t atomic_clear_uint32_t_bits(uint32_t *var, uint32_t bits)
  * This function atomic clears the bits indicated.
  *
  * @param[in,out] var  Pointer to the value to modify
- * @param[in]     bits Bits to set
+ * @param[in]     bits The bits to set
  *
  * @return The value after setting.
  */
@@ -1595,7 +1594,7 @@ static inline uint32_t atomic_set_uint32_t_bits(uint32_t *var, uint32_t bits)
  * This function atomic clears the bits indicated.
  *
  * @param[in,out] var  Pointer to the value to modify
- * @param[in]     bits Bits to clear
+ * @param[in]     bits The bits to clear
  *
  * @return The value after clearing.
  */
@@ -1618,7 +1617,7 @@ static inline uint16_t atomic_clear_uint16_t_bits(uint16_t *var, uint16_t bits)
  * This function atomic clears the bits indicated.
  *
  * @param[in,out] var  Pointer to the value to modify
- * @param[in]     bits Bits to set
+ * @param[in]     bits The bits to set
  *
  * @return The value after setting.
  */
@@ -1641,7 +1640,7 @@ static inline uint16_t atomic_set_uint16_t_bits(uint16_t *var, uint16_t bits)
  * This function atomic clears the bits indicated.
  *
  * @param[in,out] var  Pointer to the value to modify
- * @param[in]     bits Bits to clear
+ * @param[in]     bits The bits to clear
  *
  * @return The value after clearing.
  */
@@ -1664,7 +1663,7 @@ static inline uint8_t atomic_clear_uint8_t_bits(uint8_t *var, uint8_t bits)
  * This function atomic clears the bits indicated.
  *
  * @param[in,out] var  Pointer to the value to modify
- * @param[in]     bits Bits to set
+ * @param[in]     bits The bits to set
  *
  * @return The value after setting.
  */
@@ -1692,7 +1691,7 @@ static inline uint8_t atomic_set_uint8_t_bits(uint8_t *var, uint8_t bits)
  * This function atomic clears the bits indicated.
  *
  * @param[in,out] var  Pointer to the value to modify
- * @param[in]     bits Bits to clear
+ * @param[in]     bits The bits to clear
  *
  * @return The value before clearing.
  */
@@ -1717,7 +1716,7 @@ static inline uint64_t atomic_postclear_uint64_t_bits(uint64_t *var,
  * This function atomic clears the bits indicated.
  *
  * @param[in,out] var  Pointer to the value to modify
- * @param[in]     bits Bits to set
+ * @param[in]     bits The bits to set
  *
  * @return The value before setting.
  */
@@ -1742,7 +1741,7 @@ static inline uint64_t atomic_postset_uint64_t_bits(uint64_t *var,
  * This function atomic clears the bits indicated.
  *
  * @param[in,out] var  Pointer to the value to modify
- * @param[in]     bits Bits to clear
+ * @param[in]     bits The bits to clear
  *
  * @return The value before clearing.
  */
@@ -1767,7 +1766,7 @@ static inline uint32_t atomic_postclear_uint32_t_bits(uint32_t *var,
  * This function atomic clears the bits indicated.
  *
  * @param[in,out] var  Pointer to the value to modify
- * @param[in]     bits Bits to set
+ * @param[in]     bits The bits to set
  *
  * @return The value before setting.
  */
@@ -1792,7 +1791,7 @@ static inline uint32_t atomic_postset_uint32_t_bits(uint32_t *var,
  * This function atomic clears the bits indicated.
  *
  * @param[in,out] var  Pointer to the value to modify
- * @param[in]     bits Bits to clear
+ * @param[in]     bits The bits to clear
  *
  * @return The value before clearing.
  */
@@ -1817,7 +1816,7 @@ static inline uint16_t atomic_postclear_uint16_t_bits(uint16_t *var,
  * This function atomic clears the bits indicated.
  *
  * @param[in,out] var  Pointer to the value to modify
- * @param[in]     bits Bits to set
+ * @param[in]     bits The bits to set
  *
  * @return The value before setting.
  */
@@ -1842,7 +1841,7 @@ static inline uint16_t atomic_postset_uint16_t_bits(uint16_t *var,
  * This function atomic clears the bits indicated.
  *
  * @param[in,out] var  Pointer to the value to modify
- * @param[in]     bits Bits to clear
+ * @param[in]     bits The bits to clear
  *
  * @return The value before clearing.
  */
@@ -1865,7 +1864,7 @@ static inline uint8_t atomic_postclear_uint8_t_bits(uint8_t *var, uint8_t bits)
  * This function atomic clears the bits indicated.
  *
  * @param[in,out] var  Pointer to the value to modify
- * @param[in]     bits Bits to set
+ * @param[in]     bits The bits to set
  *
  * @return The value before setting.
  */
