@@ -1,10 +1,28 @@
+/* SPDX-License-Identifier: LGPL-3.0-or-later */
 /*
- * vim:noexpandtab:shiftwidth=8:tabstop=8:
- */
-
-/**
  * @file common_utils.h
  * @brief Common tools for printing, parsing, ....
+ *
+ * Copyright CEA/DAM/DIF  (2008)
+ * contributeur : Philippe DENIEL   philippe.deniel@cea.fr
+ *                Thomas LEIBOVICI  thomas.leibovici@cea.fr
+ *
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 3 of the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+ *
+ * ---------------------------------------
  */
 
 #ifndef COMMON_UTILS_H
@@ -16,9 +34,11 @@
 #include <string.h>
 #include <stdbool.h>
 #include <stdint.h>
+#include <netdb.h>
 
 #include "gsh_types.h"
 #include "log.h"
+#include "idmapper.h"
 
 /**
  * BUILD_BUG_ON - break compile if a condition is true.
@@ -69,8 +89,6 @@ extern size_t gsh_strnlen(const char *s, size_t max);
 #endif
 
 #if defined(__APPLE__)
-#define clock_gettime(a, ts) portable_clock_gettime(ts)
-extern int portable_clock_gettime(struct timespec *ts);
 #define pthread_yield() pthread_yield_np()
 #undef SCANDIR_CONST
 #define SCANDIR_CONST
@@ -222,7 +240,7 @@ extern int portable_clock_gettime(struct timespec *ts);
 				     "Acquired mutex %p (%s) at %s:%d",	\
 				     _mtx, #_mtx,			\
 				     __FILE__, __LINE__);		\
-		} else{							\
+		} else {						\
 			LogCrit(COMPONENT_RW_LOCK,			\
 				"Error %d, acquiring mutex %p (%s) "	\
 				"at %s:%d", rc, _mtx, #_mtx,		\
@@ -231,6 +249,35 @@ extern int portable_clock_gettime(struct timespec *ts);
 		}							\
 	} while (0)
 
+static inline int PTHREAD_mutex_trylock(pthread_mutex_t *mtx,
+					const char *mtx_name)
+{
+	int rc;
+
+	rc = pthread_mutex_trylock(mtx);
+	if (rc == 0) {
+		LogFullDebug(COMPONENT_RW_LOCK,
+			     "Acquired mutex %p (%s) at %s:%d",
+			     mtx, mtx_name,
+			     __FILE__, __LINE__);
+	} else if (rc == EBUSY) {
+		LogFullDebug(COMPONENT_RW_LOCK,
+			     "Busy mutex %p (%s) at %s:%d",
+			     mtx, mtx_name,
+			     __FILE__, __LINE__);
+	} else {
+		LogCrit(COMPONENT_RW_LOCK,
+			"Error %d, acquiring mutex %p (%s) at %s:%d",
+			rc, mtx, mtx_name,
+			__FILE__, __LINE__);
+		abort();
+	}
+
+	return rc;
+}
+
+#define PTHREAD_MUTEX_trylock(_mtx)  PTHREAD_mutex_trylock(_mtx, #_mtx)
+
 /**
  * @brief Logging mutex unlock
  *
@@ -247,7 +294,7 @@ extern int portable_clock_gettime(struct timespec *ts);
 				     "Released mutex %p (%s) at %s:%d",	\
 				     _mtx, #_mtx,			\
 				     __FILE__, __LINE__);		\
-		} else{							\
+		} else {						\
 			LogCrit(COMPONENT_RW_LOCK,			\
 				"Error %d, releasing mutex %p (%s) "	\
 				"at %s:%d", rc, _mtx, #_mtx,		\
@@ -333,7 +380,7 @@ extern int portable_clock_gettime(struct timespec *ts);
 	} while (0)
 
 /**
- * @brief Logging condtion variable destroy
+ * @brief Logging condition variable destroy
  *
  * @param[in,out] _cond The condition variable to destroy
  */
@@ -360,7 +407,7 @@ extern int portable_clock_gettime(struct timespec *ts);
 /**
  * @brief Inline functions for timespec math
  *
- * This is for timespec math.  If you want to do
+ * This is for timespec math.  If you want to
  * do the same kinds of math on timeval values,
  * See timeradd(3) in GLIBC.
  *
@@ -370,6 +417,13 @@ extern int portable_clock_gettime(struct timespec *ts);
  * but too complicated to explain here).
  */
 
+#ifdef __APPLE__
+/* For accessing timespec values on 'struct stat' */
+#define st_atim st_atimespec
+#define st_mtim st_mtimespec
+#define st_ctim st_ctimespec
+#endif
+
 /**
  * @brief Get the abs difference between two timespecs in nsecs
  *
@@ -396,6 +450,20 @@ timespec_diff(const struct timespec *start,
 	}
 }
 
+/**
+ * @brief update timespec fields atomically.
+ *
+ */
+static inline void
+timespec_update(const struct timespec *dest,
+		const struct timespec *src)
+{
+	(void)atomic_store_uint64_t((uint64_t *)&dest->tv_sec,
+				    (uint64_t)src->tv_sec);
+	(void)atomic_store_uint64_t((uint64_t *)&dest->tv_nsec,
+				    (uint64_t)src->tv_nsec);
+}
+
 /**
  * @brief Convert a timespec to an elapsed time interval
  *
@@ -509,4 +577,66 @@ static inline void now(struct timespec *ts)
 	}
 }
 
+/*wrapper for gethostname to capture auth stats, if required */
+static inline int gsh_gethostname(char *name, size_t len, bool stats)
+{
+	int ret;
+	struct timespec s_time, e_time;
+
+	if (stats)
+		now(&s_time);
+
+	ret = gethostname(name, len);
+
+	if (!ret && stats) {
+		now(&e_time);
+		dns_stats_update(&s_time, &e_time);
+	}
+	return ret;
+}
+
+/*wrapper for getaddrinfo to capture auth stats, if required */
+static inline int gsh_getaddrinfo(const char *node,
+				  const char *service,
+				  const struct addrinfo *hints,
+				  struct addrinfo **res,
+				  bool stats)
+{
+	int ret;
+	struct timespec s_time, e_time;
+
+	if (stats)
+		now(&s_time);
+
+	ret = getaddrinfo(node, service, hints, res);
+
+	if (!ret && stats) {
+		now(&e_time);
+		dns_stats_update(&s_time, &e_time);
+	}
+	return ret;
+}
+
+/*wrapper for getnameinfo to capture auth stats, if required */
+static inline int gsh_getnameinfo(const struct sockaddr *addr,
+				  socklen_t addrlen,
+				  char *host, socklen_t hostlen,
+				  char *serv, socklen_t servlen,
+				  int flags, bool stats)
+{
+	int ret;
+	struct timespec s_time, e_time;
+
+	if (stats)
+		now(&s_time);
+
+	ret = getnameinfo(addr, addrlen, host, hostlen, serv, servlen, flags);
+
+	if (!ret && stats) {
+		now(&e_time);
+		dns_stats_update(&s_time, &e_time);
+	}
+	return ret;
+}
+
 #endif				/* !COMMON_UTILS_H */
