@@ -1,3 +1,4 @@
+/* SPDX-License-Identifier: LGPL-3.0-or-later */
 /* ----------------------------------------------------------------------------
  * Copyright CEA/DAM/DIF  (2007)
  * contributeur : Thomas LEIBOVICI  thomas.leibovici@cea.fr
@@ -77,7 +78,8 @@ enum config_type {
 	CONFIG_BOOLBIT,
 	CONFIG_IP_ADDR,
 	CONFIG_BLOCK,
-	CONFIG_PROC
+	CONFIG_PROC,
+	CONFIG_DEPRECATED,
 };
 
 #define CONFIG_UNIQUE		0x001  /*< only one instance allowed */
@@ -103,7 +105,8 @@ struct config_error_type {
 	bool parse:1;		/*< parser rules */
 	bool init:1;		/*< block initialization */
 	bool fsal:1;		/*< fsal load failure */
-	bool export_:1;		/*< export create failure */
+	bool cur_exp_create_err:1; /*< current export create failure */
+	bool all_exp_create_err:1; /*< all exports create failure */
 	bool resource:1;	/*< system resource */
 	bool unique:1;		/*< unique block/param */
 	bool invalid:1;		/*< invalid param value */
@@ -112,6 +115,7 @@ struct config_error_type {
 	bool exists:1;		/*< block already exists */
 	bool internal:1;        /*< internal error */
 	bool bogus:1;		/*< bogus (deprecated?) param */
+	bool deprecated:1;	/*< A config item identified as deprecated */
 	bool dispose:1;		/*< Not actually an error, but we need to
 				    dispose of the config item anyway. */
 	uint32_t errors;	/*< cumulative error count for parse+proc */
@@ -134,13 +138,14 @@ static inline bool config_error_is_fatal(struct config_error_type *err_type)
 }
 
 /**
- * @brief Test for errors that make the processed block unuseable
+ * @brief Test for errors that make the processed block unusable
  */
 
 static inline bool config_error_is_crit(struct config_error_type *err_type)
 {
 	return config_error_is_fatal(err_type) || err_type->internal ||
-		err_type->invalid || err_type->export_ || err_type->missing;
+		err_type->invalid || err_type->all_exp_create_err ||
+		err_type->missing;
 }
 
 /**
@@ -153,6 +158,30 @@ static inline bool config_error_is_harmless(struct config_error_type *err_type)
 		 err_type->unique || err_type->exists || err_type->dispose);
 }
 
+/**
+ * @brief Test for errors that make the processed block unusable for current
+ *        export
+ */
+
+static inline bool cur_exp_config_error_is_crit(
+					struct config_error_type *err_type)
+{
+	return config_error_is_fatal(err_type) || err_type->internal ||
+		err_type->invalid || err_type->cur_exp_create_err ||
+		err_type->missing;
+}
+
+/**
+ * @brief Test for errors that will not cause problems
+ */
+
+static inline bool cur_exp_config_error_is_harmless(
+					struct config_error_type *err_type)
+{
+	return !(cur_exp_config_error_is_crit(err_type) ||
+		 err_type->unique || err_type->exists || err_type->dispose);
+}
+
 /**
  * @brief Test that there are no errors at all
  *
@@ -245,7 +274,7 @@ struct config_item_list {
  *   non-settable FSAL parameters.  It returns a pointer to the space.
  *
  *   link_mem != NULL, self_struct != NULL
- *   This call is to free or release any resouces in this allocated
+ *   This call is to free or release any resources in this allocated
  *   or referenced block. The link_mem argument is passed so that
  *   dereferences as above are possible.  It should not attempt to
  *   change the link_mem such as doing glist removes.  This is only
@@ -271,6 +300,7 @@ struct config_item_list {
 
 struct config_item {
 	char *name;
+	char *altname;
 	enum config_type type; /* switches union */
 	int flags;
 	union {
@@ -289,16 +319,19 @@ struct config_item {
 			int16_t minval;
 			int16_t maxval;
 			int16_t def;
+			bool zero_ok;
 		} i16;
 		struct { /* CONFIG_UINT16 */
 			uint16_t minval;
 			uint16_t maxval;
 			uint16_t def;
+			bool zero_ok;
 		} ui16;
 		struct { /* CONFIG_INT32 */
 			int32_t minval;
 			int32_t maxval;
 			int32_t def;
+			bool zero_ok;
 			uint32_t bit;
 			size_t set_off;
 		} i32;
@@ -306,11 +339,13 @@ struct config_item {
 			uint32_t minval;
 			uint32_t maxval;
 			uint32_t def;
+			bool zero_ok;
 		} ui32;
 		struct { /* CONFIG_INT64 */
 			int64_t minval;
 			int64_t maxval;
 			int64_t def;
+			bool zero_ok;
 			uint32_t bit;
 			size_t set_off;
 		} i64;
@@ -318,6 +353,7 @@ struct config_item {
 			uint64_t minval;
 			uint64_t maxval;
 			uint64_t def;
+			bool zero_ok;
 			uint32_t bit;
 			size_t set_off;
 		} ui64;
@@ -348,6 +384,8 @@ struct config_item {
 			void (*display)(const char *step,
 					void *node, void *link_mem,
 					void *self_struct);
+			bool (*check)(void *self_struct,
+					struct config_error_type *err_type);
 		} blk;
 		struct { /* CONFIG_PROC */
 			size_t set_off;
@@ -359,6 +397,9 @@ struct config_item {
 				       void *cnode,
 				       struct config_error_type *err_type);
 		} proc;
+		struct { /* CONFIG_DEPRECATED */
+			const char *message;
+		} deprecated;
 	} u;
 	size_t off; /* offset into struct pointed to by opaque_dest */
 };
@@ -589,6 +630,7 @@ struct config_item {
 	  .u.str.def = _def_,			    \
 	  .off = offsetof(struct _struct_, _mem_)   \
 	}
+
 #define CONF_MAND_STR(_name_, _minsize_, _maxsize_, _def_, _struct_, _mem_) \
 	{ .name = _name_,			    \
 	  .type = CONFIG_STRING,		    \
@@ -641,9 +683,10 @@ struct config_item {
 #define CONF_ITEM_I16(_name_, _min_, _max_, _def_, _struct_, _mem_) \
 	{ .name = _name_,			    \
 	  .type = CONFIG_INT16,		    \
-	  .u.ui16.minval = _min_,		    \
-	  .u.ui16.maxval = _max_,		    \
-	  .u.ui16.def = _def_,			    \
+	  .u.i16.minval = _min_,		    \
+	  .u.i16.maxval = _max_,		    \
+	  .u.i16.def = _def_,			    \
+	  .u.i16.zero_ok = (_min_ <= 0),	    \
 	  .off = offsetof(struct _struct_, _mem_)   \
 	}
 
@@ -653,6 +696,7 @@ struct config_item {
 	  .u.ui16.minval = _min_,		    \
 	  .u.ui16.maxval = _max_,		    \
 	  .u.ui16.def = _def_,			    \
+	  .u.ui16.zero_ok = (_min_ == 0),	    \
 	  .off = offsetof(struct _struct_, _mem_)   \
 	}
 
@@ -663,6 +707,7 @@ struct config_item {
 	  .u.ui16.minval = _min_,		    \
 	  .u.ui16.maxval = _max_,		    \
 	  .u.ui16.def = _def_,			    \
+	  .u.ui16.zero_ok = (_min_ == 0),	    \
 	  .off = offsetof(struct _struct_, _mem_)   \
 	}
 
@@ -672,6 +717,7 @@ struct config_item {
 	  .u.i32.minval = _min_,		    \
 	  .u.i32.maxval = _max_,		    \
 	  .u.i32.def = _def_,			    \
+	  .u.i32.zero_ok = (_min_ <= 0),	    \
 	  .off = offsetof(struct _struct_, _mem_)   \
 	}
 
@@ -683,6 +729,7 @@ struct config_item {
 	  .u.i32.minval = _min_,		    \
 	  .u.i32.maxval = _max_,		    \
 	  .u.i32.def = _def_,			    \
+	  .u.i32.zero_ok = (_min_ <= 0),	    \
 	  .u.i32.bit = _bit_,		   	    \
 	  .u.i32.set_off = offsetof(struct _struct_, _set_),   \
 	  .off = offsetof(struct _struct_, _mem_)   \
@@ -696,6 +743,7 @@ struct config_item {
 	  .u.i64.minval = INT32_MIN,		    \
 	  .u.i64.maxval = UINT32_MAX,		    \
 	  .u.i64.def = _def_,			    \
+	  .u.i64.zero_ok = true,		    \
 	  .u.i64.bit = _bit_,			    \
 	  .u.i64.set_off = offsetof(struct _struct_, _set_),   \
 	  .off = offsetof(struct _struct_, _mem_)   \
@@ -707,6 +755,17 @@ struct config_item {
 	  .u.ui32.minval = _min_,		    \
 	  .u.ui32.maxval = _max_,		    \
 	  .u.ui32.def = _def_,			    \
+	  .u.ui32.zero_ok = (_min_ == 0),	    \
+	  .off = offsetof(struct _struct_, _mem_)   \
+	}
+
+#define CONF_ITEM_UI32_ZERO(_name_, _min_, _max_, _def_, _struct_, _mem_) \
+	{ .name = _name_,			    \
+	  .type = CONFIG_UINT32,		    \
+	  .u.ui32.minval = _min_,		    \
+	  .u.ui32.maxval = _max_,		    \
+	  .u.ui32.def = _def_,			    \
+	  .u.ui32.zero_ok = true,		    \
 	  .off = offsetof(struct _struct_, _mem_)   \
 	}
 
@@ -717,6 +776,7 @@ struct config_item {
 	  .u.ui32.minval = _min_,		    \
 	  .u.ui32.maxval = _max_,		    \
 	  .u.ui32.def = _def_,			    \
+	  .u.ui32.zero_ok = (_min_ == 0),	    \
 	  .off = offsetof(struct _struct_, _mem_)   \
 	}
 
@@ -727,15 +787,30 @@ struct config_item {
 	  .u.ui32.minval = 0,			    \
 	  .u.ui32.maxval = 0777,		    \
 	  .u.ui32.def = _def_,			    \
+	  .u.ui32.zero_ok = true,	    \
 	  .off = offsetof(struct _struct_, _mem_)   \
 	}
 
 #define CONF_ITEM_I64(_name_, _min_, _max_, _def_, _struct_, _mem_) \
 	{ .name = _name_,			    \
-	  .type = CONFIG_INT64,		    \
+	  .type = CONFIG_INT64,			    \
 	  .u.i64.minval = _min_,		    \
 	  .u.i64.maxval = _max_,		    \
 	  .u.i64.def = _def_,			    \
+	  .u.i64.zero_ok = (_min_ <= 0),	    \
+	  .off = offsetof(struct _struct_, _mem_)   \
+	}
+
+/* The following exists because I64 is used for time_t which are likely
+ * parameters to have a non-zero min but also allow 0.
+ */
+#define CONF_ITEM_I64_ZERO(_name_, _min_, _max_, _def_, _struct_, _mem_) \
+	{ .name = _name_,			    \
+	  .type = CONFIG_INT64,			    \
+	  .u.i64.minval = _min_,		    \
+	  .u.i64.maxval = _max_,		    \
+	  .u.i64.def = _def_,			    \
+	  .u.i64.zero_ok = true,		    \
 	  .off = offsetof(struct _struct_, _mem_)   \
 	}
 
@@ -745,6 +820,17 @@ struct config_item {
 	  .u.ui64.minval = _min_,		    \
 	  .u.ui64.maxval = _max_,		    \
 	  .u.ui64.def = _def_,			    \
+	  .u.ui64.zero_ok = (_min_ == 0),	    \
+	  .off = offsetof(struct _struct_, _mem_)   \
+	}
+
+#define CONF_ITEM_UI64_ZERO(_name_, _min_, _max_, _def_, _struct_, _mem_) \
+	{ .name = _name_,			    \
+	  .type = CONFIG_UINT64,		    \
+	  .u.ui64.minval = _min_,		    \
+	  .u.ui64.maxval = _max_,		    \
+	  .u.ui64.def = _def_,			    \
+	  .u.ui64.zero_ok = true,		    \
 	  .off = offsetof(struct _struct_, _mem_)   \
 	}
 
@@ -756,11 +842,21 @@ struct config_item {
 	  .u.ui64.minval = _min_,		    \
 	  .u.ui64.maxval = _max_,		    \
 	  .u.ui64.def = _def_,			    \
+	  .u.ui64.zero_ok = (_min_ == 0),	    \
 	  .u.ui64.bit = _bit_,			    \
 	  .u.ui64.set_off = offsetof(struct _struct_, _set_),   \
 	  .off = offsetof(struct _struct_, _mem_)   \
 	}
 
+/**
+ * Note that message can be NULL.
+ */
+#define CONF_ITEM_DEPRECATED(_name_, _message_) \
+	{ .name = _name_,			    \
+	  .type = CONFIG_DEPRECATED,		    \
+	  .u.deprecated.message = _message_,	    \
+	}
+
 #define CONFIG_EOL {.name = NULL, .type = CONFIG_NULL}
 
 /**
