@@ -1,3 +1,4 @@
+/* SPDX-License-Identifier: LGPL-3.0-or-later */
 /*
  * Copyright CEA/DAM/DIF  (2008)
  * contributeur : Philippe DENIEL   philippe.deniel@cea.fr
@@ -61,7 +62,7 @@ typedef enum {
 } object_file_type_t;
 
 /* ---------------
- *  FS dependant :
+ *  FS dependent :
  * --------------*/
 
 /* export object
@@ -155,6 +156,9 @@ typedef uint32_t fsal_aceflag_t;
 #define FSAL_ACE_FLAG_FAILED          0x00000020
 #define FSAL_ACE_FLAG_GROUP_ID        0x00000040
 #define FSAL_ACE_FLAG_INHERITED       0x00000080
+#define FSAL_ACE_FLAG_MASK_READ_DENY    0x00000100
+#define FSAL_ACE_FLAG_MASK_WRITE_DENY   0x00000200
+#define FSAL_ACE_FLAG_MASK_EXECUTE_DENY 0x00000400
 
 /** ACE internal flags */
 
@@ -195,6 +199,7 @@ typedef uint32_t fsal_aceperm_t;
 #define FSAL_ACE_SPECIAL_OWNER              1
 #define FSAL_ACE_SPECIAL_GROUP              2
 #define FSAL_ACE_SPECIAL_EVERYONE           3
+#define FSAL_ACE_SPECIAL_MASK               4
 
 typedef struct fsal_ace__ {
 
@@ -283,6 +288,8 @@ typedef struct fsal_acl_data__ {
 	IS_FSAL_ACE_USER(ACE, FSAL_ACE_SPECIAL_GROUP)
 #define IS_FSAL_ACE_SPECIAL_EVERYONE(ACE) \
 	  IS_FSAL_ACE_USER(ACE, FSAL_ACE_SPECIAL_EVERYONE)
+#define IS_FSAL_ACE_SPECIAL_MASK(ACE) \
+	IS_FSAL_ACE_USER(ACE, FSAL_ACE_SPECIAL_MASK)
 #define IS_FSAL_ACE_SPECIAL(ACE) \
 	(IS_FSAL_ACE_SPECIAL_OWNER(ACE) || \
 	 IS_FSAL_ACE_SPECIAL_GROUP(ACE) || \
@@ -302,6 +309,8 @@ typedef struct fsal_acl_data__ {
 #define IS_FSAL_DIR_APPLICABLE(ACE)  \
 	(!IS_FSAL_ACE_BIT(GET_FSAL_ACE_IFLAG(ACE),	\
 			  FSAL_ACE_IFLAG_EXCLUDE_DIRS))
+#define IS_FSAL_ACE_IFLAG(ACE, BIT) \
+	IS_FSAL_ACE_BIT(GET_FSAL_ACE_IFLAG(ACE), (BIT))
 
 /* Macros for NFS4 ACE permissions. */
 
@@ -402,7 +411,7 @@ typedef uint64_t attrmask_t;
 #define ATTR_MTIME 0x0000000000008000LL
 /* space used by this file. */
 #define ATTR_SPACEUSED 0x0000000000010000LL
-/* This bit indicates that an error occured during getting object attributes */
+/* This bit indicates that an error occurred during getting object attributes */
 #define ATTR_RDATTR_ERR 0x8000000000000000LL
 /* Generation number */
 #define ATTR_GENERATION 0x0000000000080000LL
@@ -429,6 +438,8 @@ typedef uint64_t attrmask_t;
 		      ATTR_CTIME    | ATTR_SPACEUSED | \
 		      ATTR_FSID)
 
+#define ATTRS_NFS3_ACL (ATTRS_NFS3 | ATTR_ACL)
+
 #define ATTRS_TIME (ATTR_ATIME | ATTR_MTIME | ATTR_CTIME)
 #define ATTRS_CREDS (ATTR_OWNER | ATTR_GROUP)
 
@@ -441,7 +452,7 @@ typedef uint64_t attrmask_t;
 #define ATTRS_SET_TIME (ATTR_ATIME | ATTR_MTIME | \
 			ATTR_ATIME_SERVER | ATTR_MTIME_SERVER)
 
-/** Define the set of attributes contained or derrived from struct stat that
+/** Define the set of attributes contained or derived from struct stat that
  *  are supplied by posix2fsal_attributes.
  */
 #define ATTRS_POSIX (ATTR_TYPE | ATTR_SIZE | ATTR_FSID | ATTR_FILEID |     \
@@ -453,7 +464,7 @@ typedef uint64_t attrmask_t;
  * @brief A list of FS object's attributes.
  */
 
-struct attrlist {
+struct fsal_attrlist {
 	attrmask_t request_mask; /*< Indicates the requested from the FSAL. */
 	attrmask_t valid_mask;	/*< Indicates the attributes to be set or
 				   that have been filled in by the FSAL. */
@@ -524,7 +535,7 @@ struct attrlist {
 
 #define XATTR_NAME_SIZE (MAXNAMLEN + 1)
 
-/** An extented attribute entry */
+/** An extended attribute entry */
 typedef struct fsal_xattrent {
 	uint64_t xattr_id;	/*< xattr index */
 	uint64_t xattr_cookie;	/*< cookie for the next entry */
@@ -598,18 +609,6 @@ static inline fsal_accessflags_t FSAL_ACE4_MASK(fsal_accessflags_t access)
 #define FSAL_EXECUTE_ACCESS (FSAL_MODE_MASK_SET(FSAL_X_OK) | \
 			  FSAL_ACE4_MASK_SET(FSAL_ACE_PERM_EXECUTE))
 
-/* Object handle LRU resource actions
- */
-
-/**
- * @todo ACE: These should probably be moved to cache_inode_lru.h
- */
-
-typedef enum {
-	LRU_CLOSE_FILES = 0x1,
-	LRU_FREE_MEMORY = 0x2
-} lru_actions_t;
-
 /** FSAL_open behavior. */
 
 typedef uint16_t fsal_openflags_t;
@@ -675,17 +674,24 @@ typedef enum enum_fsal_fsinfo_options {
 	fso_whence_is_name,
 	fso_readdir_plus,
 	fso_compliant_eof_behavior,
+	fso_xattr_support,
 } fsal_fsinfo_options_t;
 
 /* The largest maxread and maxwrite value */
 #define FSAL_MAXIOSIZE XDR_BYTES_MAXLEN_IO
 
+/* Default time_delta nanoseconds.
+ * FSALs may set time_delta.tv_nsec to this value or override with a different
+ * value or even fill in programmatically with the result from some call.
+ */
+#define FSAL_DEFAULT_TIME_DELTA_NSEC 100
+
 typedef struct fsal_staticfsinfo_t {
 	uint64_t maxfilesize;	/*< maximum allowed filesize     */
 	uint32_t maxlink;	/*< maximum hard links on a file */
 	uint32_t maxnamelen;	/*< maximum name length */
 	uint32_t maxpathlen;	/*< maximum path length */
-	bool no_trunc;		/*< is it errorneous when name>maxnamelen? */
+	bool no_trunc;		/*< is it erroneous when name>maxnamelen? */
 	bool chown_restricted;	/*< is chown limited to super-user. */
 	bool case_insensitive;	/*< case insensitive FS? */
 	bool case_preserving;	/*< FS preserves case? */
@@ -723,6 +729,7 @@ typedef struct fsal_staticfsinfo_t {
 	bool readdir_plus;	/*< FSAL supports readdir_plus */
 	bool compliant_eof_behavior;	/* FSAL compliant to end-of-file NFS
 					  v3 read behavior. */
+	bool xattr_support;	/*< fsal can support xattrs */
 	int32_t expire_time_parent; /*< Expiration time interval in
 				       seconds for parent handle.
 				       If FS gives information about parent
@@ -789,6 +796,8 @@ typedef enum fsal_errors_t {
 	ERR_FSAL_NO_ACE = 10096,
 	ERR_FSAL_CROSS_JUNCTION = 10097,
 	ERR_FSAL_BADNAME = 10098,
+	ERR_FSAL_NOXATTR = 10099,
+	ERR_FSAL_XATTR2BIG = 10100,
 } fsal_errors_t;
 
 /**
