@@ -1,3 +1,21 @@
+/* SPDX-License-Identifier: LGPL-3.0-or-later */
+/*
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 3 of the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ *
+ */
 
 #undef TRACEPOINT_PROVIDER
 #define TRACEPOINT_PROVIDER nfs_rpc
@@ -16,9 +34,9 @@
 TRACEPOINT_EVENT(
 	nfs_rpc,
 	start,
-	TP_ARGS(request_data_t *, req),
+	TP_ARGS(nfs_request_t *, req),
 	TP_FIELDS(
-		ctf_integer_hex(request_data_t *, req, req)
+		ctf_integer_hex(nfs_request_t *, req, req)
 	)
 )
 
@@ -38,9 +56,9 @@ TRACEPOINT_LOGLEVEL(
 TRACEPOINT_EVENT(
 	nfs_rpc,
 	end,
-	TP_ARGS(request_data_t *, req),
+	TP_ARGS(nfs_request_t *, req),
 	TP_FIELDS(
-		ctf_integer_hex(request_data_t *, req, req)
+		ctf_integer_hex(nfs_request_t *, req, req)
 	)
 )
 
@@ -58,11 +76,11 @@ TRACEPOINT_LOGLEVEL(
 TRACEPOINT_EVENT(
 	nfs_rpc,
 	op_start,
-	TP_ARGS(request_data_t *, req,
+	TP_ARGS(nfs_request_t *, req,
 		const char *, op_name,
 		int, export_id),
 	TP_FIELDS(
-		ctf_integer_hex(request_data_t *, req, req)
+		ctf_integer_hex(nfs_request_t *, req, req)
 		ctf_string(op_name, op_name)
 		ctf_integer(int, export_id, export_id)
 	)
@@ -84,9 +102,9 @@ TRACEPOINT_LOGLEVEL(
 TRACEPOINT_EVENT(
 	nfs_rpc,
 	op_end,
-	TP_ARGS(request_data_t *, req),
+	TP_ARGS(nfs_request_t *, req),
 	TP_FIELDS(
-		ctf_integer_hex(request_data_t *, req, req)
+		ctf_integer_hex(nfs_request_t *, req, req)
 	)
 )
 
@@ -152,6 +170,66 @@ TRACEPOINT_LOGLEVEL(
 	v4op_end,
 	TRACE_INFO)
 
+TRACEPOINT_EVENT(
+	nfs_rpc,
+	before_reply,
+	TP_ARGS(const char *, function,
+		unsigned int, line,
+		void *, xprt),
+	TP_FIELDS(
+		ctf_string(fnc, function)
+		ctf_integer(unsigned int, line, line)
+		ctf_integer_hex(void *, xprt, xprt)
+	)
+)
+
+TRACEPOINT_LOGLEVEL(
+	nfs_rpc,
+	before_reply,
+	TRACE_INFO)
+
+TRACEPOINT_EVENT(
+	nfs_rpc,
+	before_recv,
+	TP_ARGS(const char *, function,
+		unsigned int, line,
+		void *, xprt),
+	TP_FIELDS(
+		ctf_string(fnc, function)
+		ctf_integer(unsigned int, line, line)
+		ctf_integer_hex(void *, xprt, xprt)
+	)
+)
+
+TRACEPOINT_LOGLEVEL(
+	nfs_rpc,
+	before_recv,
+	TRACE_INFO)
+
+TRACEPOINT_EVENT(
+	nfs_rpc,
+	valid,
+	TP_ARGS(const char *, function,
+		unsigned int, line,
+		void *, xprt,
+		unsigned int, prog,
+		unsigned int, vers,
+		unsigned int, proc),
+	TP_FIELDS(
+		ctf_string(fnc, function)
+		ctf_integer(unsigned int, line, line)
+		ctf_integer_hex(void *, xprt, xprt)
+		ctf_integer(unsigned int, prog, prog)
+		ctf_integer(unsigned int, vers, vers)
+		ctf_integer(unsigned int, proc, proc)
+	)
+)
+
+TRACEPOINT_LOGLEVEL(
+	nfs_rpc,
+	valid,
+	TRACE_INFO)
+
 #endif /* GANESHA_LTTNG_NFS_RPC_H */
 
 #undef TRACEPOINT_INCLUDE
