@@ -1,5 +1,33 @@
+/* SPDX-License-Identifier: LGPL-3.0-or-later */
+/*
+ * @file common_utils.h
+ * @brief Common tools for printing, parsing, ....
+ *
+ * Copyright CEA/DAM/DIF  (2008)
+ * contributeur : Philippe DENIEL   philippe.deniel@cea.fr
+ *                Thomas LEIBOVICI  thomas.leibovici@cea.fr
+ *
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 3 of the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+ *
+ * ---------------------------------------
+ */
+
 /* This is a central clearing house for RPC definitions. Nothing
-   should included anything related to RPC except this file */
+ * should included anything related to RPC except this file
+ */
 
 #ifndef GSH_RPC_H
 #define GSH_RPC_H
@@ -114,7 +142,7 @@ typedef struct nfs_krb5_param {
 	/** Kerberos keytab.  Defaults to DEFAULT_NFS_KEYTAB, settable
 	    with KeytabPath. */
 	char *keytab;
-	/** The ganesha credential cache.  Defautls to
+	/** The ganesha credential cache.  Defaults to
 	    DEFAULT_NFS_CCACHE_DIR, unsettable by user. */
 	char *ccache_dir;
 	/**
@@ -142,7 +170,7 @@ void log_sperror_gss(char *, OM_uint32, OM_uint32);
 const char *str_gc_proc(rpc_gss_proc_t);
 #endif /* _HAVE_GSSAPI */
 
-bool copy_xprt_addr(sockaddr_t *, SVCXPRT *);
+void copy_xprt_addr(sockaddr_t *, SVCXPRT *);
 
 int display_sockaddr_port(struct display_buffer *dspbuf, sockaddr_t *addr,
 			  bool ignore_port);
@@ -222,4 +250,9 @@ struct READ4resok;
 struct xdr_uio *xdr_READ4res_uio_setup(struct READ4resok *objp);
 
 extern tirpc_pkg_params ntirpc_pp;
+
+sockaddr_t *convert_ipv6_to_ipv4(sockaddr_t *ipv6, sockaddr_t *ipv4);
+bool is_loopback(sockaddr_t *addr);
+
+
 #endif /* GSH_RPC_H */
