@@ -1,3 +1,4 @@
+/* SPDX-License-Identifier: LGPL-3.0-or-later */
 /*
  * definition des codes d'error
  *
@@ -203,10 +204,11 @@ void set_const_log_str(void);
 struct log_component_info {
 	const char *comp_name;	/* component name */
 	const char *comp_str;	/* shorter, more useful name */
-	bool comp_env_set;	/* level was set by env(), now RO */
 };
 
 extern log_levels_t *component_log_level;
+extern log_levels_t original_log_level;
+extern log_levels_t default_log_level;
 
 extern struct log_component_info LogComponents[COMPONENT_COUNT];
 
