@@ -1,3 +1,4 @@
+/* SPDX-License-Identifier: LGPL-3.0-or-later */
 /*
  *
  *
@@ -181,6 +182,7 @@ typedef struct exportlist_client_entry__ {
 				       EXPORT_OPTION_AUTH_UNIX)
 
 #define EXPORT_OPTION_EXPIRE_SET 0x00080000	/*< Inode expire was set */
+#define EXPORT_DEFAULT_CACHE_EXPIRY 60	/*< Default cache expiry */
 
 /* Protocol flags */
 #define EXPORT_OPTION_NFSV3 0x00100000	/*< NFSv3 operations are supported */
@@ -223,7 +225,7 @@ int init_export_root(struct gsh_export *exp);
 
 fsal_status_t nfs_export_get_root_entry(struct gsh_export *exp,
 					struct fsal_obj_handle **obj);
-void unexport(struct gsh_export *exp);
+void release_export(struct gsh_export *exp, bool config);
 /* XXX */
 /*void kill_export_root_entry(cache_entry_t *entry);*/
 /*void kill_export_junction_entry(cache_entry_t *entry);*/
@@ -232,8 +234,39 @@ int ReadExports(config_file_t in_config,
 		struct config_error_type *err_type);
 int reread_exports(config_file_t in_config,
 		struct config_error_type *err_type);
-void free_export_resources(struct gsh_export *exp);
+void free_export_resources(struct gsh_export *exp, bool config);
 
 void exports_pkginit(void);
 
+struct log_exports_parms {
+	log_levels_t level;
+	const char *file;
+	int line;
+	const char *func;
+	const char *tag;
+	bool clients;
+};
+
+bool log_an_export(struct gsh_export *exp, void *state);
+
+uint32_t export_check_options(struct gsh_export *exp);
+
+static inline bool export_can_be_mounted(struct gsh_export *export)
+{
+	uint32_t options = export_check_options(export);
+
+	return (options & EXPORT_OPTION_NFSV4) != 0
+	       && export->cfg_pseudopath != NULL
+	       && export->export_id != 0
+	       && export->cfg_pseudopath[1] != '\0';
+}
+
+#define LOG_EXPORT(level, tag, exp, clients)				   \
+	do {								   \
+		struct log_exports_parms lep = {level, __FILE__, __LINE__, \
+						__func__, tag, clients};   \
+									   \
+		(void) log_an_export(exp, &lep);			   \
+	} while (0)
+
 #endif				/* !NFS_EXPORTS_H */
