@@ -1,3 +1,4 @@
+/* SPDX-License-Identifier: LGPL-3.0-or-later */
 /*
  *
  *
@@ -120,7 +121,31 @@ static inline void nfs4_freeFH(nfs_fh4 *fh)
 
 static inline size_t nfs4_sizeof_handle(struct file_handle_v4 *hdl)
 {
-	return offsetof(struct file_handle_v4, fsopaque)+hdl->fs_len;
+	return offsetof(struct file_handle_v4, fsopaque) + hdl->fs_len;
+}
+
+static inline size_t nfs4_sizeof_handle_padding(struct file_handle_v4 *hdl)
+{
+
+	int hsize;
+	int aligned_hsize;
+
+	hsize = offsetof(struct file_handle_v4, fsopaque) + hdl->fs_len;
+	/* align on 4 byte block boundary */
+	aligned_hsize = roundup(hsize, 4);
+	if (aligned_hsize <= NFS4_FHSIZE)
+		hsize = aligned_hsize;
+	return hsize;
+}
+
+static inline bool valid_Fh4_Len(nfs_fh4 *fh, file_handle_v4_t *pfile_handle)
+{
+	if (!nfs_param.core_param.enable_v3_fh_for_v4) {
+		return fh->nfs_fh4_len == nfs4_sizeof_handle(pfile_handle);
+	} else {
+		return ((fh->nfs_fh4_len == nfs4_sizeof_handle(pfile_handle))
+	|| (fh->nfs_fh4_len == nfs4_sizeof_handle_padding(pfile_handle)));
+	}
 }
 
 #define LEN_FH_STR OPAQUE_BYTES_SIZE(NFS4_FHSIZE)
@@ -186,7 +211,7 @@ static inline int nlm4_FhandleToExportId(netobj *pfh3)
  *
  * @param pfh [IN] file handle to test.
  *
- * @return NFS4_OK if successfull, NFS4ERR_NOFILEHANDLE is fh is empty.
+ * @return NFS4_OK if successful, NFS4ERR_NOFILEHANDLE is fh is empty.
  *
  */
 static inline int nfs4_Is_Fh_Empty(nfs_fh4 *pfh)
@@ -310,4 +335,26 @@ nfsstat4 nfs4_sanity_check_saved_FH(compound_data_t *data, int required_type,
 		} \
 	} while (0)
 
+#define LogNFSACL_Operation(component, req, fh, format, args...) \
+	do { \
+		if (unlikely(component_log_level[component] >= NIV_DEBUG)) { \
+			char str[LEN_FH_STR]; \
+			struct display_buffer dspbuf = { \
+				sizeof(str), str, str}; \
+			\
+			display_opaque_bytes(&dspbuf, (fh)->data.data_val, \
+					     (fh)->data.data_len); \
+			\
+			DisplayLogComponentLevel( \
+				component,  __FILE__, __LINE__, __func__, \
+				NIV_DEBUG, \
+				"REQUEST PROCESSING: Calling %s " \
+				"File Handle V3: Len=%u %s" \
+				format, \
+			    nfsacl_func_desc[req->rq_msg.cb_proc].funcname, \
+				(fh)->data.data_len, str, \
+				## args); \
+		} \
+	} while (0)
+
 #endif				/* NFS_FILE_HANDLE_H */
