@@ -1,3 +1,4 @@
+/* SPDX-License-Identifier: LGPL-3.0-or-later */
 /*
  * vim:noexpandtab:shiftwidth=8:tabstop=8:
  *
@@ -86,7 +87,7 @@ int init_server_pkgs(void);
 void nfs_start(nfs_start_info_t *p_start_info);
 
 /**
- * check for useable malloc implementation
+ * check for usable malloc implementation
  */
 static inline void nfs_check_malloc(void)
 {
@@ -112,8 +113,17 @@ static inline void nfs_check_malloc(void)
 /* in nfs_worker_thread.c */
 
 enum xprt_stat nfs_rpc_valid_NFS(struct svc_req *);
+#ifdef _USE_NLM
 enum xprt_stat nfs_rpc_valid_NLM(struct svc_req *);
+#endif
+#ifdef _USE_NFS3
 enum xprt_stat nfs_rpc_valid_MNT(struct svc_req *);
+#endif
+#ifdef _USE_RQUOTA
 enum xprt_stat nfs_rpc_valid_RQUOTA(struct svc_req *);
+#endif
+#ifdef USE_NFSACL3
+enum xprt_stat nfs_rpc_valid_NFSACL(struct svc_req *);
+#endif
 
 #endif				/* !NFS_INIT_H */
