@@ -1,3 +1,4 @@
+/* SPDX-License-Identifier: LGPL-3.0-or-later */
 /*
  *
  *
@@ -33,7 +34,7 @@
 #ifndef NFS_LIB_H
 #define NFS_LIB_H
 
-extern char *config_path;
+extern char *nfs_config_path;
 
 extern int nfs_libmain(
 	const char *config_path,
